/*
  The Abduction of Oscar Z
  ========================

  "Oscar loved watching night skies long after his parents slept. But tonight
  that sky was watching him..."

  * PRODUCTION & CODE / Tulo / @tulinmola
  * ART & STORY / Alberto Hache / @Alberto_H9
  * MUSIC & FX / Gryzor87 / @Gryzor87

  A 2020 Dreamin`Bits production
  https://dreaminbits.com/

  MIT License

  Copyright (c) 2020 Dreamin`Bits

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the"Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#include "sequence.h"
#include "../renderer/renderer.h"
#include "../lang/math.h"
#include "../lang/bool.h"
#include "../system/audio.h"
#include "../system/interruptions.h"
#include "../system/uncompress.h"

#define DARK_PAGE_MASK        3
// NOTE: defined en sequence.h.s
#define MAX_STRIPS_PER_PAGE   2
#define BLACK_PATTERN         cpctm_px2byteM0(6, 6)

typedef struct {
  u8* screen;
  u8* tiles;
  u8 widthInTiles;
  u8 heightInTiles;
} Strip;
// NOTE: defined en sequence.h.s

Strip sequence_strips[MAX_STRIPS_PER_PAGE];
u8 sequence_stripCount;

Strip sequence_highlighted;

bool sequence_exit;

u8* sequence_play(u8* data) __z88dk_fastcall;
u8* sequence_drawText(u8* data) __z88dk_fastcall;
u8* sequence_waitUntil(u8* data) __z88dk_fastcall;
extern u8* sequence_addStrip(u8* data) __z88dk_fastcall;
extern u8* sequence_scroll(u8* data) __z88dk_fastcall;
u8* sequence_drawStrip(u8* data) __z88dk_fastcall;
u8* sequence_highlight(u8* data) __z88dk_fastcall;
u8* sequence_highlightNext(u8* data) __z88dk_fastcall;
u8* sequence_uncompress(u8* data) __z88dk_fastcall;
u8* sequence_uncompressIndirect(u8* data) __z88dk_fastcall;
u8* sequence_background(u8* data) __z88dk_fastcall;
u8* sequence_drawSlowly(u8* data) __z88dk_fastcall;

void sequence_doHighlight(u8* tiles, u8* screen, u8 widthInTiles, u8 heightInTiles);
void sequence_forcedExit();

void sequence_start(u8* data) {
  u8 command;

  sequence_exit = false;

  while ((command = (*data++)) && !sequence_exit) {
    switch (command) {
    case 1:   // SEQUENCE_INIT_LANDSCAPE
      renderer_init(RENDERER_LANDSCAPE_ORIENTATION);
      break;

    case 2:   // SEQUENCE_INIT_PORTRAIT
      renderer_init(RENDERER_PORTRAIT_ORIENTATION);
      break;

    case 3:   // SEQUENCE_FADE_IN
      renderer_fadeIn();
      break;

    case 4:   // SEQUENCE_FADE_OUT
      renderer_fadeOut();
      break;

    case 5:   // SEQUENCE_PLAY
      data = sequence_play(data);
      break;

    case 6:   // SEQUENCE_DRAW_TEXT
      data = sequence_drawText(data);
      break;

    case 7:   // SEQUENCE_WAIT_UNTIL
      data = sequence_waitUntil(data);
      break;

    case 8:   // SEQUENCE_RESET_TIMER
      interruptions_resetFrame();
      break;

    case 9:   // SEQUENCE_CLEAR
      renderer_clear();
      break;

    case 10:  // SEQUENCE_UNCOMPRESS_INDIRECT
      data = sequence_uncompressIndirect(data);
      break;

    case 11:  // SEQUENCE_DRAW_STRIP
      data = sequence_drawStrip(data);
      break;

    case 12:  // SEQUENCE_HIGHLIGHT
      data = sequence_highlight(data);
      break;

    case 13:  // SEQUENCE_HIGHLIGHT_NEXT
      data = sequence_highlightNext(data);
      break;

    case 14:  // SEQUENCE_INIT_PAGE
      sequence_stripCount = 0;
      break;

    case 15:  // SEQUENCE_ADD_STRIP
      data = sequence_addStrip(data);
      break;

    case 16:  // SEQUENCE_SCROLL
      data = sequence_scroll(data);
      break;

    case 17:  // SEQUENCE_UNCOMPRESS
      data = sequence_uncompress(data);
      break;

    case 18:  // SEQUENCE_STOP
      audio_stop();
      break;

    case 19:  // SEQUENCE_BACKGROUND
      data = sequence_background(data);
      break;

    case 20:  // SEQUENCE_DRAW_SLOWLY
      data = sequence_drawSlowly(data);
      break;

    case 21:  // SEQUENCE_WAIT_FOR_SONG
      audio_waitForSongEnd();
      break;

    case 22:  // SEQUENCE_INIT_DEFAULT
      renderer_init(RENDERER_DEFAULT_ORIENTATION);
      break;
    }
  }

  if (sequence_exit) {
    sequence_forcedExit();
  }
}

void sequence_checkExit() {
  if (cpct_isKeyPressed(Key_Esc)) {
    sequence_exit = true;
  }
}

u8* sequence_play(u8* data) __z88dk_fastcall {
  u16 song = *((u16*)data);

  audio_start(song);

  return data + 2;
}

u8* sequence_drawText(u8* data) __z88dk_fastcall {
  u8* screen;
  u8* sprite;
  u8 width, height;

  screen = (u8*)*((u16*)data);
  data += 2;
  sprite = (u8*)*((u16*)data);
  data += 2;
  width = *(sprite++);
  height = *(sprite++);

  renderer_drawMaskedSprite(sprite, screen, width, height);

  return data;
}

u8* sequence_waitUntil(u8* data) __z88dk_fastcall {
  u16 frames = *((u16*)data);

  interruptions_waitUntilFrame(frames);

  return data + 2;
}

u8* sequence_drawStrip(u8* data) __z88dk_fastcall {
  u8* screen;
  u8* tiles;
  u8 widthInTiles, heightInTiles;

  screen = (u8*)*((u16*)data);
  data += 2;
  tiles = (u8*)*((u16*)data);
  data += 2;
  widthInTiles = *(data++);
  heightInTiles = *(data++);

  renderer_drawTiles(tiles, screen, widthInTiles, heightInTiles);

  return data;
}

void sequence_doHighlight(u8* tiles, u8* screen, u8 widthInTiles, u8 heightInTiles) {
  for (u8 i = 4 - DARK_PAGE_MASK; i < 4; i++) {
    u8 mask = RENDERER_DARKEN_MASKS_HI_ADDRESS + 3 - i;
    renderer_drawMaskedTiles(tiles, screen, widthInTiles, heightInTiles, mask, 0);
  }
  renderer_drawTiles(tiles, screen, widthInTiles, heightInTiles);

  sequence_highlighted.screen = screen + widthInTiles * 2;
  sequence_highlighted.tiles = tiles + 16 * widthInTiles * heightInTiles;
  sequence_highlighted.heightInTiles = heightInTiles;
}

u8* sequence_highlight(u8* data) __z88dk_fastcall {
  u8* screen;
  u8* tiles;
  u8 widthInTiles, heightInTiles;

  screen = (u8*)*((u16*)data);
  data += 2;
  tiles = (u8*)*((u16*)data);
  data += 2;
  widthInTiles = *(data++);
  heightInTiles = *(data++);

  sequence_doHighlight(tiles, screen, widthInTiles, heightInTiles);

  return data;
}

u8* sequence_highlightNext(u8* data) __z88dk_fastcall {
  u8 widthInTiles = *data++;

  sequence_doHighlight(sequence_highlighted.tiles, sequence_highlighted.screen, widthInTiles, sequence_highlighted.heightInTiles);

  return data;
}

u8* sequence_uncompress(u8* data) __z88dk_fastcall {
  const void* source;
  void* destination;

  source = (const void*)*((u16*)data);
  data += 2;
  destination = (void*)*((u16*)data);
  data += 2;

  uncompress(source, destination);

  return data;
}

u8* sequence_uncompressIndirect(u8* data) __z88dk_fastcall {
  const void* source;
  u16 compressedSize;
  void* destination;

  source = (const void*)*((u16*)data);
  data += 2;
  compressedSize = *((u16*)data);
  data += 2;
  destination = (void*)*((u16*)data);
  data += 2;

  cpct_memcpy((void*)CPCT_VMEM_START, (const void*)source, compressedSize);
  uncompress(CPCT_VMEM_START, destination);

  return data;
}

u8* sequence_background(u8* data) __z88dk_fastcall {
  u8* screen;
  u8* tiles;
  u8 widthInTiles, heightInTiles;

  screen = (u8*)*((u16*)data);
  data += 2;
  tiles = (u8*)*((u16*)data);
  data += 2;
  widthInTiles = *(data++);
  heightInTiles = *(data++);

  for (u8 i = 0; i < DARK_PAGE_MASK + 1; i++) {
    u8 mask = RENDERER_DARKEN_MASKS_HI_ADDRESS + i;
    renderer_drawMaskedTiles(tiles, screen, widthInTiles, heightInTiles, mask, 0);
  }

  return data;
}

u8* sequence_drawSlowly(u8* data) __z88dk_fastcall {
  u8* screen;
  u8* sprite;
  u8 width, height;

  screen = (u8*)*((u16*)data);
  data += 2;
  sprite = (u8*)*((u16*)data);
  data += 2;
  width = *(sprite++);
  height = *(sprite++);

  for (u8 y = 1; y < height; y++) {
    renderer_drawMaskedSprite(sprite, screen, width, y);
    __asm__("halt");
    __asm__("halt");
    cpct_waitVSYNC();
    __asm__("halt");
    __asm__("halt");
    cpct_waitVSYNC();
  }

  return data;
}

void sequence_forcedExit() {
  renderer_fadeOut();
  renderer_init(RENDERER_DEFAULT_ORIENTATION);
  renderer_clear();
  audio_stop();
}
