;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

; typedef struct {
;   u8* screen;
;   u8* tiles;
;   u8 widthInTiles;
;   u8 heightInTiles;
; } Strip;
.equ STRIP_SCREEN, 0
.equ STRIP_SCREEN_LO, 0
.equ STRIP_SCREEN_HI, 1
.equ STRIP_TILES, 2
.equ STRIP_TILES_LO, 2
.equ STRIP_TILES_HI, 3
.equ STRIP_WIDTH_IN_TILES, 4
.equ STRIP_HEIGHT_IN_TILES, 5
.equ STRIP_SIZE, 6

.globl _sequence_strips
.globl _sequence_stripCount


.equ DARK_PAGE_MASK, 3

.equ SEQUENCE_LANDSCAPE_SCREEN, 0
.equ SEQUENCE_PORTRAIT_SCREEN, 0
.equ SEQUENCE_DEFAULT_SCREEN, 0

.equ SEQUENCE_FORMAT, SEQUENCE_LANDSCAPE_SCREEN

.macro SEQUENCE_SCREEN_POINTER xInBytes, y
  .if SEQUENCE_LANDSCAPE_SCREEN
    RENDERER_LANDSCAPE_SCREEN_POINTER(xInBytes, y)
  .endif
  .if SEQUENCE_PORTRAIT_SCREEN
    RENDERER_PORTRAIT_SCREEN_POINTER(xInBytes, y)
  .endif
  .if SEQUENCE_DEFAULT_SCREEN
    RENDERER_DEFAULT_SCREEN_POINTER(xInBytes, y)
  .endif
.endm

.macro SEQUENCE_END
  .db #0
.endm

.macro SEQUENCE_INIT_LANDSCAPE
  .db #1
  .equ SEQUENCE_LANDSCAPE_SCREEN, 1
  .equ SEQUENCE_PORTRAIT_SCREEN, 0
  .equ SEQUENCE_DEFAULT_SCREEN, 0
.endm

.macro SEQUENCE_INIT_PORTRAIT
  .db #2
  .equ SEQUENCE_LANDSCAPE_SCREEN, 0
  .equ SEQUENCE_PORTRAIT_SCREEN, 1
  .equ SEQUENCE_DEFAULT_SCREEN, 0
.endm

.macro SEQUENCE_INIT_DEFAULT
  .db #22
  .equ SEQUENCE_LANDSCAPE_SCREEN, 0
  .equ SEQUENCE_PORTRAIT_SCREEN, 0
  .equ SEQUENCE_DEFAULT_SCREEN, 1
.endm

.macro SEQUENCE_FADE_IN
  .db #3
.endm

.macro SEQUENCE_FADE_OUT
  .db #4
.endm

.macro SEQUENCE_PLAY asset
  .db #5
  .dw #'asset'_ADDRESS
.endm

.macro SEQUENCE_DRAW_SPRITE xInBytes, y, asset
  .db #6
  SEQUENCE_SCREEN_POINTER(xInBytes, y)
  .dw #'asset'_ADDRESS
.endm

.macro SEQUENCE_DRAW_TEXT xInBytes, y, asset
  SEQUENCE_DRAW_SPRITE xInBytes, y, asset
.endm

.macro SEQUENCE_WAIT_UNTIL second, decimal
  .db #7
  .dw second * #50 + decimal * #5
.endm

.macro SEQUENCE_RESET_TIMER
  .db #8
.endm

.macro SEQUENCE_CLEAR
  .db #9
.endm

.macro SEQUENCE_UNCOMPRESS_INDIRECT source, info
  .db #10
  .dw #source
  .dw #'info'_COMPRESSED_SIZE
  .dw #'info'_ADDRESS
.endm

.macro SEQUENCE_DRAW_STRIP xInBytes, y, asset, startInTiles, widthInTiles
  .db #11

  SEQUENCE_SCREEN_POINTER(xInBytes, y)
  .dw #('asset'_ADDRESS + startInTiles * 16 * 'asset'_HEIGHT_IN_TILES)
  .db #widthInTiles, #'asset'_HEIGHT_IN_TILES
.endm

.macro SEQUENCE_HIGHLIGHT xInBytes, y, asset, widthInTiles
  .db #12
  SEQUENCE_SCREEN_POINTER(xInBytes, y)
  .dw #'asset'_ADDRESS
  .db #widthInTiles, #'asset'_HEIGHT_IN_TILES
.endm

.macro SEQUENCE_HIGHLIGHT_NEXT widthInTiles
  .db #13
  .db #widthInTiles
.endm

.macro SEQUENCE_INIT_PAGE
  .db #14
.endm

.macro SEQUENCE_ADD_STRIP xInBytes, y, asset
  .db #15
  .db #(xInBytes >> 1)
  .db #'asset'_WIDTH_IN_TILES, #'asset'_HEIGHT_IN_TILES
  SEQUENCE_SCREEN_POINTER(xInBytes, y)
  .dw #'asset'_ADDRESS
.endm

.macro SEQUENCE_SCROLL widthInTiles
  .db #16
  .db #widthInTiles
.endm

.macro SEQUENCE_UNCOMPRESS source, info
  .db #17
  .dw #source
  .dw #'info'_ADDRESS
.endm

.macro SEQUENCE_STOP
  .db #18
.endm

.macro SEQUENCE_BACKGROUND xInBytes, y, asset, widthInTiles
  .db #19
  SEQUENCE_SCREEN_POINTER(xInBytes, y)
  .dw #'asset'_ADDRESS
  .db #widthInTiles, #'asset'_HEIGHT_IN_TILES
.endm

.macro SEQUENCE_DRAW_SLOWLY xInBytes, y, asset
  .db #20
  SEQUENCE_SCREEN_POINTER(xInBytes, y)
  .dw #'asset'_ADDRESS
.endm

.macro SEQUENCE_WAIT_FOR_SONG
  .db #21
.endm
