;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "sequence.h.s"
.include "renderer/renderer.h.s"
.include "lang/macros.h.s"

.module sequence

.globl __mulint

; u8* sequence_addStrip(u8* data) {
;   Strip* strip = &sequence_strips[sequence_stripCount++];
;   u8* screen;
;   u8* tiles;
;   u8 xInTiles, widthInTiles, heightInTiles;
;   i8 widthInTilesToDraw;
;
;   screen = (u8*)*((u16*)data);
;   data += 2;
;   tiles = (u8*)*((u16*)data);
;   data += 2;
;
;   xInTiles = *data++;
;   widthInTiles = *data++;
;
;   widthInTilesToDraw = MATH_MIN(
;     RENDERER_LANDSCAPE_WIDTH_IN_TILES - xInTiles,
;     widthInTiles
;   );
;
;   heightInTiles = *data++;
;
;   renderer_drawMaskedTiles(tiles, screen, widthInTilesToDraw, heightInTiles, RENDERER_DARKEN_MASKS_HI_ADDRESS + DARK_PAGE_MASK, 0);
;
;   strip->screen = screen + widthInTilesToDraw * 2;
;   strip->tiles = tiles + 16 * widthInTilesToDraw * heightInTiles;
;   strip->widthInTiles = MATH_MAX(widthInTiles - widthInTilesToDraw, 0);
;   strip->heightInTiles = heightInTiles;
;
;   return data;
; }

; IN:
;   HL <- u8* data
_sequence_addStrip::
  push hl
    ld bc, #_sequence_strips
    ld hl, #_sequence_stripCount
    ld e, (hl)
    ld d, #0        ; DE <- sequence_stripCount
    inc (hl)        ; sequence_stripCount++

    ld l, e
    ld h, d
    add hl, hl
    add hl, de
    add hl, hl
    add hl, bc      ; HL <- strip address

    ex de, hl
    ld__iyl_e
    ld__iyh_d       ; IY <- strip address
  pop hl

  ld bc, #RENDERER_DARKEN_MASKS_HI_ADDRESS + DARK_PAGE_MASK
  push bc           ; PARAMS: RENDERER_DARKEN_MASKS_HI_ADDRESS + DARK_PAGE_MASK, 0

  ld b, (hl)        ; B <- xInTiles
  inc hl
  ld c, (hl)        ; C <- widthInTiles
  ld d, c           ; D <- widthInTiles for later
  inc hl
  ld a, #RENDERER_LANDSCAPE_WIDTH_IN_TILES
  sub b             ; A <- RENDERER_LANDSCAPE_WIDTH_IN_TILES - xInTiles
  dec a
  cp c
  jr nc, continue_with_height_in_tiles
  ld c, a
continue_with_height_in_tiles:
                    ; C <- widthInTilesToDraw = MATH_MIN(
                    ;     RENDERER_LANDSCAPE_WIDTH_IN_TILES - xInTiles,
                    ;     widthInTiles
                    ;   )
  ld b, (hl)        ; B <- heightInTiles
  inc hl
  ; strip->heightInTiles = heightInTiles;
  ld STRIP_HEIGHT_IN_TILES (iy), b
  push bc           ; PARAMS: widthInTilesToDraw, heightInTiles

  ; strip->widthInTiles = MATH_MAX(widthInTiles - widthInTilesToDraw, 0);
  ld a, d
  sub c
  jr nc, update_strip_width_in_tiles
  xor a
update_strip_width_in_tiles:
  ld STRIP_WIDTH_IN_TILES (iy), a

  ld e, (hl)
  inc hl
  ld d, (hl)        ; DE <- screen
  inc hl
  push de           ; PARAM: screen

  push hl
    ; strip->screen = screen + widthInTilesToDraw * 2;
    ld l, c
    ld h, #0
    add hl, hl      ; HL <- widthInTilesToDraw * 2
    add hl, de      ; HL <- screen + widthInTilesToDraw * 2
    ld STRIP_SCREEN_LO (iy), l
    ld STRIP_SCREEN_HI (iy), h
  pop hl

  ld e, (hl)
  inc hl
  ld d, (hl)        ; DE <- tiles
  inc hl
  push de           ; PARAM: tiles
  ld (#RESULT), hl  ; store result

  ; strip->tiles = tiles + 16 * widthInTilesToDraw * heightInTiles;
  push de
    ld l, c
    ld h, #0        ; HL <- widthInTilesToDraw
    add hl, hl      ; HL <- widthInTilesToDraw * 2
    add hl, hl      ; HL <- widthInTilesToDraw * 4
    add hl, hl      ; HL <- widthInTilesToDraw * 8
    add hl, hl      ; HL <- widthInTilesToDraw * 16
    push hl         ; 16 * widthInTilesToDraw
    ld l, b
    ld h, #0        ; HL <- heightInTiles
    push hl
      call __mulint ; HL <- 16 * widthInTilesToDraw * heightInTiles
    pop af
    pop af
  pop de
  add hl, de        ; HL <- tiles + 16 * widthInTilesToDraw * heightInTiles
  ld STRIP_TILES_LO (iy), l
  ld STRIP_TILES_HI (iy), h

  ; renderer_drawMaskedTiles(tiles, screen, widthInTilesToDraw, heightInTiles, RENDERER_DARKEN_MASKS_HI_ADDRESS + DARK_PAGE_MASK, 0)
  call _renderer_drawMaskedTiles

.equ RESULT, . + 1
  ld hl, #0
  ret
