;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "../lang/macros.h.s"
.include "renderer.h.s"

.module renderer

.equ RENDERER_TRANSPARENT_MASK_HI_ADDRESS, >RENDERER_TRANSPARENT_MASK_ADDRESS

_renderer_drawMaskedSprite::
  pop af        ;; AF <- return address
  pop bc        ;; BC <- sprite address
  pop de        ;; DE <- screen address
  pop hl        ;; H <- height, L <- width
  push af       ;; Put returning address in the stack again

  push ix
  ld a, l
  ld__ixl_a     ;; IXL <- width
  ld a, h
  ld__ixh_a     ;; IXH <- height

  ld h, #RENDERER_TRANSPARENT_MASK_HI_ADDRESS   ;; Hx <- mask address

  ;; Save Width in a placeholder for easy recovering it later
  ld__a_ixl               ;; [2] A = IXL = width of the sprite
  ld (restore_ixl + 2), a ;; [4] Save IXL (widht of the sprite) in a placeholder for recovering it later

dms_sprite_height_loop:
  push de         ;; [4] Save DE for later use (jump to next screen line)

dms_sprite_width_loop:
  ld    a, (bc)   ;; [2] Get next byte from the sprite
  ld    l, a      ;; [1] Access mask table element (table must be 256-byte aligned)
  ld    a, (de)   ;; [2] Get the value of the byte of the screen where we are going to draw
  and (hl)        ;; [2] Erase background part that is to be overwritten (Mask step 1)
  or    l         ;; [1] Add up background and sprite information in one byte (Mask step 2)
  ld  (de), a     ;; [2] Save modified background + sprite data information into memory
  inc  de         ;; [2] Next bytes (sprite and memory)
  inc  bc         ;; [2] Next byte from the sprite (must be 256-bytes aligned)

  dec__ixl        ;; [2] IXL holds sprite width, we decrease it to count pixels in this line.
  jr   nz,dms_sprite_width_loop;; [2/3] While not 0, we are still painting this sprite line 
                              ;;      - When 0, we have to jump to next pixel line

  pop  de         ;; [3] Recover DE from stack. We use it to calculate start of next pixel line on screen
                  ;;  ... It must be restored before exiting the function, or it will be taken as return address

  dec__ixh        ;; [2] IXH holds sprite height. We decrease it to count another pixel line finished
  jr    z,dms_sprite_copy_ended;; [2/3] If 0, we have finished the last sprite line.
                              ;;      - If not 0, we have to move pointers to the next pixel line
restore_ixl:
  ld__ixl #00     ;; [3] IXL = Restore IXL to the width of the sprite (00 is a placeholder)

  ld    a, d      ;; [1] Start of next pixel line normally is 0x0800 bytes away.
  add   #0x08     ;; [2]    so we add it to DE (just by adding 0x08 to D)
  ld    d, a      ;; [1]
  and   #0x38     ;; [2] We check if we have crossed memory boundary (every 8 pixel lines)
  jr   nz, dms_sprite_height_loop ;; [2/3]  by checking the 4 bits that identify present memory line. 
                                  ;; ....  If 0, we have crossed boundaries

dms_sprite_8bit_boundary_crossed:
  ld    a, e      ;; [1] DE = DE + 0xC0XXh
_renderer_drawMaskedSpriteScreenWidthInBytes::
;; screen with in bytes is modified by renderer_init
  add   #0x50     ;; [2] -- Relocate DE pointer to the start of the next pixel line:
  ld    e, a      ;; [1] -- DE is moved forward 3 memory banks plus 50 bytes (4000h * 3) 
  ld    a, d      ;; [1] -- which effectively is the same as moving it 1 bank backwards and then
  adc   #0xC0     ;; [2] -- 50 bytes forwards (which is what we want to move it to the next pixel line)
  ld    d, a      ;; [1] -- Calculations are made with 8 bit maths as it is faster than other alternatives here

  jr  dms_sprite_height_loop ;; [3] Jump to continue with next pixel line

dms_sprite_copy_ended:
  pop ix
  ret
