;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.module crtc

;;
;; CRTC configuration
;;
;; References:
;; - CRTC documentation at http://www.cpcwiki.eu/index.php/CRTC

.equ DEFAULT_WIDTH_IN_CHARS, #40
.equ DEFAULT_HSYNC_START_IN_CHARS, #46
.equ DEFAULT_HEIGHT_IN_CHARS, #25
.equ DEFAULT_VSYNC_START_IN_CHARS, #30

.equ LANDSCAPE_WIDTH_IN_CHARS, #50
.equ LANDSCAPE_HSYNC_START_IN_CHARS, #51
.equ LANDSCAPE_HEIGHT_IN_CHARS, #20
.equ LANDSCAPE_VSYNC_START_IN_CHARS, #28

.equ PORTRAIT_WIDTH_IN_CHARS, #29
.equ PORTRAIT_HSYNC_START_IN_CHARS, #38
.equ PORTRAIT_HEIGHT_IN_CHARS, #35
.equ PORTRAIT_VSYNC_START_IN_CHARS, #35

.equ R3_HALF_CHAR, #86
.equ R3_COMPLETE_CHAR, #85

crtc_default:
.db DEFAULT_WIDTH_IN_CHARS, DEFAULT_HSYNC_START_IN_CHARS
.db DEFAULT_HEIGHT_IN_CHARS, DEFAULT_VSYNC_START_IN_CHARS

crtc_landscape:
.db LANDSCAPE_WIDTH_IN_CHARS, LANDSCAPE_HSYNC_START_IN_CHARS
.db LANDSCAPE_HEIGHT_IN_CHARS, LANDSCAPE_VSYNC_START_IN_CHARS

crtc_portrait:
.db PORTRAIT_WIDTH_IN_CHARS, PORTRAIT_HSYNC_START_IN_CHARS
.db PORTRAIT_HEIGHT_IN_CHARS, PORTRAIT_VSYNC_START_IN_CHARS

; HL <- configuration address
crtc_set:
  ld bc, #0xbc01
  out (c), c
  inc b
  ld c, (hl)
  inc hl
  out (c), c

  ; R2: set horizontal sync position
  ld  bc, #0xbc02
  out (c), c
  inc b
  ld c, (hl)
  inc hl
  out (c), c

  ; R6: set vertical displayed characters
  ld  bc, #0xbc06
  out (c), c
  inc b
  ld c, (hl)
  inc hl
  out (c), c

  ; R7: set vertical sync position
  ld  bc, #0xbc07
  out (c), c
  inc b
  ld c, (hl)
  out (c), c

  ret

_crtc_setLandscape::
  ld hl, #crtc_landscape
  jr crtc_set

_crtc_setPortrait::
  ld hl, #crtc_portrait
  jr crtc_set

_crtc_setDefault::
  ld hl, #crtc_default
  jr crtc_set

_crtc_setR30::
  ld  bc, #0xbc03
  out (c), c
  ld  bc, #0xbd00 + R3_HALF_CHAR
  out (c), c
  ret

_crtc_setR31::
  ld  bc, #0xbc03
  out (c), c
  ld  bc, #0xbd00 + R3_COMPLETE_CHAR
  out (c), c
  ret
