;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "audio.h.s"
.include "akg.h.s"
.include "data/game_assets.h.s"
.include "data/songs.h.s"
.include "lang/debug.h.s"

.module audio

_audio_flags::
  .db #0

_audio_init::
  ; reset audio flag
  xor a
  ld (#_audio_flags), a
  ; init SFX
  ld hl, #SFX_ADDRESS
  jp _PLY_AKG_INITSOUNDEFFECTS

_audio_playNothing::
  ld l, #EMPTY_SONG
  ; jr _audio_playSong

; IN:
;   L <- subsong index
; DESTROYS:
;   AF, AF’, BC, DE, HL, IY
_audio_playSong::
  ld a, l
  ld hl, #SONGS_ADDRESS
  push ix
    call _PLY_AKG_INIT
  pop ix
  ; set audio playing flag
  ld a, #AUDIO_PLAYING
  ld (#_audio_flags), a
  ret


; DESTROYS:
;   More than everything
_audio_play::
  jp _PLY_AKG_PLAY


; IN:
;   HL <- sfx address
_audio_playSfx::
  ex de, hl
    ld hl, (#_PLY_AKG_CHANNEL2_SOUNDEFFECTDATA)
    ld a, l
    or h          ; Z <- is no sfx is playing
  ex de, hl

  ld a, (hl)      ; A <- priority
  inc hl          ; (Z flag not affected)

  jr z, do_play_sfx

  ; check playing priority if sfx is playing
.equ current_priority, . + 1
  cp #0
  ret c

do_play_sfx:
  ld (#current_priority), a
                  ; save current priority
  ld e, (hl)      ; E <- sfx_num/instrument
  inc hl
  ld a, #15
  sub (hl)
  ld b, a         ; A <- inverted volume
  ld c, #SFX_CHANNEL
                  ; C <- channel
  ld a, e         ; A <- sfx number
  jp _PLY_AKG_PLAYSOUNDEFFECT


; OUT:
;   A <- current audio event
_audio_getEvent_asm::
  ld a, (#_PLY_AKG_EVENT)
  ret
