;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "controls.h.s"

.module controls

.globl cpct_memset_asm
.globl cpct_isKeyPressed_asm
.globl _cpct_keyboardStatusBuffer

.equ CONTROLS_SIZE, CONTROLS_COUNT * 2


_controls_keys::
  .dw #Key_CursorLeft, #Key_CursorRight, #Key_CursorUp, #Key_CursorDown
  .dw #Key_Space, #Key_Esc


_controls_reset::
  ld a, #0xff
  ld de, #_controls_keys
  ld bc, #CONTROLS_SIZE
  jp cpct_memset_asm


_controls_isExitPressed::
  ld l, #CONTROLS_P1_EXIT
  call _controls_isPressed
  ld l, a
  ret


; IN:
;   L <- action
; OUT:
;   A, L <- 0 if false
; DESTROYS:
;   AF, BC, D, HL
_controls_isPressed::
  ld bc, #_controls_keys
  ld h, #0
  add hl, hl
  add hl, bc    ; HL <- controls_keys[action] address

  ld a, (hl)
  inc hl
  ld h, (hl)
  ld l, a       ; HL <- key = contros_keys[action]

  jp cpct_isKeyPressed_asm


_controls_getKeyPressed::
  ld de, #_cpct_keyboardStatusBuffer
  ld l, #0        ; L <- index
  ld b, #10       ; B <- counter

get_key_loop:
  ld a, (de)
  xor #0xff       ; A <- keyPressed
  jr nz, return_key

  inc l
  inc de
  djnz get_key_loop

  ld hl, #0       ; return 0
  ret

return_key:
  ld h, a         ; HL <- (keyPressed << 8) + index
  ret


; IN:
;   HL <- key id
_controls_isUsed::
  ld de, #_controls_keys
  ld b, #CONTROLS_COUNT

find_loop:
  ld a, (de)
  inc de
  cp l
  jr nz, continue_finding

  ld a, (de)
  cp h
  jr nz, continue_finding

  ld l, #1        ; found!
  ret

continue_finding:
  inc de
  djnz find_loop
  ld l, #0
  ret


; IN:
;   C <- index
;   DE <- key id
; DESTROYS:
;   BC, HL
_controls_set_asm::
  ld hl, #_controls_keys
  ld b, #0
  sla c
  add hl, bc
  ld (hl), e
  inc hl
  ld (hl), d
  ret
