;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "audio.h.s"
.include "lang/macros.h.s"
.include "lang/debug.h.s"
.include "cpc/audio.h.s"
.include "systems/renderer.h.s"
.include "data/game_assets.h.s"

.module interrupts

.equ PLAY_MUSIC, 1

.globl cpct_setPalette_asm
.globl cpct_scanKeyboard_if_asm

.equ JP_OPCODE, #0xc3
.equ PPI_PORT_B, #0xf5
.equ HANDLER_ADDRESS, #0x0039

.equ HSYNC_START_IN_CHARS, #0x34

.equ HUD_VSYNC, #0x3f ; 63
.equ HUD_HEIGHT_IN_CHARS_DISPLAYED, #RENDERER_HUD_HEIGHT_IN_CHARS
.equ HUD_WIDTH_IN_CHARS, #RENDERER_HUD_WIDTH_IN_CHARS
.equ HUD_VERTICAL_ADJUST, #0x0
.equ HUD_MAX_SCANLINE, #0x01
.equ HUD_PAGE, RENDERER_HUD_PAGE
.equ HUD_OFFSET, #0x0
.equ HUD_HEIGHT_IN_CHARS_TOTAL, #0x23 ; 35

.equ VSYNC_POSITION_OFF, #0x7f
.equ VSYNC_POSITION, #0x15

.equ GAME_VERTICAL_ADJUST, #0x04
.equ GAME_PAGE, RENDERER_GAME_PAGE
.equ GAME_OFFSET, #0
.equ GAME_VSYNC, #0x18 ; 24
.equ GAME_WIDTH_IN_CHARS, #RENDERER_GAME_WIDTH_IN_CHARS
.equ GAME_MAX_SCANLINE, #0x07

.equ PALETTES_HI_ADDRESS, >PALETTES_ADDRESS
.equ PALETTES_LO_ADDRESS, <PALETTES_ADDRESS

.equ BORDER_INK, #0x10
.equ BACKGROUND_INK, #0

.equ THIRD_INTERRUPT_BAND_POSITION, #0
.equ FOURTH_INTERRUPT_BAND_POSITION, #34
.equ FIFTH_INTERRUPT_BAND_POSITION, #89
.equ SIXTH_INTERRUPT_BAND_POSITION, #139
.equ FIRST_INTERRUPT_BAND_POSITION, #255

.if DEBUG_SHOW_INTERRUPTS
  .globl cpct_setPALColour_asm
.endif

.macro set_crtc_reg reg, value
  .if value
    ld bc, #0xbc00 + reg
    out (c), c
    ld bc, #0xbd00 + value
    out (c), c
  .else
    ld bc, #0xbc00 + reg
    out (c), c
    inc b
    out__c_0
  .endif
.endm

.macro show_interrupt hw_ink
  .if DEBUG_SHOW_INTERRUPTS
    push af
    push bc
    push hl

    ld l, #16
    ld h, #hw_ink
    call cpct_setPALColour_asm

    ld l, #0
    call cpct_setPALColour_asm

    pop hl
    pop bc
    pop af
  .endif
.endm

.macro set_next_interrupt next_interrupt
  ld bc, #next_interrupt
  ld (HANDLER_ADDRESS), bc
.endm

.macro wait_cycles cycles, ?wait_loop
  .equ loops, (cycles - 1) / 4
  .equ loopsx4, loops * 4
  .equ nops, cycles - loopsx4 - 1

  ld	b, #loops
wait_loop:
	djnz wait_loop

  .rept nops
  nop
  .endm
.endm

_interrupts_setRendererBand::
  ld a, b
.equ RENDERER_BAND_ADDRESS, . + 1
  ld bc, #_renderer_topBand
  ld (bc), a
  ret

.macro set_border color_address
  ld bc, #0x7f00 + BORDER_INK
.if DEBUG_PREVENT_SETTING_BORDERS
.else
  out (c), c
.endif
.equ color_address, . + 1
  ld a, #0x14
  add #0x40
.if DEBUG_PREVENT_SETTING_BORDERS
.else
  out (c), a
.endif
.endm

_interrupts_init::
  ; Disable firmware
  di
    im 1
    ld hl, #0xc9fb  ;; Instructions EI, RET
    ld (HANDLER_ADDRESS - 1), hl
  ei

  ; Wait for some vsyncs to ensure the beginning of the
  ; first interruption comes in the correct time
  ld e, #2
  ld b, #PPI_PORT_B
wait_vsync_end:
  in a, (c)
  rra
  jr c, wait_vsync_end
wait_vsync_start:
  in a, (c)
  rra
  jr nc, wait_vsync_start
  dec e
  jp nz, wait_vsync_end

  di
    set_crtc_reg #2, HSYNC_START_IN_CHARS

    ; Set our interrupt handler
    ld a, #JP_OPCODE
    ld (HANDLER_ADDRESS - 1), a
    ld hl, #first_interrupt
    ld (HANDLER_ADDRESS), hl
  ei
  ret

first_interrupt::
  push af
  push bc

  ; Ensure horizontal byte displacement is zero
  set_crtc_reg #3, #0x86

  show_interrupt #0x1c

  ; Update interrupt address
  set_next_interrupt second_interrupt

  ; Set palette?
.equ PALETTE_INCREMENT_ADDRESS, . + 1
  ld a, #0
  or a
  jr z, continue_first_interrupt

  ld b, a     ; B <- palette increment

.equ PALETTE_INDEX_ADDRESS, . + 1
  ld a, #0    ; A <- palette index
.equ PALETTE_LIMIT_ADDRESS, . + 1
  cp #128     ; Check limit reached
  jr nz, set_palette

  xor a       ; End setting palettes
  ld (#PALETTE_INCREMENT_ADDRESS), a
  jr continue_first_interrupt

set_palette:
  push hl
  push de
    ld h, #PALETTES_HI_ADDRESS
              ; Hx <- palette address
    ld l, a   ; HL <- current palette address

    add b
    ld (PALETTE_INDEX_ADDRESS), a

    ld de, #16
    push hl
      call cpct_setPalette_asm
    pop hl

    ld b, l   ; B <- current palette lo address
    ; Set top border color from current palette
    ld a, (#_renderer_topBorderColor)
    add b
    ld l, a
    ld a, (hl)
    ld (#interrupts_topBorder), a

    ; Set bottom border color from current palette
    ld a, (#_renderer_bottomBorderColor)
    add b
    ld l, a
    ld a, (hl)
    ld (#interrupts_bottomBorder), a
  pop de
  pop hl

continue_first_interrupt:
  ; Set top border color
  set_border interrupts_topBorder

  ld b, #FIRST_INTERRUPT_BAND_POSITION
  call _interrupts_setRendererBand

  pop bc
  pop af
  ei
  ret

second_interrupt::
  push af
  push bc

  ; Update interrupt address
  set_next_interrupt third_interrupt

  ; Set HUD screen params
  set_crtc_reg #7, HUD_VSYNC
  set_crtc_reg #6, HUD_HEIGHT_IN_CHARS_DISPLAYED
  set_crtc_reg #1, HUD_WIDTH_IN_CHARS
  set_crtc_reg #5, HUD_VERTICAL_ADJUST

  ; wait_cycles 59
  wait_cycles 43

  set_crtc_reg #9, HUD_MAX_SCANLINE
  set_crtc_reg #12, HUD_PAGE
  set_crtc_reg #13, HUD_OFFSET
  set_crtc_reg #4, HUD_HEIGHT_IN_CHARS_TOTAL

  show_interrupt #0x16

  ; Play music
  .if PLAY_MUSIC
    ld a, (#_audio_flags)
    or a
    jr z, end_playing

    push de
    push hl
    push ix
    push iy
      call _audio_getEvent_asm
      or a
      jr z, play_music

      call _audio_playNothing
play_music:
      exx
        ex af, af'
        push af
        push bc
        push de
        push hl
          call _audio_play
        pop hl
        pop de
        pop bc
        pop af
        ex af, af'
      exx
    pop iy
    pop ix
    pop hl
    pop de

end_playing:
  .endif

  pop bc
  pop af
  ei
  ret

third_interrupt::
  push af
  push bc

  ; Set game screen params
  set_crtc_reg #4, VSYNC_POSITION

  set_crtc_reg #5, GAME_VERTICAL_ADJUST
  ; set_crtc_reg #3, #0x86 | 0x85
  ld bc, #0xbc00 + #3
  out (c), c
.equ CRTC_R3_ADDRESS, . + 1
  ld bc, #0xbd00 + #0x86
  out (c), c
  ; set_crtc_reg #12, GAME_PAGE
  ld bc, #0xbc00 + #12
  out (c), c
.equ SCREEN_PAGE_ADDRESS, . + 1
  ld bc, #0xbd00 + GAME_PAGE
  out (c), c
  ; set_crtc_reg #13, GAME_OFFSET
  ld bc, #0xbc00 + #13
  out (c), c
.equ SCREEN_OFFSET_ADDRESS, . + 1
  ld bc, #0xbd00 + GAME_OFFSET
  out (c), c

  ; wait_cycles 10 or 18

  set_crtc_reg #7, GAME_VSYNC
  set_crtc_reg #1, GAME_WIDTH_IN_CHARS
  set_crtc_reg #9, GAME_MAX_SCANLINE

  show_interrupt #0x04

  push hl
    ; Are we in rendering frame?
.equ INTERRUPTS_FRAME, . + 1
    ld a, #0
    rra
    jr c, in_renderer_frame

    ; Interrupts update renderer TOP band
    ld bc, #_renderer_topBand
    jr after_updating_scroll

in_renderer_frame:
    ; Interrupts update renderer BOTTOM band
    ld bc, #_renderer_bottomBand

    ; Set render semaphore to allow starting to render
    ld a, #1  ; A <- 1, also used below
    ld (#_renderer_semaphore), a

    ; Update CRTC screen pointer
    ld hl, (#_renderer_left)
; interrupts_setCrtcR3(renderer_left & 1);
    and l     ; A <- L & 0x01 (A was 1 from above)
    add #0x85
    ld (#CRTC_R3_ADDRESS), a
; u8 offset = renderer_left >> 1;
    srl h
    rr l
    ld a, l
    ld (#SCREEN_OFFSET_ADDRESS), a
; u8 page = RENDERER_GAME_PAGE + ((renderer_left >> 9) & 0x07);
    ld a, h
    and #0x07
    add #RENDERER_GAME_PAGE
    ld (#SCREEN_PAGE_ADDRESS), a

after_updating_scroll:
    ld (#RENDERER_BAND_ADDRESS), bc
  pop hl

  ; Set bottom band to 0 (beginning of screen)
  ld b, #THIRD_INTERRUPT_BAND_POSITION
  call _interrupts_setRendererBand

  ; Update interrupt address
  set_next_interrupt fourth_interrupt

  pop bc
  pop af
  ei
  ret

fourth_interrupt::
  push af
  push bc

  ; Disable vertical sync
  set_crtc_reg #4, VSYNC_POSITION_OFF

  show_interrupt #0x0c

  ld b, #FOURTH_INTERRUPT_BAND_POSITION
  call _interrupts_setRendererBand

  ; Update interrupt address
  set_next_interrupt fifth_interrupt

  pop bc
  pop af
  ei
  ret

fifth_interrupt::
  push af
  push bc

  show_interrupt #0x12

  ; Scan keyboard
  ; TODO: think if this is the best moment for scanning keyboard
  push de
  push hl
    call cpct_scanKeyboard_if_asm
  pop hl
  pop de

  ld b, #FIFTH_INTERRUPT_BAND_POSITION
  call _interrupts_setRendererBand

  ; Update interrupt address
  set_next_interrupt sixth_interrupt

  pop bc
  pop af
  ei
  ret

sixth_interrupt::
  push af
  push bc

  show_interrupt #0x15

  ; Update interrupt address
  set_next_interrupt first_interrupt

  ld b, #SIXTH_INTERRUPT_BAND_POSITION
  call _interrupts_setRendererBand

  ; Set bottom border
  set_border interrupts_bottomBorder

  ; Next interrupt frame
  ld a, (#INTERRUPTS_FRAME)
  inc a
  ld (#INTERRUPTS_FRAME), a

  pop bc
  pop af
  ei
  ret

; DESTROYS:
;   AF
_interrupts_startFadeIn::
.if PALETTES_LO_ADDRESS
  ld a, #PALETTES_LO_ADDRESS
.else
  xor a
.endif
  ld (PALETTE_INDEX_ADDRESS), a
  ld a, #16
  ld (PALETTE_INCREMENT_ADDRESS), a
  ld a, #PALETTES_LO_ADDRESS + #128
  ld (PALETTE_LIMIT_ADDRESS), a
  ret

; DESTROYS:
;   AF
_interrupts_startFadeOut::
  ld a, #PALETTES_LO_ADDRESS + #112
  ld (PALETTE_INDEX_ADDRESS), a
  ld a, #-16
  ld (PALETTE_INCREMENT_ADDRESS), a
.if PALETTES_LO_ADDRESS
  ld a, #PALETTES_LO_ADDRESS + #-16
.endif
  ld (PALETTE_LIMIT_ADDRESS), a
  ret
