;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "entity.h.s"
.include "factories.h.s"
.include "systems/entities.h.s"
.include "systems/ai.h.s"
.include "systems/renderer.h.s"
.include "systems/alien.h.s"
.include "systems/failure.h.s"
.include "data/game_assets.h.s"
.include "lang/fixed.h.s"
.include "lang/macros.h.s"

.module alien

.equ ALIEN_Y, #30

_alien_def::
  FACTORY_DEF COMPONENT_ALL, ALIEN_PILOTING_LEFT, alien
_ko_def::
  FACTORY_DEF COMPONENT_ALL, ALIEN_KOING, bullet
_ufo_def::
  FACTORY_DEF COMPONENT_ALL, UFO_LANDED, ufo

; IN:
;   -
; OUT:
;   HL <- entity address
_alien_create::
push ix             ; TODO this should be done if needed in alien system (when in asm)

  ; get current level alien behavior
  ld hl, (#_alien_levelBehavior)
  ex de, hl
  ; updating its definition
  ld hl, #_alien_def + FACTORY_DEF_BEHAVIOR
  ld (hl), e
  inc hl
  ld (hl), d

  ; create alien entity in most-right position
  ld de, (#_renderer_right)
                    ; DE <- xInBytes
  ld b, #ALIEN_Y    ; B <- y
  ld hl, #_alien_def
  call _entities_create_asm

  ld ENTITY_COLLIDABLE_TYPE (ix), #COLLIDABLE_ENEMY | COLLIDABLE_ALIEN

  ld__e_ixl
  ld__d_ixh
  ex de, hl       ; HL <- entity address
  ; TODO use IX as result when alien system in asm
pop ix
  ret

_ko_create::
  ld a, #-16
  add b
  ld b, a         ; B <- alien.y - 16
  ld hl, #_ko_def
  call _entities_create_asm

  ld ENTITY_BODY_VX_HI (ix), #2
  ld ENTITY_BODY_VX_LO (ix), #0x10
  ld ENTITY_BODY_VY (ix), #-8

bullet_create:
  ; body.flags = BODY_HAS_GRAVITY | BODY_HAS_FRICTION | BODY_HAS_BOUNCE
  ld a, #(BODY_HAS_GRAVITY | BODY_HAS_FRICTION | BODY_HAS_BOUNCE)
  ld ENTITY_BODY_FLAGS (ix), a

  ret

_ufo_create::
  ld a, #-8
  add b
  ld b, a         ; B <- alien.y - 8
  ld hl, #_ufo_def
  call _entities_create_asm

  ld ENTITY_BODY_VX_HI (ix), #2
  ld ENTITY_BODY_VX_LO (ix), #0x80
  ld ENTITY_BODY_VY (ix), #-5

  jr bullet_create


.equ ALIEN_PILOTING_VX_FAST, #0x0400
.equ ALIEN_TURNING_VX_FAST, #0x0200

.equ ALIEN_PILOTING_VX_MEDIUM, #0x0300
.equ ALIEN_TURNING_VX_MEDIUM, #0x0200

.equ ALIEN_PILOTING_VX_SLOW, #0x0200
.equ ALIEN_TURNING_VX_SLOW, #0x0140

.equ ALIEN_PILOTING_VX_VERY_SLOW, #0x0080

.equ ALIEN_RECHARGE_VX, #0x0100
.equ ALIEN_TAKE_OFF_VX, #0x0080

_alien_patrolSequence::
  SET_ANIMATION ALIEN_PILOTING
  SET_RELATIVE_VX ALIEN_PILOTING_VX_MEDIUM
  IDLE
    CONDITION nearScreenRight, turn
    CONDITIONS_END
turn:
  SET_ANIMATION ALIEN_TURNING
  ; PLAY_SFX SKIDDING
  SET_RELATIVE_VX ALIEN_TURNING_VX_MEDIUM
  IDLE
    CONDITION almostInScreenRight, pilot
    CONDITIONS_END
pilot:
  SET_RELATIVE_VX -ALIEN_PILOTING_VX_MEDIUM
alien_patrolSequence_half:
  SET_ANIMATION ALIEN_PILOTING_LEFT
  IDLE
    CONDITION nearScreenLeft, return
    CONDITIONS_END
return:
  SET_ANIMATION ALIEN_TURNING_LEFT
  ; PLAY_SFX SKIDDING
  SET_RELATIVE_VX -ALIEN_TURNING_VX_MEDIUM
  IDLE
    CONDITION almostInScreenLeft, patrol_end
    CONDITIONS_END
patrol_end:
  SEQUENCE_END

_alien_hoverAndShootSequence::
  SET_ANIMATION ALIEN_HOVERING
  WAIT_RELATIVE_VX 40, shoot, #0
    CONDITIONS_END
shoot:
  SET_ANIMATION ALIEN_SHOOTING
  PLAY_SFX SHOT
  CREATE chicken
  CREATE shot
  WAIT_RELATIVE_VX 14, end_shooting, #0
    CONDITIONS_END
end_shooting:
  SEQUENCE_END

_alien_rechargeSequence::
  SET_ANIMATION ALIEN_PILOTING
  WAIT_RELATIVE_VX 90, get_mad, ALIEN_PILOTING_VX_VERY_SLOW
    CONDITION nearScreenRight, get_mad
    CONDITIONS_END
get_mad:
  SET_RELATIVE_VX #0
  SET_ANIMATION ALIEN_MADDING
  WAIT_RELATIVE_VX_PLAYING_SFX 80, do_recharge, #0, MAD, 6
    CONDITION almostInScreenRight, do_recharge
    CONDITION nearScreenLeft, do_recharge
    CONDITIONS_END
do_recharge:
  SET_ANIMATION ALIEN_RECHARGING
  SET_VX ALIEN_RECHARGE_VX
  ADD_GRAVITY
  WAIT 12, recharge_fx
    CONDITION almostInScreenLeft, take_off
    CONDITIONS_END
recharge_fx:
  CALL_0 _alien_createRecharge
  WAIT_PLAYING_SFX 240, take_off, RECHARGE, 60
    CONDITION playerCollision, _alienHit_behavior
    CONDITION almostInScreenLeft, take_off
    CONDITIONS_END
take_off:
  CALL_0 _alien_destroyRecharge
  SET_ANIMATION ALIEN_HOVERING
  SET_RELATIVE_VX ALIEN_TAKE_OFF_VX
  REMOVE_GRAVITY
  IGNORE_COLLISIONS
  SET_VY -10
  IDLE
    CONDITION atCruiseAltitude, end_recharge
    CONDITIONS_END
end_recharge:
  SET_VY 0
  ACKNOWLEDGE_COLLISIONS
  SEQUENCE_END


_alienEasy_behavior::
_alien_behavior::
  SET_RELATIVE_VX -ALIEN_PILOTING_VX_FAST
  SEQUENCE alien_patrolSequence_half
easy_patrol:
  SEQUENCE _alien_patrolSequence
  SEQUENCE _alien_rechargeSequence
  GOTO easy_patrol


_alienMedium_behavior::
  SET_RELATIVE_VX -ALIEN_PILOTING_VX_FAST
  SEQUENCE alien_patrolSequence_half
medium_patrol:
  SEQUENCE _alien_patrolSequence
  SEQUENCE _alien_hoverAndShootSequence
  SEQUENCE _alien_hoverAndShootSequence
  SEQUENCE _alien_patrolSequence
  SEQUENCE _alien_hoverAndShootSequence
  SEQUENCE _alien_rechargeSequence
  GOTO medium_patrol


_alienHard_behavior::
  SET_RELATIVE_VX -ALIEN_PILOTING_VX_FAST
  SEQUENCE alien_patrolSequence_half
hard_patrol:
  SEQUENCE _alien_patrolSequence
  SEQUENCE _alien_hoverAndShootSequence
  SEQUENCE _alien_hoverAndShootSequence
  SEQUENCE _alien_hoverAndShootSequence
  SEQUENCE _alien_patrolSequence
  SEQUENCE _alien_hoverAndShootSequence
  SEQUENCE _alien_hoverAndShootSequence
  SEQUENCE _alien_rechargeSequence
  GOTO hard_patrol


_alienHit_behavior::
  CALL_0 _alien_destroyRecharge
  CALL_0 _alien_hit
  CREATE ko
  CREATE ufo
  GOTO _clear_behavior

_alienEscape_behavior::
  IGNORE_COLLISIONS
  REMOVE_GRAVITY
  SET_ANIMATION ALIEN_PILOTING
  SET_VX ALIEN_PILOTING_VX_FAST
  SET_VY #-1
  IDLE
    CONDITION almostInScreenRight, escape_end
    CONDITIONS_END
escape_end:
  CALL_0 _failure_force
  IDLE
    CONDITIONS_END

_ufo_behavior::
  IDLE
    CONDITION notInScreen, DESTROY_ENTITY
    CONDITION notInScreenRight, _clear_behavior
    ; CONDITION playerCollision, _clear_behavior
    CONDITION isSlow, ufo_static
    CONDITIONS_END
ufo_static:
  SET_STATIC
  SET_COLLIDABLE UFO
  IDLE
    CONDITION playerCollision, _clear_behavior
    CONDITION notInScreen, DESTROY_ENTITY
    CONDITIONS_END
