;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "entity.h.s"
.include "factories.h.s"
.include "systems/entities.h.s"
.include "systems/ai.h.s"
.include "systems/player.h.s"
.include "data/game_assets.h.s"
.include "lang/macros.h.s"
.include "lang/math.h.s"

.module game

.globl cpct_getRandom_mxor_u8_asm

_chicken_def::
  FACTORY_DEF COMPONENT_ALL, CHICKEN_SHOOTING, chicken
_chicken_big_def::
  FACTORY_DEF COMPONENT_ALL, CHICKEN_SHOOTING_BIG_COLLIDER, chicken
_chicken_weak_def::
  FACTORY_DEF COMPONENT_ALL, CHICKEN_SHOOTING_WEAKLY, chicken

; IN:
;   DE <- x
;   B <- y
_chicken_create::
  ; u8 velocity = (random_get() & 0x03) + 1;
  push bc
  push de
    call cpct_getRandom_mxor_u8_asm
  pop de
  pop bc

  ld a, l
  and #0x03
  inc a           ; A <- velocity
  push af         ; save velocity for later

  ; u16 animation = velocity < 3?
  ;   CHICKEN_SHOOTING_WEAKLY_ADDRESS:
  ;   CHICKEN_SHOOTING_ADDRESS;
    ld hl, #_chicken_def
    cp #3
    jr nc, do_create_chicken
    ld hl, #_chicken_weak_def
do_create_chicken:
    call _entities_create_asm

  pop af
  ; body.vx = fixed_fromInt(velocity) + FIXED_HALF
  ld ENTITY_BODY_VX_HI (ix), a
  ld ENTITY_BODY_VX_LO (ix), #0x80

  set COLLIDABLE_ENEMY_BIT, ENTITY_COLLIDABLE_TYPE (ix)

set_body_flags:
  ; body.flags = BODY_HAS_GRAVITY | BODY_HAS_FRICTION | BODY_HAS_BOUNCE
  ld a, ENTITY_BODY_FLAGS (ix)
  or #(BODY_HAS_GRAVITY | BODY_HAS_FRICTION | BODY_HAS_BOUNCE)
  ld ENTITY_BODY_FLAGS (ix), a

  ret

; IN:
;   DE <- x
;   B <- y
_friendlyChicken_create::
  ; update player mojo
  ; TODO this should be done in player system
  ld a, (#_player_mojo)
  inc a
  rra
  ld (#_player_mojo), a

  ; calculate vx from mojo
  add #5
  rra             ; Carry flag <- old LSB
                  ; A <- [3, 5]
  ld h, a
  ld a, #0x02
  rra             ; MSB <- Carry flag
                  ; A <- 0x01, 0x81, 0x01, 0x81...
  ld l, a         ; HL <- vx
  push hl

  ; u16 animation = vx < 4?
  ;   CHICKEN_SHOOTING_WEAKLY_ADDRESS:
  ;   CHICKEN_SHOOTING_BIG_COLLIDER_ADDRESS;
    ld a, h
    ld hl, #_chicken_big_def
    cp #4
    jr nc, do_create_friendly_chicken
    ld hl, #_chicken_weak_def
do_create_friendly_chicken:
    call _entities_create_asm

  pop hl

  ld ENTITY_BODY_VX_LO (ix), l
  ld ENTITY_BODY_VX_HI (ix), h
  ld ENTITY_BODY_VY (ix), #-7

  ld ENTITY_COLLIDABLE_TYPE (ix), #COLLIDABLE_FURY

  jp set_body_flags


_chicken_behavior::
  IDLE
    CONDITION notInScreen, DESTROY_ENTITY
    CONDITION notInScreenRight, _clear_behavior
    ; CONDITION playerCollision, chicken_jump
    CONDITION isSlow, chicken_stop
    CONDITIONS_END
chicken_stop:
  SET_ANIMATION CHICKEN_LANDING
  GOTO _static_behavior
