;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "entity.h.s"
.include "factories.h.s"
.include "systems/renderer.h.s"
.include "systems/entities.h.s"
.include "systems/ai.h.s"
.include "stages/game.h.s"
.include "data/game_assets.h.s"
.include "data/songs.h.s"

.module failure

.globl _failure_drawBackground

.equ ANIMAL_X, 51
.equ ANIMAL_Y, 80

.equ ALIEN_X, 1
.equ ALIEN_Y, 30
.equ ALIEN_VELOCITY, #0x0200
.equ ALIEN_VELOCITY_SLOW, #0x0100

_failure_def::
  FACTORY_DEF COMPONENT_ALL, ANIMAL_FLOATING, failure

_failureAlien_def::
  FACTORY_DEF COMPONENT_ALL, ALIEN_PILOTING, failureAlien


_failure_create::
  ld hl, (#_renderer_left)
  ld de, #ANIMAL_X
  add hl, de
  ex de, hl         ; DE <- renderer_left + ANIMAL_X
  ld b, #ANIMAL_Y   ; B <- ANIMAL_Y
  ld hl, #_failure_def
  jp _entities_create_asm


_failureAlien_create::
  ld de, (#_renderer_left)
.ifeq ALIEN_X - 1
  inc de            ; DE <- renderer_left + 1
.endif
  ld b, #ALIEN_Y
  ld hl, #_failureAlien_def
  jp _entities_create_asm


_failure_behavior::
  CALL_0 _failure_drawBackground
  WAIT 50, failure_create_alien
    CONDITIONS_END
failure_create_alien:
  ; TODO play alien appearing sound?
  CREATE failureAlien
  WAIT 50, failure_go
    CONDITIONS_END
failure_go:
  SET_VY -1
  PLAY_SFX RECHARGE
  WAIT 40, failure_end
    CONDITIONS_END
failure_end:
  PLAY_SONG FAILURE
  CLEAR_ENTITY


_failureAlien_behavior::
  SET_VX ALIEN_VELOCITY
  WAIT 19, failureAlien_break
    CONDITIONS_END
failureAlien_break:
  ; PLAY_SFX SKIDDING
  SET_ANIMATION ALIEN_TURNING
  WAIT 8, failureAlien_hover
    CONDITIONS_END
failureAlien_hover:
  SET_ANIMATION ALIEN_HOVERING
  SET_VX #0
  WAIT 80, failure_alien_go
    CONDITIONS_END
failure_alien_go:
  SET_ANIMATION ALIEN_PILOTING
  SET_VX ALIEN_VELOCITY_SLOW
  WAIT 40, failure_alien_fade
    CONDITIONS_END
failure_alien_fade:
  FADE_OUT
  PLAY_SONG TITLE
  WAIT 20, failure_alien_exit
    CONDITIONS_END
failure_alien_exit:
  SET_EVENT #GAME_FAILURE
  CLEAR_ENTITY
