;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "entity.h.s"
.include "factories.h.s"
.include "stages/game.h.s"
.include "systems/entities.h.s"
.include "systems/ai.h.s"
.include "systems/player.h.s"
.include "systems/renderer.h.s"
.include "data/game_assets.h.s"
.include "data/songs.h.s"
.include "lang/macros.h.s"
.include "lang/debug.h.s"

.module player

.equ START_X, 30
.equ START_Y, 44

_player_def::
  FACTORY_DEF COMPONENT_ALL, PLAYER_FLYING, player

; IN:
;   -
; OUT:
;   HL <- entity address
_player_create::
push ix             ; TODO this should be done in player system (when in ASM)
  ld hl, (#_renderer_left)
  ld de, #START_X
  add hl, de
  ex de, hl         ; DE <- xInBytes = checkpoint_position + 30
  ld b, #START_Y    ; B <- y

  ld hl, #_player_def
  call _entities_create_asm

  ld ENTITY_COLLIDABLE_TYPE (ix), #COLLIDABLE_PLAYER
  set BODY_HAS_GRAVITY_BIT, ENTITY_BODY_FLAGS (ix)
  ld ENTITY_BODY_VX_HI (ix), #1   ; TODO get player velocity from elsewhere?

  ld__e_ixl
  ld__d_ixh
  ex de, hl       ; HL <- entity address
  ; TODO use IX as result when player system in asm
pop ix
  ret

player_fall:
  SET_ANIMATION PLAYER_FLYING
player_do_fall:
_player_behavior::
  IDLE
    CONDITION onFloor, player_land
    CONDITIONS_END
player_land:
  SET_ANIMATION PLAYER_LANDING
  STOP
  PLAY_SFX LAND
  WAIT 4, player_run
    CONDITION playerFury, player_run_fast
    CONDITIONS_END
player_run:
  SET_ANIMATION PLAYER_RUNNING
  RUN
    CONDITION notOnFloor, player_fall
    CONDITION sliding, player_slide
    CONDITION jumping, player_jump
    CONDITION actionWithMojo, player_set_fury
    CONDITIONS_END
player_slide:
  SET_ANIMATION PLAYER_SLIDING
  PLAY_SFX SLIDE
  WAIT 18, player_run
    CONDITIONS_END
player_jump:
  SET_ANIMATION PLAYER_JUMPING
  REMOVE_GRAVITY
  SET_VY 0
  PLAY_SFX JUMP
  WAIT 6, player_do_jump
    CONDITIONS_END
player_do_jump:
  ADD_GRAVITY
  JUMP
  GOTO player_fall
player_set_fury:
  PLAY_SFX FURY
  CALL_0 _player_setFury
player_run_fast:
  SET_ANIMATION PLAYER_RUNNING_FAST
player_do_run_fast:
  RUN
    CONDITION notOnFloor, player_fall_fast
    CONDITION sliding, player_slide_fast
    CONDITION jumping, player_jump_fast
    CONDITION enemyCollision, player_jump_fast
    CONDITION notPlayerFury, player_run
    CONDITIONS_END
player_slide_fast:
  SET_ANIMATION PLAYER_SLIDING_FAST
  WAIT 12, player_run_fast
    CONDITIONS_END
player_jump_fast:
  PLAY_SFX JUMP
  JUMP
player_fall_fast:
  SET_ANIMATION PLAYER_FLYING_FAST
  IDLE
    CONDITION onFloor, player_land_fast
    CONDITION enemyCollision, player_punch
    CONDITIONS_END
player_punch:
  SET_ANIMATION PLAYER_HITTING
  WAIT 10, player_fall_fast
    CONDITION onFloor, player_land_fast
    CONDITION alienCollision, player_hit_alien
    CONDITIONS_END
player_land_fast:
  SET_ANIMATION PLAYER_RUNNING_FAST
  WAIT 4, player_do_run_fast
    CONDITIONS_END
player_hit_alien:
  CALL_0 _player_setInmortal
  CALL_0 _player_resetFury
  IDLE
    CONDITION onFloor, player_land
    CONDITIONS_END

_playerHit_behavior::
  CREATE mojo
  CREATE mojo
player_do_hit:
  ADD_GRAVITY
  JUMP
  SET_VX #0x0080
  PLAY_SFX HIT
  GOTO player_fall

_playerWater_behavior::
  PLAY_SFX SPLASH
  SET_ANIMATION PLAYER_REPELLING
  ADD_GRAVITY
  JUMP
  GOTO player_do_fall

_playerDie_behavior::
  PLAY_SFX HIT
  SET_ANIMATION PLAYER_DAMAGED
  ADD_GRAVITY
  JUMP
  SET_VX #0x0080
  WAIT 60, player_ko
    CONDITION isSlow, player_ko
    CONDITIONS_END
player_ko:
  PLAY_SONG FAILURE
  SET_ANIMATION PLAYER_KOING
  WAIT 40, player_fade
    CONDITIONS_END
player_fade:
  FADE_OUT
  WAIT 40, player_back_to_checkpoint
    CONDITIONS_END
player_back_to_checkpoint:
  SET_EVENT GAME_BACK_TO_CHECKPOINT
  IDLE
    CONDITIONS_END

_ride_behavior::
  REMOVE_GRAVITY
  SET_ANIMATION PLAYER_RIDING
  RIDE
    CONDITION actionWithMojo, shoot
    CONDITIONS_END
shoot:
  PLAY_SFX SHOT
  CREATE friendlyChicken
  CREATE shot
  SET_ANIMATION PLAYER_SHOOTING
  WAIT 10, ride_without_shooting
    CONDITIONS_END
ride_without_shooting:
  SET_ANIMATION PLAYER_RIDING
  RIDE_WAITING 20, _ride_behavior
    CONDITIONS_END

_rideHit_behavior::
  CREATE mojo
  CREATE mojo
  PLAY_SFX HIT
ride_do_hit:
  SET_VX #0x0080
  SET_ANIMATION PLAYER_SHOOTING
  WAIT 10, _ride_behavior
    CONDITIONS_END

_rideWater_behavior::
  PLAY_SFX SPLASH
  SET_VY -4
  GOTO ride_do_hit

_rideEnd_behavior::
  CREATE ufo
  GOTO player_do_hit
