;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "macros.h.s"
.include "debug.h.s"

.module pvector

; IN:
;   DE <- pvector pointer
;   IX <- pointer to add
;   HL <- comparator function with IX and IY as parameters, with Carry set
;         if swap is needed. This function needs to save all modified
;         registers, breaking the "caller saves its registers" contract
;         for efficiency.
; DESTROYS:
;   AF, BC, DE, HL, IY
_pvector_addSorted_ir_asm::
  ld (#function_pointer), hl
  ex de, hl   ; HL <- pvector address

  ; pvector->next = pvector->next + 1
  ld e, (hl)
  inc hl
  ld d, (hl)

  inc de
  push de     ; storing lddr destination
  inc de
  ld (hl), d
  dec hl
  ld (hl), e

  inc hl
  inc hl

loop:
  ld e, (hl)
  inc hl
  ld d, (hl)  ; DE <- second pointer

  ld a, e
  or d
  jr z, add_to_last_position

  ld__iyl_e
  ld__iyh_d   ; IY <- DE <- second pointer
.equ function_pointer, . + 1
  call loop

  jr c, add_to_current_position
  inc hl
  jr loop

add_to_current_position:
  pop de
  inc de
  inc de
  ld a, e
  sub l
  ld c, a
  ld a, d
  sbc h
  ld b, a     ; BC <- DE - HL = counter

  ld l, e
  ld h, d
  dec hl
  dec hl
  lddr        ; move pointers to the right
  ; set new pointer
  inc hl
  ld__a_ixl
  ld (hl), a
  inc hl
  ld__a_ixh
  ld (hl), a
  ret

add_to_last_position:
  ld__a_ixh
  ld (hl), a
  dec hl
  ld__a_ixl
  ld (hl), a

  ; set next pointer to null
  inc hl
  inc hl
  xor a
  ld (hl), a
  inc hl
  ld (hl), a

  pop de
  ret
