;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "macros.h.s"
.include "debug.h.s"

.module pvector

; IN:
;   DE <- pvector pointer
;   HL <- comparator function with IX and IY as parameters, with Carry set
;         if swap is needed. This function needs to save all modified
;         registers, breaking the "caller saves its registers" contract
;         for efficiency.
; DESTROYS:
;   AF, BC, DE, HL, IX, IY
_pvector_sort_ir_asm::
  ld (#function_pointer), hl

  ex de, hl
  inc hl
  inc hl      ; HL <- first pointer address

  ld c, (hl)
  inc hl
  ld b, (hl)  ; BC <- first pointer

  ld a, c
  or b
  ret z

  inc hl      ; HL <- next pointer address
  ld__ixl_c
  ld__ixh_b   ; IX <- BC <- first pointer

loop:
  ld e, (hl)
  inc hl
  ld d, (hl)  ; DE <- second pointer

  ld a, e
  or d
  ret z

  ld__iyl_e
  ld__iyh_d   ; IY <- DE <- second pointer

.equ function_pointer, . + 1
  call loop

  jr nc, swap_values

  ld c, e
  ld b, d     ; BC <- second pointer (first pointer for the next iteration)
  ld__ixl_c
  ld__ixh_b   ; IX <- BC <- first pointer

  inc hl      ; HL <- next pointer address
  jr loop

swap_values:
  ld (hl), b
  dec hl
  ld (hl), c
  dec hl
  ld (hl), d
  dec hl
  ld (hl), e
  inc hl
  inc hl
  inc hl
  inc hl
  jr loop
