/*
  The Abduction of Oscar Z
  ========================

  "Oscar loved watching night skies long after his parents slept. But tonight
  that sky was watching him..."

  * PRODUCTION & CODE / Tulo / @tulinmola
  * ART & STORY / Alberto Hache / @Alberto_H9
  * MUSIC & FX / Gryzor87 / @Gryzor87

  A 2020 Dreamin`Bits production
  https://dreaminbits.com/

  MIT License

  Copyright (c) 2020 Dreamin`Bits

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the"Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#include "game.h"
#include "../cpc/audio.h"
#include "../cpc/wait.h"
#include "../cpc/controls.h"
#include "../systems/collision.h"
#include "../systems/renderer.h"
#include "../systems/entities.h"
#include "../systems/terrain.h"
#include "../systems/platform.h"
#include "../systems/water.h"
#include "../systems/checkpoint.h"
#include "../systems/background.h"
#include "../systems/hud.h"
#include "../systems/alien.h"
#include "../systems/player.h"
#include "../systems/physics.h"
#include "../systems/animator.h"
#include "../systems/camera.h"
#include "../systems/ai.h"
#include "../systems/success.h"
#include "../systems/failure.h"
#include "../systems/record.h"
#include "../systems/playback.h"
#include "../systems/event.h"
#include "../data/game_assets.h"
#include "../data/songs.h"

#include "../lang/debug.h"

void game_step();
void game_end();
void game_checkExit();

// #define DEBUG_KEYS_ENABLED  1

#ifdef DEBUG_KEYS_ENABLED
  #include <cpctelera.h>
  #include "../cpc/palette.h"
  void game_checkDebugKeys();
#endif

void game_start(u8 levelIndex, u8 mode) {
  const Level* level = &game_levels[levelIndex];

  if (mode == GAME_NORMAL_MODE || mode == GAME_ATTRACT_MODE){
    audio_playSong(GAME_SONG);
  } else {
    audio_playNothing();
  }

  renderer_clearWithPosition(level, checkpoint_position);
  entities_clear(level);
  terrain_clear(level);
  platform_clear(level);
  water_clear(level);
  checkpoint_clear(level);
  physics_clear();
  background_clear();
  collision_clear();
  alien_clear(level, mode);
  player_clear(mode);
  hud_clear(level);
  camera_clear();
  animator_clear();
  ai_clear();
  success_clear(level, mode);
  failure_clear(level, mode);
  playback_clear(mode);
  record_clear();
  event_clear();

  game_step();
  renderer_ensureFirstSync();
  renderer_fadeIn();
  while (!event) {
    game_step();
  }
}

void game_step() {
  water_step();
  terrain_step();
  background_step();
  platform_step();
  checkpoint_step();
  entities_step();
  animator_step();
  physics_step();
  alien_step();
  player_step();
  hud_step();
  ai_step();
  success_step();
  failure_step();
  renderer_step();
  collision_step();
  record_step();
  playback_step();

  renderer_draw();

#ifdef DEBUG_KEYS_ENABLED
  game_checkDebugKeys();
#endif

  camera_apply();
  camera_step();

  game_checkExit();
}

void game_checkExit() {
  // TODO this should be some kind of "options stage"
  if (controls_isExitPressed()) {
    renderer_fadeOutWaiting();
    event = GAME_EXIT;
  }
}

#ifdef DEBUG_KEYS_ENABLED
void game_checkDebugKeys() {
  if (cpct_isKeyPressed(Key_L)) {
    while (cpct_isKeyPressed(Key_L));
    alien_position = player_entity->position.x;
  }
  if (cpct_isKeyPressed(Key_R)) {
    player_setRiding();
  }
  if (cpct_isKeyPressed(Key_B)) {
    event = GAME_BACK_TO_CHECKPOINT;
  }
  if (cpct_isKeyPressed(Key_N)) {
    checkpoint_position = checkpoint_getNext();
    if (checkpoint_position != 0xffff) {
      if ((checkpoint_position >> 7) + 5 > alien_levelPosition) {
        checkpoint_alienState = alien_ko;
      }
      renderer_fadeOutWaiting();
      event = GAME_BACK_TO_CHECKPOINT;
    }
  }
  if (cpct_isKeyPressed(Key_M)) {
    player_addMojo();
  }
  if (cpct_isKeyPressed(Key_I)) {
    player_setInmortal();
  }
  if (cpct_isKeyPressed(Key_F)) {
    failure_force();
  }
}
#endif
