/*
  The Abduction of Oscar Z
  ========================

  "Oscar loved watching night skies long after his parents slept. But tonight
  that sky was watching him..."

  * PRODUCTION & CODE / Tulo / @tulinmola
  * ART & STORY / Alberto Hache / @Alberto_H9
  * MUSIC & FX / Gryzor87 / @Gryzor87

  A 2020 Dreamin`Bits production
  https://dreaminbits.com/

  MIT License

  Copyright (c) 2020 Dreamin`Bits

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the"Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#include "manager.h"
#include "title.h"
#include "game.h"
#include "ending.h"
#include "../systems/checkpoint.h"
#include "../systems/renderer.h"
#include "../systems/event.h"
#include "../data/game_assets.h"

#define FORCE_TITLE   255

void manager_start() {
  u8 level = 0;
  u8 lastLevel = FORCE_TITLE;
  u8 mode = GAME_NORMAL_MODE;

  event_clear();
  checkpoint_reset();

  while (!event) {
    if (level != lastLevel) {
      title_start(level);
      lastLevel = level;
    }
    game_start(level, mode);

    switch (event) {
      case GAME_BACK_TO_CHECKPOINT: break;
      case GAME_SUCCESS: level++; checkpoint_reset(); break;
      case GAME_FAILING: renderer_fadeOutWaiting(); mode = GAME_FAILING_MODE; break;
      case GAME_FAILURE: lastLevel = FORCE_TITLE; mode = GAME_NORMAL_MODE; checkpoint_reset(); break;
    }

    if (event != GAME_EXIT) {
      event_clear();
    }

    if (level == LEVEL_COUNT) {
      ending_start();
      event = GAME_EXIT;
    }
  }
}
