/*
  The Abduction of Oscar Z
  ========================

  "Oscar loved watching night skies long after his parents slept. But tonight
  that sky was watching him..."

  * PRODUCTION & CODE / Tulo / @tulinmola
  * ART & STORY / Alberto Hache / @Alberto_H9
  * MUSIC & FX / Gryzor87 / @Gryzor87

  A 2020 Dreamin`Bits production
  https://dreaminbits.com/

  MIT License

  Copyright (c) 2020 Dreamin`Bits

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the"Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#include "menu.h"
#include "manager.h"
#include "game.h"
#include "../cpc/audio.h"
#include "../cpc/wait.h"
#include "../systems/event.h"
#include "../systems/renderer.h"
  #include "../systems/collision.h" // TODO (look below comment)
#include "../systems/terrain.h"
#include "../systems/sunset.h"
#include "../systems/entities.h"
#include "../systems/physics.h"
#include "../systems/animator.h"
#include "../systems/ai.h"
#include "../systems/menu.h"
#include "../systems/checkpoint.h"
#include "../data/level.h"
#include "../data/game_assets.h"
#include "../data/songs.h"
#include "../lang/debug.h"
#include "../lang/random.h"

#define TOP_BORDER_COLOR      0
#define BOTTOM_BORDER_COLOR   0

#define DISPLACEMENT          2

#define EVENT_GAME          1
#define EVENT_ATTRACT_MODE  2

void menu_loop();

LEVEL(menu_level, TOP_BORDER_COLOR, BOTTOM_BORDER_COLOR, BLACK_INKS, MENU_TEXTURE)

void attract_start();

void menu_start() {
  const Level* level = (const Level*)&menu_level;

  renderer_clear(level);
  entities_clear(level);
  terrainRandom_clear(level);
  collision_clear();  // TODO this is not cool here...
                      // TODO entities should add to collision system only if entity is collidable
  sunset_clear();
  physics_clear();
  animator_clear();
  ai_clear();
  event_clear();
  menu_clear();

  menu_loop();
  audio_playSong(MENU_SONG);
  renderer_ensureFirstSync();
  renderer_fadeIn();
  while (!event) {
    menu_loop();
  }

  switch (event) {
    case EVENT_GAME: manager_start(); break;
    case EVENT_ATTRACT_MODE: attract_start(); break;
  }
}

void menu_loop() {
  terrain_step();
  sunset_step();
  physics_step();
  animator_step();
  ai_step();
  menu_step();
  renderer_step();

  renderer_draw();
  renderer_move(DISPLACEMENT);
}

void attract_start() {
  event_clear();
  checkpoint_reset();
  random_reset();
  game_start(2, GAME_ATTRACT_MODE);
}
