;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.globl _ai_setBehavior_asm
.globl _ai_runBehavior_asm

.globl _ai_nextBehavior
.globl _ai_checkConditions

.globl _clear_behavior
.globl _static_behavior
.globl _bullet_behavior

.globl _action_idle
.globl _action_wait
.globl _action_setWaitRelativeVx
.globl _action_waitRelativeVx
.globl _action_sequence
.globl _action_sequenceEnd
.globl _action_setAnimation
.globl _action_drawString
.globl _action_fadeIn
.globl _action_fadeOut
.globl _action_playSong
.globl _action_playSfx
.globl _action_setLoop
.globl _action_waitPlayingSfx
.globl _action_setStatic
.globl _action_resetStatic
.globl _action_addGravity
.globl _action_removeGravity
.globl _action_resetEnemy
.globl _action_setBullet
.globl _action_ignore_collisions
.globl _action_acknowledge_collisions
.globl _action_setEvent
.globl _action_setTimer
.globl _action_setRandomTimer
.globl _action_setVx
.globl _action_setRelativeVx
.globl _action_setVy
.globl _action_setCollidable
.globl _action_ride
.globl _action_rideWaiting
.globl _action_run
.globl _action_idleRunning
.globl _action_jump
.globl _action_create
.globl _action_call_0
.globl _action_stopCamera
.globl _action_die
.globl _action_clear
.globl _action_setControl
.globl _action_debug

.globl _condition_timeout
.globl _condition_true
.globl _condition_onFloor
.globl _condition_inWater
.globl _condition_notOnFloor
.globl _condition_sliding
.globl _condition_jumping
.globl _condition_actionWithMojo
.globl _condition_action
.globl _condition_notInScreen
.globl _condition_notInScreenRight
.globl _condition_nearScreenLeft
.globl _condition_almostInScreenLeft
.globl _condition_nearScreenRight
.globl _condition_almostInScreenRight
.globl _condition_nearPlayer
.globl _condition_nearestToPlayer
.globl _condition_playerCollision
.globl _condition_alienCollision
.globl _condition_furyCollision
.globl _condition_enemyCollision
.globl _condition_playerFury
.globl _condition_notPlayerFury
.globl _condition_isSlow
.globl _condition_atCruiseAltitude
.globl _condition_playPressed
.globl _condition_redefinePressed
.globl _condition_anyKeyPressed
.globl _condition_notAnyKeyPressed
.globl _condition_getUnusedControl

.equ DESTROY_ENTITY, #0

.macro CLEAR_ENTITY
  .dw _action_clear
.endm

.macro SET_EVENT value
  .dw _action_setEvent
  .db value
.endm

.macro SET_TIMER value
  .dw _action_setTimer
  .db value
.endm

.macro SET_RANDOM_TIMER min, mask
  .dw _action_setRandomTimer
  .db min, mask
.endm

.macro SEQUENCE address
  .dw _action_sequence
  .dw address
.endm

.macro SEQUENCE_END
  .dw _action_sequenceEnd
.endm

.macro WAIT_PLAYING_SFX time, next, name, period
  SET_TIMER time
  .dw _action_setLoop, 'name'_SFX_ADDRESS
  .db period
  .dw _action_waitPlayingSfx
  .dw _condition_timeout, next
.endm

.macro WAIT time, next
  SET_TIMER time
  .dw _action_wait
  .dw _condition_timeout, next
.endm

.macro CONTINUE_WAITING next
  .dw _action_wait
  .dw _condition_timeout, next
.endm

; .macro CONTINUE_WAITING_PLAYING_SFX next
;   .dw _action_waitPlayingSfx
;   .dw _condition_timeout, next
; .endm

.macro WAIT_RELATIVE_VX time, next, vx
  SET_TIMER time
  .dw _action_setWaitRelativeVx, vx
  .dw _action_waitRelativeVx
  .dw _condition_timeout, next
.endm

.macro WAIT_RELATIVE_VX_PLAYING_SFX time, next, vx, name, period
  SET_TIMER time
  .dw _action_setWaitRelativeVx, vx
  .dw _action_setLoop, 'name'_SFX_ADDRESS
  .db period
  .dw _action_waitPlayingSfx
  .dw _condition_timeout, next
.endm

.macro RANDOM_WAIT min, mask, next
  SET_RANDOM_TIMER min, mask
  .dw _action_wait
  .dw _condition_timeout, next
.endm

.macro NEXT_STEP ?continue
  WAIT 10, continue
    CONDITIONS_END
continue:
.endm

.macro IDLE
  .dw _action_idle
.endm

.macro CONDITIONS_END
  .dw #0
.endm

.macro CONDITION condition, next
  .dw _condition_'condition', next
.endm

.macro GOTO behavior
  .dw _action_idle
  CONDITION true, behavior
  ; CONDITIONS_END
.endm

.macro SET_ANIMATION animation
  .dw _action_setAnimation, 'animation'_ADDRESS
.endm

.macro DRAW_STRING x, y, name
  .dw _action_drawString
  RENDERER_GAME_SCREEN_POINTER x, y
  .dw 'name'_ADDRESS
.endm

.macro DRAW_STRING_WITH_LABEL x, y, name, label
  .dw _action_drawString
  RENDERER_GAME_SCREEN_POINTER x, y
label::
  .dw 'name'_ADDRESS
.endm

.macro FADE_IN
  .dw _action_fadeIn
.endm

.macro FADE_OUT
  .dw _action_fadeOut
.endm

.macro PLAY_SONG name
  .dw _action_playSong
  .db 'name'_SONG
.endm

.macro PLAY_SFX name
  .dw _action_playSfx, 'name'_SFX_ADDRESS
.endm

.macro SET_STATIC
  .dw _action_setStatic
.endm

.macro RESET_STATIC
  .dw _action_resetStatic
.endm

.macro IGNORE_COLLISIONS
  .dw _action_ignore_collisions
.endm

.macro ACKNOWLEDGE_COLLISIONS
  .dw _action_acknowledge_collisions
.endm

.macro SET_VX vx
  .dw _action_setVx, vx
.endm

.macro SET_RELATIVE_VX vx
  .dw _action_setRelativeVx, vx
.endm

.macro STOP
  SET_VX #0
.endm

.macro SET_VY vy
  .dw _action_setVy
  .db vy
.endm

.macro SET_COLLIDABLE value
  .dw _action_setCollidable
  .db COLLIDABLE_'value'
.endm

.macro ADD_GRAVITY
  .dw _action_addGravity
.endm

.macro REMOVE_GRAVITY
  .dw _action_removeGravity
.endm

.macro RESET_ENEMY
  .dw _action_resetEnemy
.endm

.macro SET_BULLET
  .dw _action_setBullet
.endm

.macro RIDE
  .dw _action_ride
.endm

.macro RIDE_WAITING time, next
  SET_TIMER time
  .dw _action_rideWaiting
  .dw _condition_timeout, next
.endm

.macro RUN
  .dw _action_run
.endm

.macro IDLE_RUNNING
  .dw _action_idleRunning
.endm

.macro JUMP
  .dw _action_jump
.endm

.macro CREATE type
  .dw _action_create, _'type'_create
.endm

.macro CALL_0 address
  .dw _action_call_0, address
.endm

.macro STOP_CAMERA
  .dw _action_stopCamera
.endm

.macro DIE
  .dw _action_die
.endm

.macro SET_CONTROL index
  .dw _action_setControl
  .db index
.endm

.macro DEBUG
  .dw _action_debug
.endm

.macro RETURN_IF_ALIEN_IN_SCREEN
  call _alien_inScreen
  ld a, l
  or a
  ret nz
.endm
