;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "ai.h.s"
.include "entities.h.s"
.include "lang/macros.h.s"
.include "entities/entity.h.s"
.include "lang/debug.h.s"

.module ai

.globl ___sdcc_call_hl

; void ai_setBehavior(Entity* entity, u16 address) {
;   entity->behavior.action = (Action)address;
; }
; _ai_setBehavior::
;   pop af      ;; AF <- return address
;   pop bc      ;; BC <- entity address
;   pop de      ;; DE <- behavior address
;   push af     ;; Put returning address in the stack again

;   push ix
;   ld__ixl_c
;   ld__ixh_b     ; IX <- entity address
;   call _ai_setBehavior_asm
;   pop ix
;   ret

; IN:
;   IX <- entity address
;   DE <- behavior address
; DESTROYS:
;   NA
_ai_setBehavior_asm::
  ld ENTITY_BEHAVIOR_ACTION_LO (ix), e
  ld ENTITY_BEHAVIOR_ACTION_HI (ix), d
  ret

_ai_step::
  push ix
  ld hl, #_ai_runBehavior_asm
  call _entities_each_asm
  pop ix
  ret

; IN:
;   IX <- Entity pointer
_ai_runBehavior_asm::
  ld e, ENTITY_BEHAVIOR_ACTION_LO (ix)
  ld d, ENTITY_BEHAVIOR_ACTION_HI (ix)
                ; DE <- behavior address

do_run:
  ld a, (de)
  ld l, a
  inc de
  ld a, (de)
  ld h, a       ; HL <- action address
  inc de        ; DE <- behaviour address after action address

  jp (hl)

_ai_nextBehavior::
  ld a, e
  or d
  jr z, destroy_entity
  ; TODO this will eventualy be just: jp z, _entities_destroy_asm

  ld ENTITY_BEHAVIOR_ACTION_LO (ix), e
  ld ENTITY_BEHAVIOR_ACTION_HI (ix), d
  jr do_run

destroy_entity:
  ld__e_ixl
  ld__d_ixh
  ex de, hl     ; HL <- entity address
  jp _entities_destroy
  ; ret

_ai_checkConditions::
  ld a, (de)
  ld l, a
  inc de
  ld a, (de)
  ld h, a       ; HL <- condition address
  or l          ; Check for NULL condition
  ret z

  inc de
  call ___sdcc_call_hl
  jr z, set_condition_action

  inc de
  inc de
  jr _ai_checkConditions

set_condition_action:
  ex de, hl
  ld e, (hl)
  inc hl
  ld d, (hl)
  jp _ai_nextBehavior


_clear_behavior::
  CLEAR_ENTITY


_static_behavior::
  IDLE
    CONDITION onFloor, static_set
    CONDITION notInScreen, DESTROY_ENTITY
    CONDITIONS_END
static_set:
  SET_STATIC
  IDLE
    CONDITION notInScreen, DESTROY_ENTITY
    CONDITIONS_END


_bullet_behavior::
  IDLE
    CONDITION notInScreen, DESTROY_ENTITY
    CONDITION notInScreenRight, _clear_behavior
    CONDITION isSlow, _static_behavior
    CONDITIONS_END
