;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "lang/macros.h.s"
.include "lang/debug.h.s"
.include "cpc/controls.h.s"
.include "cpc/audio.h.s"
.include "entities/entity.h.s"
.include "ai.h.s"
.include "animator.h.s"
.include "terrain.h.s"
.include "player.h.s"
.include "entities.h.s"
.include "renderer.h.s"
.include "event.h.s"

.module ai_actions


.globl ___sdcc_call_hl
.globl cpct_getRandom_mxor_u8_asm


_action_setAnimation::
  ex de, hl
  ld e, (hl)
  inc hl
  ld d, (hl)    ; DE <- animation
  inc hl

  push hl
    call _animator_setAnimation_asm
  pop de        ; DE <- next behavior position

  jp _ai_nextBehavior


_action_drawString::
  ex de, hl
  ld e, (hl)
  inc hl
  ld d, (hl)    ; DE <- screen address
  inc hl
  ld c, (hl)
  inc hl
  ld b, (hl)
  inc hl        ; BC <- string address
  push hl
    call _renderer_drawString_asm
  pop de
  jp _ai_nextBehavior


_action_fadeIn::
  call _interrupts_startFadeIn
  jp _ai_nextBehavior


_action_fadeOut::
  call _interrupts_startFadeOut
  jp _ai_nextBehavior


_action_playSong::
  ld a, (de)
  ld l, a       ; L <- song index
  inc de
  push de
    call _audio_playSong
  pop de
  jp _ai_nextBehavior


_action_playSfx::
  ld a, (de)
  ld l, a
  inc de
  ld a, (de)
  ld h, a       ; HL <- sfx address
  inc de
  push de
  push ix
    call _audio_playSfx
  pop ix
  pop de
  jp _ai_nextBehavior


_action_loopSfx::
  .dw #0
_action_loopCounter::
  .db #0
_action_loopPeriod::
  .db #0

_action_setLoop::
  ld a, (de)
  inc de
  ld l, a
  ld a, (de)
  inc de
  ld h, a       ; HL <- sfx address
  ld (#_action_loopSfx), hl
  ld a, #1
  ld (#_action_loopCounter), a
  ld a, (de)
  inc de
  ld (#_action_loopPeriod), a
  jp _ai_nextBehavior


_action_waitPlayingSfx::
  ld hl, #_action_loopCounter
  dec (hl)
  jp nz, _action_wait

  push de
  push ix
    ld hl, (#_action_loopSfx)
    call _audio_playSfx
  pop ix
  pop de
  ld a, (#_action_loopPeriod)
  ld (#_action_loopCounter), a

  jp _action_wait


_action_setStatic::
  res #COMPONENT_BODY_BIT, ENTITY_COMPONENTS (ix)
                ; remove body component
  ; TODO remove from physics system when available
  ; call _physics_removeEntity_asm
  set #DRAWING_OPAQUE_BIT, ENTITY_DRAWING_FLAGS (ix)
                ; and make not clear background for this entity
  push de
  push ix
    call _renderer_addStain
  pop ix
  pop de
  jp _ai_nextBehavior


_action_resetStatic::
  set #COMPONENT_BODY_BIT, ENTITY_COMPONENTS (ix)
                ; add body component
  ; TODO add to physic system when available
  ; call _physic_addEntity_asm
  res #DRAWING_OPAQUE_BIT, ENTITY_DRAWING_FLAGS (ix)
                ; and make *do* clear background for this entity
  jp _ai_nextBehavior


_action_addGravity::
  set #BODY_HAS_GRAVITY_BIT, ENTITY_BODY_FLAGS (ix)
  jp _ai_nextBehavior


_action_removeGravity::
  res #BODY_HAS_GRAVITY_BIT, ENTITY_BODY_FLAGS (ix)
  jp _ai_nextBehavior


_action_resetEnemy::
  res #COLLIDABLE_ENEMY_BIT, ENTITY_COLLIDABLE_TYPE (ix)
  jp _ai_nextBehavior


_action_setBullet::
  ; set physical component
  ; TODO add to physic system when available *IF NOT YET ADDED*
  ; bit COMPONENT_BODY_BIT, ENTITY_COMPONENTS (ix)
  ; jr nz, . + 3
  ; call _physic_addEntity_asm
  ; or _physic_addEntityIfXXX or check existence in add function
  set COMPONENT_BODY_BIT, ENTITY_COMPONENTS (ix)
  ld a, #(BODY_HAS_GRAVITY | BODY_HAS_FRICTION | BODY_HAS_BOUNCE)
  ld ENTITY_BODY_FLAGS (ix), a

  ld de, #_bullet_behavior
  call _ai_setBehavior_asm

  jp _ai_runBehavior_asm


_action_ignore_collisions::
  set #BODY_IGNORE_COLLISIONS_BIT, ENTITY_BODY_FLAGS (ix)
  jp _ai_nextBehavior


_action_acknowledge_collisions::
  res #BODY_IGNORE_COLLISIONS_BIT, ENTITY_BODY_FLAGS (ix)
  jp _ai_nextBehavior


_action_setEvent::
  ld a, (de)
  inc de
  EVENT_SET
  jp _ai_nextBehavior


_action_setTimer::
  ld a, (de)
  inc de
  ld ENTITY_TIMER_VALUE (ix), a
  jp _ai_nextBehavior


_action_setRandomTimer::
  ld a, (de)
  inc de        ; A <- minimum value

  push af
    push de
      call cpct_getRandom_mxor_u8_asm
                ; L <- random u8
    pop de

    ld a, (de)  ; A <- mask value
    inc de
    and l
    ld l, a     ; L <- random & mask

  pop bc        ; B <- minimum value
  
  add l         ; A <- min + (rand & mask)
  ld ENTITY_TIMER_VALUE (ix), a

  jp _ai_nextBehavior


_action_setVx::
  ld a, (de)
  ld ENTITY_BODY_VX_LO (ix), a
  inc de
  ld a, (de)
  ld ENTITY_BODY_VX_HI (ix), a
  inc de
  jp _ai_nextBehavior


_action_setRelativeVx::
  ld a, (de)
  ld l, a
  inc de
  ld a, (de)
  ld h, a       ; HL <- vx
  inc de

  ld bc, (#_player_velocity)
  add hl, bc    ; HL <- vx + player_velocity

  ld ENTITY_BODY_VX_LO (ix), l
  ld ENTITY_BODY_VX_HI (ix), h

  jp _ai_nextBehavior


_action_setVy::
  ld a, (de)
  ld ENTITY_BODY_VY (ix), a
  inc de
  jp _ai_nextBehavior


_action_setCollidable::
  ld a, (de)
  or ENTITY_COLLIDABLE_TYPE (ix)
  ld ENTITY_COLLIDABLE_TYPE (ix), a
  inc de
  jp _ai_nextBehavior


; TODO extract to player system
_action_applyVelocityControls::
  push de
    ld l, #CONTROLS_P1_LEFT
    call _controls_isPressed
    or a
    jr z, check_right_control
    ld bc, #-0x0080
    jr apply_velocity

check_right_control:
    ld l, #CONTROLS_P1_RIGHT
    call _controls_isPressed
    or a
    jr z, no_direction_control
    ld bc, #0x0080
    jr apply_velocity

no_direction_control:
    ld bc, #0

apply_velocity:
  pop de

  ; ld hl, #_player_status
  ; bit #PLAYER_FURY_BIT, (hl)
  ;                   ; Z <- in fury mode?

  ld hl, (#_player_velocity)
  ; jr z, do_apply_velocity

;   sla l
;   rl h              ; x2 velocity in fury mode

; do_apply_velocity:
  add hl, bc        ; HL <- new velocity

  ld ENTITY_BODY_VX_LO (ix), l
  ld ENTITY_BODY_VX_HI (ix), h
  ret

.equ RIDE_MAX_VX, 3
.equ RIDE_MIN_VX, 0
.equ RIDE_MAX_VY, 6
.equ RIDE_MIN_VY, -RIDE_MAX_VY

; TODO extract to player system
_player_ride::
  ld l, #CONTROLS_P1_LEFT
  call _controls_isPressed
  or a
  jr z, check_ax_control
  ld bc, #-0x0020
  jr check_ay

check_ax_control:
  ld l, #CONTROLS_P1_RIGHT
  call _controls_isPressed
  or a
  jr z, no_ax_control
  ld bc, #0x0020
  jr check_ay

no_ax_control:
  ld hl, (#_player_velocity)
  ld c, ENTITY_BODY_VX_LO (ix)
  ld b, ENTITY_BODY_VX_HI (ix)

  or a
  sbc hl, bc      ; HL <- -player_velocity ...
  ld c, l
  ld b, h         ; BC <- -vx (relative to player_velocity)
  sra b
  rr c            ; BC <- -VX / 2
  sra b
  rr c            ; BC <- -VX / 4
  sra b
  rr c            ; BC <- -VX / 4

check_ay:
  ld l, #CONTROLS_P1_UP
  push bc
    call _controls_isPressed
  pop bc
  or a
  jr z, check_ay_control
  ld l, #-1
  jr apply_accelerations

check_ay_control:
  ld l, #CONTROLS_P1_DOWN
  push bc
    call _controls_isPressed
  pop bc
  or a
  jr z, no_ay_control
  ld l, #1
  jr apply_accelerations

no_ay_control:
  ld a, ENTITY_BODY_VY (ix)
  neg
  sra a
  ld l, a         ; L <- -vy / 2

apply_accelerations:
  ld a, ENTITY_BODY_VY (ix)
  add l

  cp #RIDE_MAX_VY
  jp m, continue_checking_vy
  ld a, #RIDE_MAX_VY
  jr update_ride_vy

continue_checking_vy:
  cp #RIDE_MIN_VY
  jp p, update_ride_vy
  ld a, #RIDE_MIN_VY

update_ride_vy:
  ld ENTITY_BODY_VY (ix), a

  ld l, ENTITY_BODY_VX_LO (ix)
  ld h, ENTITY_BODY_VX_HI (ix)
  add hl, bc

  ld a, h
  cp #RIDE_MAX_VX
  jp m, continue_checking_vx
  ld h, #RIDE_MAX_VX
  jr update_ride_vx

continue_checking_vx:
  cp #RIDE_MIN_VX
  jp p, update_ride_vx
  ld h, #RIDE_MIN_VX

update_ride_vx:
  ld ENTITY_BODY_VX_LO (ix), l
  ld ENTITY_BODY_VX_HI (ix), h

  ret

_action_ride::
  push de
    call _player_ride
  pop de
  jp _ai_checkConditions


_action_rideWaiting::
  push de
    call _player_ride
  pop de
  jp _action_wait


_action_run::
  call _action_applyVelocityControls

_action_idleRunning::
  ; keep entity tied to the floor (until there is too much slope)
  ld ENTITY_BODY_VY (ix), #4

  jp _ai_checkConditions


_action_jump::
  call _action_applyVelocityControls

  ld a, #PLAYER_JUMP_ACCELERATION
  ld hl, #_player_status
  bit #PLAYER_FURY_BIT, (hl)
  jr z, continue_jumping

  ld a, #PLAYER_JUMP_FAST_ACCELERATION

continue_jumping:
  ; apply acceleration
  ld ENTITY_BODY_VY (ix), a
  ; ensuring this affects instantly to velocity
  add ENTITY_POSITION_Y (ix)
  dec a             ; add acceleration -1
  ld ENTITY_POSITION_Y (ix), a

  ; and remove floor flags preventing fall state to be truncated
  ld a, ENTITY_BODY_FLAGS (ix)
  and #~(BODY_ON_FLOOR | BODY_IN_WATER)
  ld ENTITY_BODY_FLAGS (ix), a

  jp _ai_nextBehavior


_action_create::
  ld a, (de)
  ld l, a
  inc de
  ld a, (de)
  ld h, a       ; HL <- factory address
  inc de

  ; ensure there is an available entity
  call _entities_checkAvailability_asm
  jp z, _ai_nextBehavior

  push ix
  push de
    ld e, ENTITY_POSITION_X_LO (ix)
    ld d, ENTITY_POSITION_X_HI (ix)
    ld b, ENTITY_POSITION_Y (ix)
    call ___sdcc_call_hl
  pop de
  pop ix

  jp _ai_nextBehavior


_action_call_0::
  ld a, (de)
  ld l, a
  inc de
  ld a, (de)
  ld h, a       ; HL <- call address
  inc de

  push de
  push ix
    call ___sdcc_call_hl
  pop ix
  pop de

  jp _ai_nextBehavior


_action_stopCamera::
  ld hl, #0
  ld (#_player_velocity), hl
  jp _ai_nextBehavior


_action_setControl::
  push de
    call _controls_getKeyPressed
                ; HL <- key id
  pop de
  ex de, hl     ; DE <- key id
                ; HL <- behavior address
  ld c, (hl)    ; C <- index
  inc hl        ; HL <- next behavior address
  push hl
    call _controls_set_asm
  pop de        ; DE <- next behavior address
  jp _ai_nextBehavior


_action_clear::
  push de
  push ix
    call _renderer_addStain
  pop ix
  pop de
  ; and then die

_action_die::
  ; TODO this will eventualy be just: jp _entities_destroy_asm
  ld__e_ixl
  ld__d_ixh
  ex de, hl     ; HL <- entity address
  jp _entities_destroy


_action_debug::
  BREAKPOINT
  jp _ai_checkConditions


_action_sequenceReturn::
  .dw #0

_action_sequence::
  ld a, (de)
  ld l, a
  inc de
  ld a, (de)
  ld h, a       ; HL <- sequence address
  inc de
  ex de, hl
  ld (#_action_sequenceReturn), hl
  call _ai_setBehavior_asm
  jp _ai_nextBehavior


_action_sequenceEnd::
  ld hl, (#_action_sequenceReturn)
  ex de, hl
  call _ai_setBehavior_asm
  jp _ai_nextBehavior


_action_relativeVx::
  .dw #0
_action_setWaitRelativeVx::
; BREAKPOINT
  ld a, (de)
  ld l, a
  inc de
  ld a, (de)
  ld h, a
  inc de
  ld (#_action_relativeVx), hl
  jp _ai_nextBehavior


_action_waitRelativeVx::
  ld hl, (#_action_relativeVx)
  ld c, l
  ld b, h
  ld hl, (#_player_velocity)
  add hl, bc
  ld ENTITY_BODY_VX_LO (ix), l
  ld ENTITY_BODY_VX_HI (ix), h
  ; jp _action_wait


_action_wait::
  dec ENTITY_TIMER_VALUE (ix)
  ; jp _ai_checkConditions


_action_idle::
  jp _ai_checkConditions
  
