;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "renderer.h.s"
.include "player.h.s"
.include "lang/macros.h.s"
.include "lang/debug.h.s"
.include "cpc/controls.h.s"
.include "entities/entity.h.s"

.module ai_conditions

.globl cpct_isKeyPressed_asm

.macro INVERT_AND_RETURN ?do_invert
  jr z, do_invert
  xor a
  ret
do_invert:
  inc a
  ret
.endm

.macro CHECK_KEY code
  push de
    ld l, #code
    call _controls_isPressed
    or a
  pop de
  INVERT_AND_RETURN
  ; TODO It'd be nice removing this invert here avoiding jumps...
.endm


_condition_timeout::
  ld a, ENTITY_TIMER_VALUE (ix)
  or a
  ret

_condition_true::
  xor a
  ret

_condition_onFloor::
  ld a, ENTITY_BODY_FLAGS (ix)
  cpl
  bit BODY_ON_TERRAIN_BIT, a
  ret z
  bit BODY_ON_PLATFORM_BIT, a
  ret

_condition_inWater::
  ld a, ENTITY_BODY_FLAGS (ix)
  cpl
  bit BODY_IN_WATER_BIT, a
  ret

_condition_notOnFloor::
  ld a, ENTITY_BODY_FLAGS (ix)
  and #BODY_ON_FLOOR
  ret

_condition_playerFury::
  ld a, (#_player_status)
  cpl
  bit #PLAYER_FURY_BIT, a
  ret

_condition_notPlayerFury::
  ld hl, #_player_status
  bit #PLAYER_FURY_BIT, (hl)
  ret

_condition_playerCollision::
  ld a, ENTITY_COLLIDABLE_STATUS (ix)
  cpl
  bit COLLIDABLE_PLAYER_BIT, a
  ret

_condition_furyCollision::
  ld a, ENTITY_COLLIDABLE_STATUS (ix)
  cpl
  bit COLLIDABLE_FURY_BIT, a
  ret

_condition_alienCollision::
  ld a, ENTITY_COLLIDABLE_STATUS (ix)
  cpl
  bit COLLIDABLE_ALIEN_BIT, a
  ret

_condition_enemyCollision::
  ld a, ENTITY_COLLIDABLE_STATUS (ix)
  cpl
  bit COLLIDABLE_ENEMY_BIT, a
  ret

_condition_sliding::
  CHECK_KEY CONTROLS_P1_SLIDE

_condition_jumping::
  CHECK_KEY CONTROLS_P1_JUMP

_condition_actionWithMojo::
  ld a, (#_player_mojo)
  or a
  jr nz, _condition_action
  inc a         ; reset Z flag
  ret

_condition_action::
  CHECK_KEY CONTROLS_P1_ACTION

_condition_notInScreen::
  ld c, ENTITY_DRAWING_BOUNDS_LEFT_LO (ix)
  ld b, ENTITY_DRAWING_BOUNDS_LEFT_HI (ix)
  or a
	ld hl, #_renderer_left
	ld a, c
	sub (hl)
	ld a, b
	inc hl
	sbc (hl)
	ld a, #0x00
	rla
	xor #0x01
  ret

_condition_notInScreenRight::
  ld c, ENTITY_DRAWING_BOUNDS_RIGHT_LO (ix)
  ld b, ENTITY_DRAWING_BOUNDS_RIGHT_HI (ix)
  or a
	ld hl, #_renderer_right
	ld a, c
	sub (hl)
	ld a, b
	inc hl
	sbc (hl)
	; ld a, #0x00
	rla
	and #0x01
  ret

.equ NEAR_LEFT_DISTANCE, 28
_condition_nearScreenLeft::
  ld b, #NEAR_LEFT_DISTANCE
check_with_renderer_left:
  ld a, ENTITY_DRAWING_BOUNDS_LEFT_LO (ix)
  sub b
  ld l, a
  ld a, ENTITY_DRAWING_BOUNDS_LEFT_HI (ix)
  ld h, #0
  sbc h
  ld h, a         ; HL <- entity.bounds.left - LEFT_DISTANCE

  ld bc, (#_renderer_left)
  xor a
  sbc hl, bc
	rla
	xor #0x01

  ret

.equ ALMOST_IN_LEFT_DISTANCE, 8
_condition_almostInScreenLeft::
  ld b, #ALMOST_IN_LEFT_DISTANCE
  jp check_with_renderer_left

.equ RIGHT_DISTANCE, 28
_condition_nearScreenRight::
  ld b, #RIGHT_DISTANCE
check_with_renderer_right:
  ld a, ENTITY_DRAWING_BOUNDS_RIGHT_LO (ix)
  add b
  ld l, a
  ld a, ENTITY_DRAWING_BOUNDS_RIGHT_HI (ix)
  adc #0
  ld h, a         ; HL <- entity.bounds.left + RIGHT_DISTANCE

  ld bc, (#_renderer_right)
  xor a
  sbc hl, bc
  rla
  and #0x01

  ret

.equ ALMOST_IN_RIGHT_DISTANCE, 8
_condition_almostInScreenRight::
  ld b, #ALMOST_IN_RIGHT_DISTANCE
  jp check_with_renderer_right

.equ NEAREST_PLAYER_DISTANCE, #15
_condition_nearestToPlayer::
  ld hl, #player_distance_address
  ld (hl), #NEAREST_PLAYER_DISTANCE
  jp check_player_distance

.equ NEAR_PLAYER_DISTANCE, #30
_condition_nearPlayer::
  ld hl, #player_distance_address
  ld (hl), #NEAR_PLAYER_DISTANCE
  ; jp check_player_distance

check_player_distance:
  ld l, ENTITY_POSITION_X_LO (ix)
  ld h, ENTITY_POSITION_X_HI (ix)
                  ; HL <- slug position
  push iy
    ld iy, (#_player_entity)
    ld c, ENTITY_POSITION_X_LO (iy)
    ld b, ENTITY_POSITION_X_HI (iy)
                  ; BC <- player position
  pop iy
  or a            ; reset C
  sbc hl, bc      ; HL <- slug->position.x - player->position.x
  ld a, l
.equ player_distance_address, . + 1
  cp #NEAR_PLAYER_DISTANCE
	ld a, #0x00
	rla
	xor #0x01
  ret

_condition_isSlow::
  ld a, ENTITY_BODY_VX_LO (ix)
  or a
  ret

_condition_atCruiseAltitude::
  ld a, ENTITY_BODY_VY (ix)
  neg
  ld b, a
  ld a, ENTITY_DRAWING_BOUNDS_TOP (ix)
  sub b
	ld a, #0x00
	rla
	xor #0x01
  ret

_condition_playPressed::
  ld hl, #Key_1
  push de
    call cpct_isKeyPressed_asm
  pop de
  INVERT_AND_RETURN

_condition_redefinePressed::
  ld hl, #Key_2
  push de
    call cpct_isKeyPressed_asm
  pop de
  INVERT_AND_RETURN

_condition_anyKeyPressed::
  call _condition_notAnyKeyPressed
  INVERT_AND_RETURN

_condition_notAnyKeyPressed::
  push de
    call _controls_getKeyPressed
  pop de
  ld a, h
  or l
  ret

_condition_getUnusedControl::
  push de
    call _controls_getKeyPressed
  pop de
  ld a, h
  or l
  jr z, invert_and_return
  push de
    call _controls_isUsed
  pop de
  ld a, l
  or a
  ret
invert_and_return:
  INVERT_AND_RETURN
