;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "alien.h.s"
.include "checkpoint.h.s"
.include "hud.h.s"
.include "entities.h.s"
.include "player.h.s"
.include "ai.h.s"
.include "lang/fixed.h.s"
.include "lang/debug.h.s"
.include "entities/factories.h.s"
.include "entities/entity.h.s"
.include "stages/game.h.s"
.include "data/level.h.s"
.include "data/game_assets.h.s"

.module alien

; .equ LEVEL_VELOCITY,  (FIXED_ONE >> 7)
.equ APPEAR_DISTANCE, (HUD_CHARACTERS_WIDTH_IN_BYTES - 1)

.globl ___sdcc_call_hl

.globl _alienEscape_behavior

; u8 alien_levelPosition
_alien_levelPosition::
  .db #0
; u16 alien_levelBehavior
_alien_levelBehavior::
  .dw #0
; fixed alien_position
_alien_position::
  .dw #0
; void (*alien_state)()
_alien_state::
  .dw #0
; fixed alien_escapePosition;
_alien_escapePosition::
  .dw #0

; Entity* alien_entity
_alien_entity::
  .dw #0
; Entity* alien_recharge
_alien_recharge::
  .dw #0

_alien_clear::
  pop  af       ;; AF <- return address
  pop  iy       ;; IY <- level address
  pop  bc       ;; C <- game mode
  push af       ;; Put returning address in the stack again

  ; position = checkpoint_alienPosition
  CHECKPOINT_GET_ALIEN_POSITION
  ; position == 0?
  ld a, l
  or h
  jr nz, clear_position

  ; position = fixed_fromInt(level->alienPosition)
  ld l, #0
  ld h, LEVEL_ALIEN_POSITION (iy)
  ; ld hl, #0x3200

clear_position:
  ; alien_position = position
  ld (#_alien_position), hl

  ; alien_levelPosition = fixed_toInt(alien_position)
  ld a, h
  ld (#_alien_levelPosition), a

  ; alien_recharge = NULL
  ld hl, #0
  ld (#_alien_recharge), hl

  ; alien_escapePosition = (level->ending << 1)
  ld l, LEVEL_ENDING_LO (iy)
  ld h, LEVEL_ENDING_HI (iy)
  add hl, hl
  ld (#_alien_escapePosition), hl

  ; alien_aheadVelocity = level->alienVelocity
  ld a, LEVEL_ALIEN_VELOCITY (iy)
  ld (#alien_aheadVelocity), a

  ; alien_behavior = level->alienBehavior
  ld l, LEVEL_ALIEN_BEHAVIOR_LO (iy)
  ld h, LEVEL_ALIEN_BEHAVIOR_HI (iy)
  ld (#_alien_levelBehavior), hl

  ; alien_state = gameMode == GAME_FAILING_MODE?
  ;   alien_winning: checkpoint_alienState
  ld hl, #_alien_winning
  ld a, #GAME_FAILING_MODE
  cp c
  jr z, clear_alien_state

  CHECKPOINT_GET_ALIEN_STATE

clear_alien_state:
  ld (#_alien_state), hl

  ; alien_entity = alien_state == alien_withPlayer?
  ;   alien_create(): NULL
  ld bc, #_alien_withPlayer
  xor a
  sbc hl, bc

  ld hl, #0
  jr nz, clear_alien_entity

  call _alien_create
clear_alien_entity:
  ld (#_alien_entity), hl
  ret


_alien_step::
  ld hl, (#_alien_state)
  jp ___sdcc_call_hl


_alien_hit::
  ; alien_state = alien_ko
  ld hl, #_alien_ko
  ld (#_alien_state), hl
  ; remove alien from hud
  jp _hud_removeAlien


_alien_createRecharge::
  ; ensure there is an available entity
  call _entities_checkAvailability_asm
  ret z

  call _recharge_create
  ld (#_alien_recharge), hl
  ret


_alien_destroyRecharge::
  ; alien_recharge == NULL?
  ld hl, (#_alien_recharge)
  ld a, l
  or h
  ret z

  ; destroy entity
  call _entities_destroy
  ; alien_recharge = NULL
  ld hl, #0
  ld (#_alien_recharge), hl
  ret


_alien_aheadPlayer::
  ; alien_position += LEVEL_VELOCITY
  ld hl, (#_alien_position)
.equ alien_aheadVelocity, . + 1
  ld bc, #0
  add hl, bc
  ld (_alien_position), hl

  ; alien_levelPosition = fixed_toInt(alien_position)
  ld a, (#_alien_position + 1)
  ld (#_alien_levelPosition), a

  ; alien_levelPosition - player_levelPosition < APPEAR_DISTANCE?
  ; alien_levelPosition < APPEAR_DISTANCE + player_levelPosition?
  ld c, a
  ld a, (#_player_levelPosition)
  add #APPEAR_DISTANCE
  cp c
  ret c

  ; alien_entity = alien_create()
  call _alien_create
  ld (#_alien_entity), hl
  ; alien_state = alien_withPlayer
  ld hl, #_alien_withPlayer
  ld (#_alien_state), hl
  ret


_alien_withPlayer::
  ; alien_levelPosition = player_levelPosition + HUD_CHARACTERS_WIDTH_IN_BYTES
  ld a, (#_player_levelPosition)
  add #HUD_CHARACTERS_WIDTH_IN_BYTES
  ld (#_alien_levelPosition), a
  ; alien_position = fixed_fromInt(alien_levelPosition)
  ld h, a
  ld l, #0
  ld (#_alien_position), hl

  ; alien_entity->body.flags & BODY_IN_WATER?
  ld iy, (#_alien_entity)
  bit BODY_IN_WATER_BIT, ENTITY_BODY_FLAGS (iy)
  jr z, update_recharge

  ; alien_entity->body.vy = -10
  ld a, ENTITY_BODY_VY (iy)
  add #-10
  ld ENTITY_BODY_VY (iy), a
  ; alien_entity->body.vx = 1 << 8
  ld ENTITY_BODY_VX_HI (iy), #1
  ld ENTITY_BODY_VX_LO (iy), #0

update_recharge:
  ; alien_recharge?
  ld hl, (#_alien_recharge)
  ld a, l
  or h
  jr z, check_escape_position

  ld l, ENTITY_POSITION_X_LO (iy)
  ld h, ENTITY_POSITION_X_HI (iy)
  ld c, ENTITY_POSITION_Y (iy)
  ; alien_recharge->position.x = alien_entity->position.x
  ld iy, (#_alien_recharge)
  ld ENTITY_POSITION_X_LO (iy), l
  ld ENTITY_POSITION_X_HI (iy), h
  ; alien_recharge->position.y = alien_entity->position.y
  ld ENTITY_POSITION_Y (iy), c

check_escape_position:
  ; alien_position > alien_escapePosition
  ld bc, (#_alien_escapePosition)
  ld hl, (#_alien_position)
  xor a
  sbc hl, bc
  ret c

  jp _alien_escape


_alien_ko::
  ret


_alien_winning::
  ret


_alien_escape::
  ; alien_state = alien_winning
  ld hl, #_alien_winning
  ld (_alien_state), hl
  ; destroy recharge if needed
  call _alien_destroyRecharge

  ; ai_setBehavior(alien_entity, (u16)&alienEscape_behavior)
  push ix
  ld ix, (#_alien_entity)
  ld de, #_alienEscape_behavior
  call _ai_setBehavior_asm
  pop ix
  ret
