;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "background.h.s"
.include "renderer.h.s"
.include "lang/debug.h.s"
.include "data/game_assets.h.s"

.module background

.globl _background_stars
.globl _background_planets
.globl _background_bushes

_background_index::
  .dw 0
_background_column::
  .dw 0

_background_clear::
  ld hl, (#_renderer_left)
  push hl
  srl h
  rr l          ; HL <- renderer_left >> 1
  ld (#_background_index), hl

  pop de        ; DE <- xInBytes = renderer_left
  ld b, #0      ; B <- y = 0
  call _background_getTilePointer_asm
  ld (#_background_column), hl
  ret


_background_step::
  ; u16 index = renderer_right >> 1;
  ld bc, (_renderer_right)
  srl b
  rr c          ; BC <- renderer_right >> 1

  ld hl, (#_background_index)
                ; HL <- background_index

  ; while (background_index < index) {
step_loop:
  ld a, l
  sub c
  ld a, h
  sbc b
  ret nc

  push bc
  push hl
    call _background_stars
    call _background_planets
    call _background_bushes

    ; background_column++
    ld hl, (#_background_column)
    ld bc, #RENDERER_GAME_HEIGHT_IN_TILES
    add hl, bc
    ENSURE_HL_IN_BUFFER
    ld (#_background_column), hl
  pop hl
  pop bc

  ; background_index++;
  inc hl
  ld (#_background_index), hl
  jr step_loop


; u8* background_getTilePointer(u16 xInBytes, u8 y) {
;   u16 address = ((xInBytes >> 1) * RENDERER_GAME_HEIGHT_IN_TILES + (y >> 3)) & 0x3ff;
;   return (u8*)(TILE_BUFFER_ADDRESS + address);
; }
; TODO add _background_getTilePointer_asm to call with values in registers
; TODO use precalculated mult table?
; _background_getTilePointer::
;   pop hl      ;; HL <- return address
;   pop de      ;; DE <- xInBytes
;   dec sp
;   pop bc      ;; B = y
;   push hl     ;; Put returning address in the stack again

; IN:
;   DE <- xInBytes
;   B <- y
; OUT:
;   HL <- tile buffer address
; DESTROYS:
;   AF, BC, DE
_background_getTilePointer_asm::
  ; u16 address = ((xInBytes >> 1) * RENDERER_GAME_HEIGHT_IN_TILES + (y >> 3)) & 0x3ff;
	srl	d
	rr	e         ; DE <- xInBytes >> 1

  ld l, e
  ld h, d       ; HL <- xInBytes >> 1
	add	hl, hl
	add	hl, hl
	add	hl, de
	add	hl, hl
	add	hl, hl    ; HL <- ((xInBytes >> 1) * 20

  ld c, b
	srl	c
	srl	c
	srl	c
	ld	b, #0x00
	add	hl, bc    ; HL <- HL + (y >> 3)

	ld	a, h
	and	a, #0x03
	ld	h, a      ; HL <- HL & 0x3ff

  ; return (u8*)(TILE_BUFFER_ADDRESS + address);
	ld	bc, #TILE_BUFFER_ADDRESS
	add	hl, bc

	ret
