;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "background.h.s"
.include "terrain.h.s"
.include "water.h.s"
.include "lang/macros.h.s"
.include "lang/debug.h.s"
.include "data/game_assets.h.s"

.module background

.equ MIN_WAITING_COLUMNS, #2
.equ WAITING_COLUMNS, #16

.globl cpct_getRandom_mxor_u8_asm

.macro DATA item
  .db ('item'_0_INDEX << 1) + 1
  .db 'item'_HEIGHT_IN_TILES
  .db 'item'_WIDTH_IN_TILES - 1
.endm

_background_bushes_tiles:
  DATA BUSH
  DATA BUSH
  DATA MEDIUM_WEED
  DATA HIGH_WEED

.macro SET_STATE state_step, next_state_init
  ld a, #next_state_init - #next_state_jump - 1
  ld (#next_state_jump), a
  ld a, #state_step - #state_step_jump - 1
  ld (#state_step_jump), a
.endm

_background_bushes::
.equ counter, . + 1
  ld a, #0
  or a
.equ next_state_jump, . + 1
  jr z, start_waiting
  dec a
  ld (#counter), a
.equ state_step_jump, . + 1
  jr draw

start_waiting:
  SET_STATE wait, start_drawing

  call cpct_getRandom_mxor_u8_asm
  ld a, l
  and #WAITING_COLUMNS - 1
  add #MIN_WAITING_COLUMNS
  ld (#counter), a

wait:
  ret

start_drawing:
  ld hl, (#_background_index)
  add hl, hl
  call _water_getHeight
  ld a, l
  or a
  jr nz, start_waiting
                  ; wait if there is water!

  SET_STATE draw, start_waiting
  call cpct_getRandom_mxor_u8_asm
  ld a, l
  and #0x03
  ld b, a
  add a           ; A <- A * 2
  add b           ; A <- A * 3
  ld hl, #_background_bushes_tiles
  ld d, #0
  ld e, a
  add hl, de      ; HL <- current data position

  ld de, #tile
  ldi             ; set base tile
  ld a, (hl)
  ld (de), a      ; set height
  inc hl

  push af
    ld a, (hl)
    ld (#counter), a
                  ; set counter as with - 1

    ld hl, (#_background_index)
    inc hl
    add hl, hl
    call _terrain_getHeight
    ld a, l
    rra
    rra
    rra
    and #0x1f
  pop bc
  sub b
  inc a
  ld (#y), a

draw:
  ld hl, (#_background_column)
.equ y, . + 1
  ld a, #0
  add_hl_a

.equ height, . + 2
.equ tile, . + 1
  ld bc, #0

loop:
  ENSURE_HL_IN_BUFFER
  ld (hl), c
  inc hl
  inc c
  inc c
  djnz loop

  ld hl, #tile
  ld (hl), c

  ret

