;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "collision.h.s"
.include "terrain.h.s"
.include "platform.h.s"
.include "water.h.s"
.include "entities.h.s"
.include "entities/entity.h.s"
.include "lang/macros.h.s"
.include "lang/debug.h.s"
.include "lang/pvector.h.s"

.module collision

_collision_collidables::
  PVECTOR ENTITY_COUNT


_collision_clear::
  ld hl, #_collision_collidables
  jp _pvector_init


; ; IN:
; ;   HL <- entity address
; _collision_addEntity::
;   push ix
;   push iy
;
;   ex de, hl
;   ld__ixl_e
;   ld__ixh_d       ; IX <- entity address
;
;   call _collision_addEntity_asm
;
;   pop iy
;   pop ix
;   ret

; IN:
;   IX <- entity address
; DESTROYS:
;   AF, BC, DE, HL, IY
_collision_addEntity_asm::
  set COLLIDABLE_DIRTY_BIT, ENTITY_COLLIDABLE_STATUS (ix)
                  ; set dirty bit to force first update
  call _collision_updateCollidable

  ld de, #_collision_collidables
  ld hl, #_collision_sorter
  jp _pvector_addSorted_ir_asm


; IN:
;   HL <- entity address
_collision_removeEntity::
  ld c, l
  ld b, h         ; BC <- entity address
  ld hl, #_collision_collidables
  jp _pvector_remove_asm


_collision_step::
  push ix
  push iy

  ; Ensure collidables are sorted
  ld de, #_collision_collidables
  ld hl, #_collision_sorter
  call _pvector_sort_ir_asm

  ; Check if drawings changed, updating collidable bounds if needed
  ld de, #_collision_collidables
  ld hl, #_collision_updateCollidable
  call _pvector_apply_ix_asm

  ; Check collisions between collidables
  call _collision_checkCollisions

  pop iy
  pop ix
  ret


_collision_checkCollisions::
  ld de, #_collision_collidables + PVECTOR_ITEMS
                  ; DE <- first collidable pointer address
  
outer_loop:
  ld a, (de)
  ld c, a
  inc de
  ld a, (de)
  ld b, a         ; BC <- first collidable

  or c
  ret z

  ld__ixl_c
  ld__ixh_b       ; IX <- current collidable address
  inc de          ; DE <- next collidable pointer address

  ld l, e
  ld h, d         ; HL <- next collidable pointer address

inner_loop:
  ld c, (hl)
  inc hl
  ld b, (hl)

  ld a, b
  or c
  jr z, outer_loop

  ld__iyl_c
  ld__iyh_b       ; IY <- next collidable address
  inc hl          ; HL <- next next collidable pointer address

  ; next->bounds.left < current->bounds.right
  ld a, ENTITY_COLLIDABLE_BOUNDS_RIGHT_LO (ix)
  sub ENTITY_COLLIDABLE_BOUNDS_LEFT_LO (iy)
  ld a, ENTITY_COLLIDABLE_BOUNDS_RIGHT_HI (ix)
  sbc ENTITY_COLLIDABLE_BOUNDS_LEFT_HI (iy)
  jr c, outer_loop

  ; overlaps vertically?
  ; current->bounds.bottom > next->bounds.top?
  ld a, ENTITY_COLLIDABLE_BOUNDS_TOP (iy)
  cp ENTITY_COLLIDABLE_BOUNDS_BOTTOM (ix)
  jr nc, inner_loop
  ; current->bounds.top < next->bounds.bottom?
  ld a, ENTITY_COLLIDABLE_BOUNDS_BOTTOM (iy)
  cp ENTITY_COLLIDABLE_BOUNDS_TOP (ix)
  jr c, inner_loop

  ; current->status |= next->type
  ld a, ENTITY_COLLIDABLE_STATUS (ix)
  or ENTITY_COLLIDABLE_TYPE (iy)
  ld ENTITY_COLLIDABLE_STATUS (ix), a
  ; next->status |= current->type
  ld a, ENTITY_COLLIDABLE_STATUS (iy)
  or ENTITY_COLLIDABLE_TYPE (ix)
  ld ENTITY_COLLIDABLE_STATUS (iy), a

  jr inner_loop


; _collision_check::
;   pop de      ;; DE <- return address
;   pop hl      ;; HL <- xInBytes
;   dec sp
;   pop bc      ;; B = y
;   push de     ;; Put returning address in the stack again

; IN:
;   HL <- xInBytes
;   B <- y
; OUT:
;   L, C <- collision terrain/platform/water bits
; DESTROYS:
;   AF, BC
_collision_check_asm::
  ld c, #0        ; C <- result

  ; Check with terrain
  push hl
    call _terrain_getHeight
                  ; L <- terrain height
    ; y >= terrain? === y > terrain - 1?
    ld a, l
    dec a
    cp b
  pop hl
  jr nc, check_with_platform

  set #COLLISION_TERRAIN_BIT, c

  ; Check with platform
check_with_platform:
  push hl
; BREAKPOINT
    call _platform_getHeight
                  ; L <- platform height

    ; y >= platform? === y > platform - 1?
    ld a, l
    dec a
    cp b
    ld a, l       ; restore A (not changing flags)
  pop hl
  jr nc, check_with_water

  ; y < platform + 16?
  add #16
  cp b
  jr c, check_with_water

  set #COLLISION_PLATFORM_BIT, c

  ; Check with water
check_with_water:
  call _water_getHeight
                  ; L <- water height
  ; y >= water? === y > water - 1?
  ld a, l
  dec a
  cp b
  jr nc, end_checking

  set #COLLISION_WATER_BIT, c

end_checking:
  ld l, c
  ret


; IX <- first entity address
; IY <- second entity address
_collision_sorter::
  ld a, ENTITY_COLLIDABLE_BOUNDS_LEFT_LO (ix)
  sub ENTITY_COLLIDABLE_BOUNDS_LEFT_LO (iy)
  ld a, ENTITY_COLLIDABLE_BOUNDS_LEFT_HI (ix)
  sbc ENTITY_COLLIDABLE_BOUNDS_LEFT_HI (iy)
  ret


; IX <- entity address
_collision_updateCollidable::
  ld b, ENTITY_COLLIDABLE_STATUS (ix)
                  ; B <- store status

  xor a
  ld ENTITY_COLLIDABLE_STATUS (ix), a
                  ; reset status

  bit COLLIDABLE_DIRTY_BIT, b
  ret z           ; prevent updating if not modified

  ld e, ENTITY_ANIMATION_STEP_LO (ix)
  ld d, ENTITY_ANIMATION_STEP_HI (ix)
  ld__iyl_e
  ld__iyh_d       ; IY <- step address

  ld l, ENTITY_POSITION_X_LO (ix)
  ld h, ENTITY_POSITION_X_HI (ix)
  ld e, l
  ld d, h         ; DE <- entity.position.x
  ld a, STEP_CCX (iy)
  ld b, a         ; B <- step.ccx
  sub_hl_a        ; HL <- left = entity.position.x - step.ccx
  ld ENTITY_COLLIDABLE_BOUNDS_LEFT_LO (ix), l
  ld ENTITY_COLLIDABLE_BOUNDS_LEFT_HI (ix), h

  ld a, b
  add_de_a        ; DE <- right = entity.position.x + step.ccx
  ld ENTITY_COLLIDABLE_BOUNDS_RIGHT_LO (ix), e
  ld ENTITY_COLLIDABLE_BOUNDS_RIGHT_HI (ix), d

  ld a, ENTITY_POSITION_Y (ix)
                  ; A <- bottom = entity.position.y
  ld ENTITY_COLLIDABLE_BOUNDS_BOTTOM (ix), a
  sub STEP_CCY (iy)
                  ; A <- top = entity.position.y - step.ccy
  ld ENTITY_COLLIDABLE_BOUNDS_TOP (ix), a

  ret
