;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "entities.h.s"
.include "renderer.h.s"
.include "collision.h.s"
.include "animator.h.s"
.include "ai.h.s"
.include "entities/factories.h.s"
.include "entities/entity.h.s"
.include "data/level.h.s"
.include "lang/macros.h.s"
.include "lang/debug.h.s"

.module entities

.globl cpct_memset_asm
.globl ___sdcc_call_hl

_entities::
  .ds #ENTITY_COUNT * ENTITY_SIZE
_entities_availableCounter:
  .db #0

_entities_counter::
  .db #0
_entities_level::
  .dw #0

.macro GET_COUNTER
  rra
  rra
  and #0x3c
  add #4          ; A <- counter = ((next >> 2) & 0xfc) + 4
.endm

.macro GET_TYPE
  and #0x0f       ; A <- type = next & 0x0f
.endm

; IN:
;   HL <- level address
_entities_clear::
  push ix

  ; entities_level = level->entities
  ld de, #LEVEL_ENTITIES
  add hl, de
  ld e, (hl)
  inc hl
  ld d, (hl)      ; DE <- entities_level

  ; entities_x = renderer_left
  ld hl, (#_renderer_left)
  ld (#entities_x), hl

  ; find level start address
  ld b, #0
find_start_loop:
  ld a, (de)      ; A <- next
  GET_COUNTER     ; A <- counter
  ld c, a         ; BC <- counter
  or a
  sbc hl, bc      ; HL -= counter
  jr c, found_start
  ld a, (de)      ; A <- next
  GET_TYPE
  jr nz, skip_entity
  inc de          ; skip y if entity type == 0
skip_entity:
  inc de          ; skip counter
  jr find_start_loop

found_start:
  ; get first data from level
  add hl, bc
  ld a, c         ; A <- counter
  sub l
  ld (#_entities_counter), a
  ld a, (de)
  inc de
  and #0x0f
  ld (#entities_type), a
  ld (#_entities_level), de

  ; entities_availableCounter = ENTITY_COUNT
  ld a, #ENTITY_COUNT
  ld (#_entities_availableCounter), a
  ; entities_each(entity_clear)
  ld hl, #_entity_clear_ix
  call _entities_each_asm
  
  pop ix
  ret


_entities_step::
.equ entities_x, . + 1
  ld hl, #0
  ; while (entities_x < renderer_right)
step_while:
  ld bc, (#_renderer_right)
  ld a, l
  sub c
  ld a, h
  sbc b
  ret nc

  ; --entities_counter == 0?
  ld hl, #_entities_counter
  dec (hl)
  jr nz, continue_step

.equ entities_type, . + 1
  ld a, #0
                    ; A <- entity type

  ; entities_type == 0?
  or a
  jr nz, create_entity

  ; then get Y from level data
  ld hl, (#_entities_level)
  ld b, (hl)        ; B <- entity Y (from level)
  inc hl
  ld (#_entities_level), hl

create_entity:
  ld de, (#entities_x)
                    ; DE <- entity xInBytes
                    ; B <- undefined or entity Y
  ld c, a           ; C <- entity type
  push ix
    call _entities_checkAvailability_asm
    call nz, _factories_create_asm
  pop ix

  call _entities_next_from_level

continue_step:
  ; entities_x++
  ld hl, (#entities_x)
  inc hl
  ld (#entities_x), hl
  jr step_while


_entities_next_from_level::
  ; next = entities_level++
  ld hl, (#_entities_level)
  ld b, (hl)          ; B <- next
  inc hl
  ld (#_entities_level), hl

  ; entities_counter = ((next >> 2) & 0xfc) + 4
  ld a, b
  GET_COUNTER
  ld (#_entities_counter), a

  ; entities_type = next & 0x0f
  ld a, b
  GET_TYPE
  ld (#entities_type), a

  ret


_entities_add_asm::
  ld b, #ENTITY_COUNT
  ld ix, #_entities
  ld de, #ENTITY_SIZE
  xor a
add_loop:
  ; entity->components == 0?
  cp ENTITY_COMPONENTS (ix)
  ret z

  ; entity++
  add ix, de
  djnz add_loop

  ; TODO No available entities!
  BREAKPOINT
  ld ix, #0
  ret


_entities_destroy::
  ; TODO maybe using IX from all of them? even when calling?
  push hl
    call _renderer_removeEntity
  pop hl

  push hl
    call _collision_removeEntity

  ; entities_availableCounter++
    ld hl, #_entities_availableCounter
    inc (hl)
  pop hl

  jp _entity_clear_hl


_entity_clear_hl::
  ld (hl), #0   ; TODO using no index is not cool...
                ; TODO this function should be unused and removed soon...
  ret


_entity_clear_ix::
  ld ENTITY_COMPONENTS (ix), #0
  ret


; OUT:
;   Z flag if there is no availability
; DESTROYS:
;   AF
_entities_checkAvailability_asm::
  ld a, (#_entities_availableCounter)
  or a
  ret


; TODO this will be removed when not needed from C
; _entities_checkAvailability::
;   ld l, #1
;   ld a, (#_entities_availableCounter)
;   or a
;   ret nz
;   dec l
;   ret


; IN:
;   HL <- definition
;   DE <- x
;   B <- y
; OUT:
;   IX <- new entity address
; DESTROYS:
;   AF, BC, DE, HL
_entities_create_asm::
  push iy
  push bc
    push de
      push hl

        call _entities_add_asm

  ; entities_availableCounter--
        ld hl, #_entities_availableCounter
        dec (hl)

        xor a
        ld bc, #ENTITY_SIZE
        ld__e_ixl
        ld__d_ixh
        call cpct_memset_asm
                  ; TODO don't put all entity bytes to zero, just the needed
                  ; ones!

  ; u8 COMPONENTS
      pop hl
      ld a, (hl)
      inc hl
      ld ENTITY_COMPONENTS (ix), a

    pop de
  ; position.x
    ld ENTITY_POSITION_X_LO (ix), e
    ld ENTITY_POSITION_X_HI (ix), d
    ld ENTITY_BODY_IX_LO (ix), e
    ld ENTITY_BODY_IX_HI (ix), d
  pop af
  ; position.y
  ld ENTITY_POSITION_Y (ix), a

  ; animation address
  ld e, (hl)
  inc hl
  ld d, (hl)
  inc hl
  push hl
  ; [if components & ANIMATION]
    call _animator_setAnimation_asm
  ; pop hl

  ; [if components & DRAWING]
  ; push hl
    call _renderer_addEntity_asm
  pop hl

  ; behavior address
  ; [if components & BEHAVIOR]
  ld e, (hl)
  inc hl
  ld d, (hl)
  call _ai_setBehavior_asm

  ; TODO if has collidable component?
  call _collision_addEntity_asm

  pop iy
  ret


; IN:
;   HL <- function to call with IX as entity address
; DESTROY:
;   All
_entities_each_asm::
  ld ix, #_entities
  ld b, #ENTITY_COUNT

each_loop:
  ld a, ENTITY_COMPONENTS (ix)
  or a
  jr z, continue_each

  push bc
  push hl
    call ___sdcc_call_hl
  pop hl
  pop bc

continue_each:
  ld de, #ENTITY_SIZE
  add ix, de

  djnz each_loop
  ret
