;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "entities.h.s"
.include "alien.h.s"
.include "event.h.s"
.include "renderer.h.s"
.include "checkpoint.h.s"
.include "background.h.s"
.include "stages/game.h.s"
.include "entities/factories.h.s"
.include "entities/entity.h.s"
.include "data/level.h.s"
.include "data/game_assets.h.s"
.include "lang/macros.h.s"
.include "lang/debug.h.s"

.module failure

.equ ANIMAL_FLOATING_STEPS, 6     ; TODO this should come from *_assets.h.s

; IN:
;   HL <- level address
_failure_clear::
  pop  af         ;; AF <- return address
  pop  de         ;; DE <- level address
  pop  bc         ;; C <- game mode
  push af         ;; Put returning address in the stack again

  ld__iyl_e
  ld__iyh_d       ; IY <- level address

  ; failing?
  ld a, c
  cp #GAME_FAILING_MODE
  jr nz, continue_clear

  ; set level animal directly into animation
  ld e, LEVEL_FLOATING_ANIMAL_LO (iy)
  ld d, LEVEL_FLOATING_ANIMAL_HI (iy)
                  ; DE <- animal sprite address
  ld hl, #ANIMAL_FLOATING_ADDRESS
  ld b, #ANIMAL_FLOATING_STEPS
step_loop:
  ld (hl), e
  inc hl
  ld (hl), d
  .rept #STEP_SIZE - 1
  inc hl
  .endm
  djnz step_loop

  ; create entities
  push ix
  call _failure_create
  pop ix

  ; set unreachable failure position
  ld hl, #0x6fff
  jr set_position

continue_clear:
  ; get failure position from level ending
  ld l, LEVEL_ENDING_LO (iy)
  ld h, LEVEL_ENDING_HI (iy)
set_position:
  ld (#failure_checkpoint), hl
  sla l
  rl h            ; HL <- fixed_fromInt(ending position >> 7) = ending position << 1
  inc h           ; HL += 0x0100
  ld (#failure_position), hl

  ret

_failure_step::
  ; failure_position < alien_position?
.equ failure_position, . + 1
  ld de, #0
  ld bc, (#_alien_position)
                  ; BC <- alien_position
  ld a, e
  sub c
  ld a, d
  sbc b
  ret nc

_failure_force::
  ; set game failing state
  ld a, #GAME_FAILING
  EVENT_SET

  ; and update checkpoint to last level position
.equ failure_checkpoint, . + 1
  ld hl, #0
  CHECKPOINT_SET_POSITION

  ; and alien position
  ld hl, (#failure_position)
  CHECKPOINT_SET_ALIEN_POSITION

  ret


; #define TILE(n) ((PLATFORMS_0_INDEX + n) << 1) + 1
.macro DRAW_TILE n
  ld (hl), #((PLATFORMS_0_INDEX + n) << 1) + 1
.endm

.equ PLATFORM_X, 23 * 2
.equ PLATFORM_Y, 12 * 8

.equ PLATFORM_LEFT_INDEX, 0
.equ PLATFORM_RIGHT_INDEX, 1
.equ PLATFORM_SLOPE_UP_INDEX, 2
.equ PLATFORM_LIGHT_INDEX, 10

.macro TILE n
  .db #((PLATFORMS_0_INDEX + n) << 1) + 1
.endm

.equ TILE_COUNT, 6

_failure_tiles::
  TILE PLATFORM_LEFT_INDEX
  TILE PLATFORM_LIGHT_INDEX
  TILE PLATFORM_LIGHT_INDEX
  TILE PLATFORM_LIGHT_INDEX
  TILE PLATFORM_SLOPE_UP_INDEX
  TILE PLATFORM_RIGHT_INDEX

_failure_drawBackground::
  ld hl, (#_renderer_left)
  ld bc, #PLATFORM_X
  add hl, bc
  ex de, hl         ; DE <- xInBytes
  ld b, #PLATFORM_Y ; B <- y
  call _background_getTilePointer_asm
                    ; HL <- tile buffer address

  ld b, #TILE_COUNT
  ld de, #_failure_tiles
tile_loop:
  ld a, (de)
  ld (hl), a
  push bc
    ld bc, #RENDERER_GAME_HEIGHT_IN_TILES
    add hl, bc
  pop bc
  ENSURE_HL_IN_BUFFER
  inc de
  djnz tile_loop
  ret
