;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "player.h.s"
.include "alien.h.s"
.include "renderer.h.s"
.include "data/level.h.s"
.include "data/game_assets.h.s"
.include "lang/macros.h.s"
.include "lang/debug.h.s"

.module hud

.equ MAP_CHARACTERS_Y,  2

.equ MAP_START_SPRITE,  HUD_MAP_START_ADDRESS
.equ MAP_START_X,       21
.equ MAP_START_Y,       26

.equ MAP_SLASH_SPRITE,  HUD_MAP_SLASH_ADDRESS
.equ MAP_SLASHS_X,      25
.equ MAP_SLASHS_Y,      30
; .equ MAP_SLASH_COUNT,   13

.equ MAP_FLAG_SPRITE, HUD_MAP_FLAG_ADDRESS
.equ MAP_END_X,       MAP_START_X + HUD_ANIMALS_WIDTH_IN_BYTES * 2 + 2
.equ MAP_END_Y,       10

.equ MAP_ANIMAL_X,  MAP_START_X + HUD_ANIMALS_WIDTH_IN_BYTES + 2
.equ MAP_ANIMAL_Y,  MAP_CHARACTERS_Y

.equ MOJO_BAR_SIZE,     10
.equ MOJO_BAR_X,        6
.equ MOJO_BAR_X_FIRST,  MOJO_BAR_X + 1
.equ MOJO_BAR_Y,        (HUD_HEAD_HEIGHT + 1)
.equ MOJO_BAR_HEIGHT,   HUD_BAR_FILLING_HEIGHT
.equ MOJO_BAR_END_X,    MOJO_BAR_X + MOJO_BAR_SIZE + 1

.equ MOJO_BAR_START_SPRITE,   HUD_BAR_FILLING_0_ADDRESS
.equ MOJO_BAR_END_SPRITE,     HUD_BAR_FILLING_3_ADDRESS
.equ MOJO_BAR_FILLED_SPRITE,  HUD_BAR_FILLING_1_ADDRESS
.equ MOJO_BAR_EMPTY_SPRITE,   HUD_BAR_FILLING_2_ADDRESS

.equ HEAD_NORMAL_SPRITE,  HUD_HEAD_0_ADDRESS
.equ HEAD_FURY_SPRITE,    HUD_HEAD_1_ADDRESS
.equ HEAD_X,              6
.equ HEAD_Y,              0

.equ MAP_PLAYER_SPRITE,   HUD_CHARACTERS_0_ADDRESS
.equ MAP_PLAYER_X,        MAP_START_X - 2
.equ MAP_PLAYER_X_NEXT,   MAP_PLAYER_X + 1
.equ MAP_PLAYER_Y,        MAP_CHARACTERS_Y

.equ MAP_ALIEN_SPRITE,    HUD_CHARACTERS_2_ADDRESS
.equ MAP_FLYING_SPRITE,   HUD_CHARACTERS_1_ADDRESS


_hud_status::
  .db #0


; IN:
;   HL <- level address
_hud_clear::
  ; Init variables to invalid values
  xor a         ; A <- 0
  ld (#hud_mojo), a
  dec a         ; A <- 0xff
  ld (#_hud_status), a
  ld (#hud_playerPosition), a
  ld (#hud_alienPosition), a

  ex de, hl
  ld__iyl_e
  ld__iyh_d       ; IY <- level address

  ; Draw target animal
  ld l, LEVEL_ENDING_HI (iy)
  ld h, #0

  ; storing map displacement
  ld a, #30
  sub l
  ld (#hud_mapDisplacement), a

  push hl
    add hl, hl
    RENDERER_HUD_SCREEN_POINTER de, MAP_ANIMAL_X, MAP_ANIMAL_Y
    add hl, de
    ld e, LEVEL_HUD_ANIMAL_LO (iy)
    ld d, LEVEL_HUD_ANIMAL_HI (iy)
    call _hud_drawMapSprite

  ; Draw map start
    RENDERER_HUD_SCREEN_POINTER hl, MAP_START_X, MAP_START_Y
    ld de, #MAP_START_SPRITE
    call _hud_drawMapSprite
  pop hl

  ; Draw slashes
  push hl
    ld b, l
    dec b
    sra b
    RENDERER_HUD_SCREEN_POINTER hl, MAP_SLASHS_X, MAP_SLASHS_Y
slash_loop:
    push bc
      push hl
        ld de, #MAP_SLASH_SPRITE
        call _hud_drawMapSprite
      pop hl
      ld bc, #HUD_MAP_SLASH_WIDTH_IN_BYTES
      add hl, bc
    pop bc
    djnz slash_loop

  ; Draw map end
  pop hl
  add hl, hl
  RENDERER_HUD_SCREEN_POINTER de, MAP_END_X, MAP_END_Y
  add hl, de
  ld de, #MAP_FLAG_SPRITE
  call _hud_drawMapSprite

  ; Draw mojo bar
  ld de, #MOJO_BAR_START_SPRITE
  RENDERER_HUD_SCREEN_POINTER hl, MOJO_BAR_X, MOJO_BAR_Y
  call _hud_drawMapSprite

  ld de, #MOJO_BAR_END_SPRITE
  RENDERER_HUD_SCREEN_POINTER hl, MOJO_BAR_END_X, MOJO_BAR_Y
  call _hud_drawMapSprite

  jp _hud_drawMojo


; IN:
;   HL <- screen pointer
;   DE <- sprite pointer
_hud_drawMapSprite::
.equ hud_mapDisplacement, . + 1
  ld bc, #0
  add hl, bc
  ex de, hl
  jp _renderer_drawHudSprite_asm


_hud_step::
  ; draw head? check fury bit to be changed
  ld a, (#_player_status)
  ld c, a         ; C <- player status for later
  ld hl, #_hud_status
  add (hl)
  and #PLAYER_FURY
  jr z, check_mojo

  ; hud_status = player_status
  ld (hl), c

  ; draw head
  ld de, #HEAD_NORMAL_SPRITE
  bit PLAYER_FURY_BIT, c
  jr z, draw_head
  ld de, #HEAD_FURY_SPRITE
draw_head:
  RENDERER_HUD_SCREEN_POINTER hl, HEAD_X, HEAD_Y
  call _hud_drawMapSprite

check_mojo:
  ; player_mojo != hud_mojo
  ld a, (#_player_mojo)
  ld hl, #hud_mojo
  cp (hl)
  jr z, check_player

  ; player_mojo > hud_mojo
  jr c, check_mojo_lt
  ; hud_mojo++
  inc (hl)
  jr check_mojo_draw
check_mojo_lt:
  ; hud_mojo--
  dec (hl)
check_mojo_draw:
  call _hud_drawMojo

check_player:
  ; player_levelPosition != hud_playerPosition?
  ld a, (#_player_levelPosition)
.equ hud_playerPosition, . + 1
  cp #0
  jr z, check_alien

  ; hud_playerPosition = player_levelPosition
  ld (#hud_playerPosition), a

  ; draw player
  ld de, #MAP_PLAYER_SPRITE
  bit PLAYER_RIDING_BIT, c
  jr z, draw_player
  ld de, #MAP_FLYING_SPRITE
draw_player:
  RENDERER_HUD_SCREEN_POINTER hl, MAP_PLAYER_X, MAP_PLAYER_Y
  ld c, a
  ld b, #0
  add hl, bc
  call _hud_drawMapSprite

check_alien:
  ; alien_state != alien_ko?
  ld hl, (#_alien_state)
  ld bc, #_alien_ko
  xor a
  sbc hl, bc
  ret z

  ; alien_levelPosition != hud_alienPosition?
  ld a, (#_alien_levelPosition)
.equ hud_alienPosition, . + 1
  cp #0
  ret z

  ; hud_alienPosition = alien_levelPosition
  ld (#hud_alienPosition), a

  ; draw alien
  ld de, #MAP_ALIEN_SPRITE
  RENDERER_HUD_SCREEN_POINTER hl, MAP_PLAYER_X, MAP_PLAYER_Y
  ld c, a
  ld b, #0
  add hl, bc
  jp _hud_drawMapSprite


_hud_drawMojo::
  ld de, #MOJO_BAR_FILLED_SPRITE
                ; DE <- filled mojo sprite
  RENDERER_HUD_SCREEN_POINTER hl, MOJO_BAR_X_FIRST, MOJO_BAR_Y
                ; HL <- screen address

.equ hud_mojo, . + 1
  ld c, #0      ; C <- hud_mojo
  xor a         ; A <- counter

mojo_loop:
  cp c
  jr nz, continue_mojo_loop

  ld de, #MOJO_BAR_EMPTY_SPRITE
                ; DE <- empty mojo sprite

continue_mojo_loop:
  push af
  push bc
  push de
  push hl
    call _hud_drawMapSprite
  pop hl
  pop de
  pop bc
  pop af

  inc a
  cp #MOJO_BAR_SIZE
  ret z

  inc hl
  jr mojo_loop


_hud_removeAlien::
  ld a, (#hud_mapDisplacement)
  ld hl, #_alien_levelPosition
  add (hl)
  ld c, a
  ld b, #0
  RENDERER_HUD_SCREEN_POINTER hl, MAP_PLAYER_X_NEXT, MAP_PLAYER_Y
  add hl, bc
  jp _renderer_drawHudBox
