;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "entities.h.s"
.include "collision.h.s"
.include "terrain.h.s"
.include "platform.h.s"
.include "water.h.s"
.include "entities/entity.h.s"
.include "lang/fixed.h.s"
.include "lang/macros.h.s"
.include "lang/debug.h.s"

.module physics

.equ GRAVITY, 1
.equ LARGE_FX, 32 << FIXED_RATIO
.equ MIN_Y, 30
.equ MAX_Y, 158
.equ FLOOR_FRICTION, 50

_physics_step::
  push ix
  push iy
  ld hl, #_physics_update
  call _entities_each_asm
  pop iy
  pop ix
  ret

_physics_update::
  bit COMPONENT_BODY_BIT, ENTITY_COMPONENTS (ix)
                  ; TODO if having proper entity vector this is not needed
  ret z

  ; Apply acceleration
  bit BODY_HAS_GRAVITY_BIT, ENTITY_BODY_FLAGS (ix)
  jr z, apply_velocity

  ; body.vy += GRAVITY
.ifeq GRAVITY - 1
  inc ENTITY_BODY_VY (ix)
.else
  ld a, ENTITY_BODY_VY (ix)
  add #GRAVITY
  ld ENTITY_BODY_VY (ix), a
.endif

apply_velocity:
  ; body.fx += body.vx
  ld l, ENTITY_BODY_FX_LO (ix)
  ld h, ENTITY_BODY_FX_HI (ix)
  ld c, ENTITY_BODY_VX_LO (ix)
  ld b, ENTITY_BODY_VX_HI (ix)
  add hl, bc      ; HL <- body.fx
  ld ENTITY_BODY_FX_LO (ix), l
  ld ENTITY_BODY_FX_HI (ix), h
  ld e, h         ; E <- integer part of fx
  ld d, #0        ; DE <- integer part of fx

  ld l, ENTITY_BODY_IX_LO (ix)
  ld h, ENTITY_BODY_IX_HI (ix)
                  ; HL <- body.ix

  ; Take care of not having a large --positive-- fx
  ; body.fx >= 0?
  bit 7, e
  jr z, check_large_fx

  ld d, #0xff     ; DE <- valid negative value
  jr apply_position

check_large_fx:
  ; body.fx > LARGE_FX
  ld a, e
  cp #>LARGE_FX
  jr c, apply_position

  ; body.ix += integer part of body.fx
  add hl, de      ; HL <- updated body.ix
  ld ENTITY_BODY_IX_LO (ix), l
  ld ENTITY_BODY_IX_HI (ix), h
  ; body.fx &= 0x00ff
  xor a
  ; ld h, l         ; HL <- updated body.ix
  ld ENTITY_BODY_FX_HI (ix), a
  ld e, a         ; DE <- 0 = integer part of body.fx

apply_position:
  ; ix = body.ix + fixed_toInt(body.fx)
  add hl, de      ; HL <- ix

  ; iy = position.y + body.vy
  ld a, ENTITY_POSITION_Y (ix)
  add ENTITY_BODY_VY (ix)

  ; check if iy too small
  ; iy < animation.step->cy?
  ld e, ENTITY_ANIMATION_STEP_LO (ix)
  ld d, ENTITY_ANIMATION_STEP_HI (ix)
  ld__iyl_e
  ld__iyh_d
  cp STEP_CY (iy)
  jr nc, continue_checking_y
  ; iy = entity->animation.step->cy
  ld a, STEP_CY (iy)
;   jr end_checking_y

continue_checking_y:
;   ; iy > MAX_Y?
;   ; TODO this check may not be neccesary when renderer supports big y values
;   cp #MAX_Y
;   jr c, end_checking_y
;   ; iy = MAX_Y
;   ld a, #MAX_Y
;   ; body.flags |= BODY_ON_TERRAIN
;   set BODY_ON_TERRAIN_BIT, ENTITY_BODY_FLAGS (ix)

end_checking_y:
  ld b, a         ; B <- iy

  ld e, #0        ; E <- floor friction

  ; body.flags &= ~(BODY_ON_FLOOR | BODY_IN_WATER)
  ld a, ENTITY_BODY_FLAGS (ix)
  and #~(BODY_ON_FLOOR | BODY_IN_WATER)
  ld ENTITY_BODY_FLAGS (ix), a

  ; ignore collisions at all?
  bit BODY_IGNORE_COLLISIONS_BIT, ENTITY_BODY_FLAGS (ix)
  jp nz, update_position

  push bc
  push hl
    call _collision_check_asm
    ld a, l       ; A <- collision bits
  pop hl
  pop bc

  or a
  jp z, apply_friction

  ; check collision with floor, platforms and water
  ld c, a         ; C <- collision bits
  bit COLLISION_TERRAIN_BIT, c
  jr z, check_platform

  ; iy = terrain_getHeight(ix)
  push hl
    call _terrain_getHeight
    ld b, l       ; B <- iy = terrain height
  pop hl
  ; body.flags |= BODY_ON_TERRAIN
  set BODY_ON_TERRAIN_BIT, ENTITY_BODY_FLAGS (ix)

  bit COLLISION_WATER_BIT, c
  jr z, check_platform
  ; still colliding with water after updating iy?
  push hl
    call _water_getHeight
    ld a, b
    cp l
  pop hl
  jr nc, check_platform
  ; if not, remove water collision bit
  res COLLISION_WATER_BIT, c

check_platform:
  bit COLLISION_PLATFORM_BIT, c
  jr z, check_water

  ; iy = platform_getHeight(ix)
  push hl
    call _platform_getHeight
    ld b, l       ; B <- iy = platform height
  pop hl
  ; body.flags |= BODY_ON_PLATFORM
  set BODY_ON_PLATFORM_BIT, ENTITY_BODY_FLAGS (ix)

check_water:
  bit COLLISION_WATER_BIT, c
  jr z, apply_bounce

  ; body.flags |= BODY_IN_WATER
  set BODY_IN_WATER_BIT, ENTITY_BODY_FLAGS (ix)
  ; body.vx >>= 1
  ld a, ENTITY_BODY_VX_LO (ix)
  ld d, ENTITY_BODY_VX_HI (ix)
  srl d
  rra
  ld ENTITY_BODY_VX_LO (ix), a
  ld ENTITY_BODY_VX_HI (ix), d

apply_bounce:
  ; has friction and body.vy > 0?
  bit BODY_HAS_BOUNCE_BIT, ENTITY_BODY_FLAGS (ix)
  jr z, reset_vy
  bit #7, ENTITY_BODY_VY (ix)
  jr nz, reset_vy

  ; Add a little bouncing!
  ; body.vy = -(body.vy >> 1)
  ld a, ENTITY_BODY_VY (ix)
  or a            ; rcf
  rra
  neg
  ld ENTITY_BODY_VY (ix), a
  ; body.flags &= ~BODY_ON_FLOOR
  ld a, ENTITY_BODY_FLAGS (ix)
  and #~BODY_ON_FLOOR
  ld ENTITY_BODY_FLAGS (ix), a

  jr update_friction

reset_vy:
  ; body.vy = 0
  ld ENTITY_BODY_VY (ix), #0

update_friction:
  bit BODY_HAS_FRICTION_BIT, ENTITY_BODY_FLAGS (ix)
  jr z, apply_friction

  ; integer part of body.vx between (-1, 1)?
  ld a, ENTITY_BODY_VX_HI (ix)
  or a
  jr z, reset_vx
  cp #0xff
  jr nz, set_floor_friction

reset_vx:
  xor a
  ld ENTITY_BODY_VX_LO (ix), a
  ld ENTITY_BODY_VX_HI (ix), a
  jr apply_friction

set_floor_friction:
  ld e, #FLOOR_FRICTION
                  ; E <- friction = FLOOR_FRICTION

apply_friction:
  bit BODY_HAS_FRICTION_BIT, ENTITY_BODY_FLAGS (ix)
  jr z, update_position

  ld a, e
  or a            ; don't apply zero friction...
  jr z, update_position

  ; body.vx > friction?
  push hl
    ld d, #0        ; DE <- friction
    ld l, ENTITY_BODY_VX_LO (ix)
    ld h, ENTITY_BODY_VX_HI (ix)
    or a
    sbc hl, de
    jr c, continue_apply_friction
  ; body.vx -= friction
    ld ENTITY_BODY_VX_LO (ix), l
    ld ENTITY_BODY_VX_HI (ix), h
    jr end_apply_friction

continue_apply_friction:
  ; body.vx < -friction?
    ld l, ENTITY_BODY_VX_LO (ix)
    ld h, ENTITY_BODY_VX_HI (ix)
    add hl, de
    jr c, end_apply_friction
  ; body.vx += friction
    ld ENTITY_BODY_VX_LO (ix), l
    ld ENTITY_BODY_VX_HI (ix), h
end_apply_friction:
  pop hl

update_position:
  ; position.x = ix
  ld ENTITY_POSITION_X_LO (ix), l
  ld ENTITY_POSITION_X_HI (ix), h
  ; position.y = iy
  ld ENTITY_POSITION_Y (ix), b

  ret
