;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "background.h.s"
.include "renderer.h.s"
.include "data/game_assets.h.s"
.include "data/level.h.s"
.include "lang/debug.h.s"

.module platform

.equ NO_PLATFORM_HEIGHT,  200

.equ BASE, (PLATFORMS_0_INDEX << 1) + 1
.equ PLATFORM_LEFT_INDEX, BASE
.equ PLATFORM_RIGHT_INDEX, BASE + 2
.equ PLATFORM_SLOPE_UP_INDEX, BASE + 4
.equ PLATFORM_SLOPE_DOWN_INDEX, BASE + 12

.globl cpct_getRandom_mxor_u8_asm

_platform_level::
  .dw #0
_platform_counter::
  .db #0
_platform_y::
  .db #0
_platform_height::
  .db #0
_platform_width::
  .db #0

_platform_index::
  .dw #0
_platform_tiles::
  .dw #0
_platform_tileIndex::
  .db #0

_platform_clear::
  ; platform_level = level->platforms
  ld de, #LEVEL_PLATFORMS
  add hl, de
  ld e, (hl)
  inc hl
  ld d, (hl)    ; DE <- level platform data

  ; platform_index = renderer_left >> 1
  ld hl, (#_renderer_left)
  srl h
  rr l          ; HL <- renderer_right >> 1
  ld (#_platform_index), hl

  ; find level start address
  ld b, #0
find_start_loop:
  ld a, (de)    ; A <- *platform_level
  ld c, a       ; BC <- counter
  or a
  sbc hl, bc    ; HL -= counter
  jr c, found_start
  inc de        ; skip counter
  inc de        ; skip height
  inc de        ; skip length
  jr find_start_loop

found_start:
  add hl, bc
  sub l
  ld (#_platform_counter), a
  inc de
  ld (#_platform_level), de

  ; platform_y = 0
  xor a
  ld (#_platform_y), a
  
  ; platform_height = NO_PLATFORM_HEIGHT
  ld a, #NO_PLATFORM_HEIGHT
  ld (#_platform_height), a
  
  ; platform_tileIndex = PLATFORM_SLOPE_UP_INDEX
  ld a, #PLATFORM_SLOPE_UP_INDEX
  ld (#_platform_tileIndex), a

  ret

_platform_step::
  ; index = renderer_right >> 1
  ld bc, (_renderer_right)
  srl b
  rr c          ; BC <- renderer_right >> 1

  ld hl, (#_platform_index)
                ; HL <- platform_index

  ; while (platform_index < index) {
step_loop:
  ld a, l
  sub c
  ld a, h
  sbc b
  ret nc

  push bc
  push hl
    call _platform_generate
    ; TODO inline this?
  pop hl
  pop bc

  ; _platform_index++
  inc hl
  ld (#_platform_index), hl
  jr step_loop



_platform_generate::
  ; --platform_counter == 0?
  ld hl, #_platform_counter
  dec (hl)
  jr nz, update_height

  ; platform_y = *platform_level++
  ld hl, (#_platform_level)
  ld a, (hl)
  ld (#_platform_y), a
  ld c, a         ; C <- platform_y for later
  inc hl
  ; platform_height = platform_y << 3
  rla
  rla
  rla
  and #0xf8
  ld (#_platform_height), a
  ; platform_width = *platform_level++ + 1
  ld a, (hl)
  inc a           ; TODO this +1 should be added in asset tool?
  ld (#_platform_width), a
  inc hl
  ; platform_counter = *platform_level++
  ld a, (hl)
  ld (#_platform_counter), a
  inc hl
  ld (#_platform_level), hl

  ; platform_tiles = (platform_index * RENDERER_GAME_HEIGHT_IN_TILES + platform_y);
  ld hl, (#_platform_index)
  ld e, l
  ld d, h       ; DE <- platform_index
	add	hl, hl
	add	hl, hl
	add	hl, de
	add	hl, hl
	add	hl, hl    ; HL <- (platform_index) * 20

  ld b, #0      ; BC <- platform_y
  add hl, bc

  ENSURE_HL_IN_BUFFER
  ld (#_platform_tiles), hl

  ; platform_tile = PLATFORM_LEFT_INDEX
  ld a, #PLATFORM_LEFT_INDEX
  ld (#platform_tile), a

update_height:
  ; heights[platform_index & 0x3f] = platform_height
  ld hl, #PLATFORM_HEIGHTS_ADDRESS
  ld a, (#_platform_index)
  and #0x3f
  add l
  ld l, a

  ld a, (#_platform_height)
  ld (hl), a

  ; platform_y?
  ld a, (#_platform_y)
  or a
  ret z

  ; tiles[platform_tiles & 0x3ff] = platform_tile
  ld hl, (#_platform_tiles)
.equ platform_tile, . + 1
  ld (hl), #0
  ; platform_tiles += RENDERER_GAME_HEIGHT_IN_TILES
  ld bc, #RENDERER_GAME_HEIGHT_IN_TILES
  add hl, bc
  ENSURE_HL_IN_BUFFER
  ld (#_platform_tiles), hl

  ; --platform_width == 0
  ld hl, #_platform_width
  dec (hl)
  jr nz, update_tile

  ; platform_y = 0
  xor a
  ld (#_platform_y), a
  ; platform_height = NO_PLATFORM_HEIGHT
  ld a, #NO_PLATFORM_HEIGHT
  ld (#_platform_height), a

update_tile:
  ; platform_width == 1?
  ld a, #1
  cp (hl)
  jr nz, not_in_border

  ; platform_tile = PLATFORM_RIGHT_INDEX
  ld a, #PLATFORM_RIGHT_INDEX
  ld (#platform_tile), a
  ret

not_in_border:
  ; index = (random_get() %4) << 1
  call cpct_getRandom_mxor_u8_asm
  ld a, l
  and #0x06         ; A <- index

  ld hl, #_platform_tileIndex
  ld b, (hl)        ; B <- platform_tileIndex

  ; index?
  or a
  jr z, update_slope

  ; platform_tile = platform_tileIndex + index
  add b
  ld (#platform_tile), a
  ret

update_slope:
  ; platform_tileIndex == PLATFORM_SLOPE_UP_INDEX?
  ld a, b
  cp #PLATFORM_SLOPE_UP_INDEX
  jr nz, set_slope_up

  ; platform_tileIndex = platform_tile = PLATFORM_SLOPE_DOWN_INDEX
  ld a, #PLATFORM_SLOPE_DOWN_INDEX
  jr update_slope_tile

set_slope_up:
  ; platform_tileIndex = platform_tile = PLATFORM_SLOPE_UP_INDEX
  ld a, #PLATFORM_SLOPE_UP_INDEX

update_slope_tile:
  ld (#_platform_tileIndex), a
  ld (#platform_tile), a
  ret
