;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "renderer.h.s"
.include "event.h.s"
.include "cpc/controls.h.s"
.include "stages/game.h.s"
.include "data/game_assets.h.s"

.module playback

.equ JP_OPCODE, #0xc3

; _playback_data::
;   .db #0x55, #0xFF, #0x1E, #0xFB, #0x48, #0xFF, #0x0F, #0xFD, #0x06, #0xF9, #0x17, #0xFD, ...

_playback_counter::
  .db #0

_playback_controls_backup::
  .ds 3

_playback_init::
  ; backup controls_isPressed first instructions
  ld hl, #_controls_isPressed
  ld de, #_playback_controls_backup
  ld bc, #3
  ldir
  ret


; IN:
;   L <- game mode
_playback_clear::
  ld a, l
  cp #GAME_ATTRACT_MODE
  jr nz, unset_playback

  ; set playback data address
  ; ld hl, #_playback_data
  ld hl, #PLAYBACK_DATA_ADDRESS
  ; set first counter
  ld a, (hl)
  ld (#_playback_counter), a
  ; set first data position
  inc hl
  ld (#playback_position), hl

  ; monkey patch playback function
  ld hl, #_controls_isPressed
  ld (hl), #JP_OPCODE
  inc hl
  ld (hl), #<_playback_isKeyPressed
  inc hl
  ld (hl), #>_playback_isKeyPressed

  ; active system
  ld a, #1
  ld (#active), a

  ret

unset_playback:
  ; restore original controls_isPressed function instructions
  ld hl, #_playback_controls_backup
  ld de, #_controls_isPressed
  ld bc, #3
  ldir
  ; unactivate system
  xor a
  ld (#active), a
  ret


; _playback_isKeyPressed::
; ; bit 0, (hl) -> cb46   0100 0110
; ; bit 1, (hl) -> cb4e   0100 1110
; ; bit 2, (hl) -> cb56   0101 0110
; ; bit 3, (hl) -> cb5e   0101 1110
; ; bit 4, (hl) -> cb66   0110 0110
; ; bit 5, (hl) -> cb6e   0110 1110
;   ld a, l
;   rla
;   rla
;   rla
;   or #0b01000010
;   ld (#nbit), a
;   ld hl, (#playback_position)
; .equ nbit, . + 1
;   bit 0, (hl)
;
;   ld a, #0
;   ret z
;
;   inc a
;   ret


_playback_isKeyPressed::
  ld b, l
  inc b
  ld hl, (#playback_position)
  ld a, (hl)
check_loop:
  rra
  djnz check_loop

  ld a, #0      ; ld a doesn't change Carry flag (don't use xor a here!)
  ret c

  inc a
  ret


_playback_step::
.equ active, . + 1
  ld a, #0
  or a
  ret z

  ; check if any key is pressed to exit game
  call _controls_getKeyPressed
  ld a, l
  or h
  jr nz, end_playback

  ; --playback_counter == 0?
  ld hl, #_playback_counter
  dec (hl)
  ret nz

  ; *(playback_position + 1) == 0?
.equ playback_position, . + 1
  ld hl, #0
  inc hl
  ld a, (hl)
  or a
  jr z, end_playback

  ; set new counter
  ld (#_playback_counter), a
  inc hl
  ; update position
  ld (#playback_position), hl
  ret

end_playback:
  ld a, #GAME_EXIT
  EVENT_SET 

  jp _renderer_fadeOutWaiting
