;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "player.h.s"
.include "ai.h.s"
.include "renderer.h.s"
.include "entities/entity.h.s"
.include "stages/game.h.s"
.include "lang/macros.h.s"
.include "lang/debug.h.s"
.include "data/game_assets.h.s"

.module player

.equ MIN_BORDER_DISTANCE,   10

.equ MOJO_INITIAL,  1
.equ MOJO_MAX,      10

.equ PLAYER_NORMAL_VELOCITY,  #0x0100  ; 0x0180 is cool too
.equ PLAYER_FURY_VELOCITY,    #0x0200
.equ PLAYER_RIDING_VELOCITY,  #0x0200  ; 0x0300 is crazy but cool

.equ INMORTAL_TIME, 60

.globl _player_create

.globl _playerHit_behavior
.globl _playerDie_behavior
.globl _ride_behavior
.globl _rideHit_behavior
.globl _rideEnd_behavior
.globl _successFromRiding_behavior
.globl _success_behavior
.globl _playerWater_behavior
.globl _rideWater_behavior

_player_entity::
  .dw #0
_player_velocity::
  .dw #0
_player_status::
  .db #0
_player_levelPosition::
  .db #0
_player_mojo::
  .db #0

_player_furyTimePerMojo::
  .db 20, 40, 60, 80, 100, 120, 140, 170, 210, 255

; IN:
;   L <- game mode
_player_clear::
  ; player_status = 0
  xor a
  ld (#_player_status), a

  ; game mode == GAME_FAILING_MODE?
  ld a, #GAME_FAILING_MODE
  cp l
  jr nz, do_player_clear

  ; player_velocity = 0
  ld hl, #0
  ld (#_player_velocity), hl
  ; player_entity = NULL
  ld (#_player_entity), hl

  ret

do_player_clear:
  xor a
  ; player_furyTime = 0
  ld (#player_furyTime), a
  ; player_inmortalTime = 0;
  ld (#player_inmortalTime), a
  ; player_levelPosition = 0;
  ld (#_player_levelPosition), a

  ; player_mojo = MOJO_INITIAL;
  ld a, #MOJO_INITIAL
  ld (#_player_mojo), a
  ; player_velocity = PLAYER_NORMAL_VELOCITY;
  ld hl, #PLAYER_NORMAL_VELOCITY
  ld (#_player_velocity), hl

  ; player_entity = player_create();
  call _player_create
  ld (#_player_entity), hl

  ; and set inmortal at the beginning
  jp _player_setInmortal


_player_step::
  ; player_entity?
  ld hl, (#_player_entity)
  ld a, l
  or h
  ret z

  ; player_furyTime?
.equ player_furyTime, . + 1
  ld a, #0
  or a
  jr z, check_inmortality

  ; --player_furyTime == 0?
  dec a
  ld (#player_furyTime), a
  or a
  call z, _player_resetFury

check_inmortality:
.equ player_inmortalTime, . + 1
  ld a, #0
  ; ld a, #10   ; uncommenting this line makes player inmortal forever
  or a
  jr z, check_valid_position

  ; --player_inmortalTime == 0?
  dec a
  ld (#player_inmortalTime), a
  call z, _player_resetInmortal

check_valid_position:
  ld iy, (#_player_entity)
                    ; IY <- player entity address
  call _player_ensureValidPosition_asm
                    ; TODO inline this??

  ; player_levelPosition = player_entity->position.x >> 7
  ; player_entity->position.x = 0b ABCD EFGH I??? ????
  ; player_entity->position.x >> 7 = 0000 000A BCDE FGHI
  ; (U8)(player_entity->position.x >> 7) = BCDE FGHI
  ld a, ENTITY_POSITION_X_LO (iy)
  rla               ; Carry flag <- position.x MSB
  ld a, ENTITY_POSITION_X_HI (iy)
  rla               ; A <- (U8)(player_entity->position.x >> 7)
  ld (#_player_levelPosition), a

  ; player_status & PLAYER_IGNORED?
  ld a, (#_player_status)
                    ; C <- player status
  bit PLAYER_IGNORED_BIT, a
  ret nz

  ; player_status & PLAYER_INMORTAL?
  bit PLAYER_INMORTAL_BIT, a
  jr z, check_when_not_inmortal

  ; isInWater?
  bit BODY_IN_WATER_BIT, ENTITY_BODY_FLAGS (iy)
  call nz, _player_applyWaterRepellent
  jr check_taking_ufo

check_when_not_inmortal:
  ; collidableStatus & COLLIDABLE_ENEMY || isInWater?
  bit COLLIDABLE_ENEMY_BIT, ENTITY_COLLIDABLE_STATUS (iy)
  jr nz, apply_enemy_collisition
  bit BODY_IN_WATER_BIT, ENTITY_BODY_FLAGS (iy)
  jr z, check_taking_ufo
apply_enemy_collisition:
  call _player_applyEnemyCollision

check_taking_ufo:
  ; collidableStatus & COLLIDABLE_UFO?
  bit COLLIDABLE_UFO_BIT, ENTITY_COLLIDABLE_STATUS (iy)
  ret z

  jp _player_setRiding


; IN:
;   IY <- entity address
_player_ensureValidPosition_asm::
  ; fx = fixed_toInt(player_entity->body.fx)
  ld c, ENTITY_BODY_FX_HI (iy)
  ld b, #0
  ; x = player_entity->body.ix + fx
  ld l, ENTITY_BODY_IX_LO (iy)
  ld h, ENTITY_BODY_IX_HI (iy)
  add hl, bc
  ex de, hl         ; DE <- player x from body

  ; x0 = renderer_left + MIN_BORDER_DISTANCE
  ld hl, (#_renderer_left)
  ld bc, #MIN_BORDER_DISTANCE
  add hl, bc
  ; x < x0?
  ld a, e
  sub l
  ld a, d
  sbc h
  jr nc, check_with_right_border

update_position:
  ; player_entity->position.x = x
  ld ENTITY_POSITION_X_LO (iy), l
  ld ENTITY_POSITION_X_HI (iy), h
  ; player_entity->body.ix = x - fx
  xor a
  ld c, ENTITY_BODY_FX_HI (iy)
  ld b, a           ; BC <- fx
  sbc hl, bc
  ld ENTITY_BODY_IX_LO (iy), l
  ld ENTITY_BODY_IX_HI (iy), h
  ret

check_with_right_border:
  ; x1 = renderer_right - MIN_BORDER_DISTANCE
  ld hl, (#_renderer_right)
  ; xor a           ; not needed, because this is call from a nc condition
  sbc hl, bc
  ; x > x1?
  ld a, l
  sub e
  ld a, h
  sbc d
  ret nc

  jr update_position


_player_setFury::
  ; ensure player_mojo
  ld a, (#_player_mojo)
  or a
  ret z

  ; ensure not yet in fury status
  ld a, (#_player_status)
  ld c, a           ; C <- player status for later
  and #PLAYER_FURY
  ret nz

  ; player_status |= PLAYER_FURY | PLAYER_INMORTAL
  ld a, #PLAYER_FURY | PLAYER_INMORTAL
  or c  
  ld (#_player_status), a
  ; player_entity->collidable.type |= COLLIDABLE_FURY
  ld iy, (#_player_entity)
  set COLLIDABLE_FURY_BIT, ENTITY_COLLIDABLE_TYPE (iy)
  ; player_furyTime = player_furyTimePerMojo[player_mojo - 1]
  ld hl, #_player_furyTimePerMojo
  ld a, (#_player_mojo)
  dec a
  ld c, a
  ld b, #0
  add hl, bc
  ld a, (hl)
  ld (#player_furyTime), a
  ; player_mojo = 0
  xor a
  ld (#_player_mojo), a
  ; player_velocity = PLAYER_FURY_VELOCITY
  ld hl, #PLAYER_FURY_VELOCITY
  ld (#_player_velocity), hl
  ret


_player_setRiding::
  ; player_status = PLAYER_RIDING
  ld a, #PLAYER_RIDING
  ld (#_player_status), a
  ; player_furyTime = 0
  xor a
  ld (#player_furyTime), a
  ; player_velocity = PLAYER_RIDING_VELOCITY
  ld hl, #PLAYER_RIDING_VELOCITY
  ld (#_player_velocity), hl

  ; ai_setBehavior(player_entity, (u16)&ride_behavior)
  push ix
    ld ix, (#_player_entity)
    ld de, #_ride_behavior
    call _ai_setBehavior_asm
  pop ix
  ret


_player_addMojo::
  ; player_mojo < MOJO_MAX?
  ld hl, #_player_mojo
  ld a, #MOJO_MAX
  cp (hl)
  ret z

  ; player_mojo++
  inc (hl)
  ret


_player_resetFury::
  ; player_status &= ~PLAYER_FURY
  ld hl, #_player_status
  res PLAYER_FURY_BIT, (hl)
  ; player_entity->collidable.type &= ~COLLIDABLE_FURY
  ld iy, (#_player_entity)
  res COLLIDABLE_FURY_BIT, ENTITY_COLLIDABLE_TYPE (iy)

set_normal_velocity_and_inmortal:
  ; player_velocity = PLAYER_NORMAL_VELOCITY
  ld hl, #PLAYER_NORMAL_VELOCITY
  ld (#_player_velocity), hl
  ; set inmortal
  jp _player_setInmortal


_player_resetRiding::
  ; player_status &= ~PLAYER_RIDING
  ld hl, #_player_status
  res PLAYER_RIDING_BIT, (hl)

  jr set_normal_velocity_and_inmortal


_player_success::
  push ix
    ld ix, (#_player_entity)
  ; player_entity->collidable.type = 0
    xor a
    ld ENTITY_COLLIDABLE_TYPE (ix), a
  ; player_status & PLAYER_RIDING?
  ;   _successFromRiding_behavior: _success_behavior
    ld hl, #_player_status
    ld de, #_success_behavior
    bit PLAYER_RIDING_BIT, (hl)
    jr z, set_success_behavior
    ld de, #_successFromRiding_behavior
set_success_behavior:
    call _ai_setBehavior_asm
  pop ix
  ; player_velocity = PLAYER_NORMAL_VELOCITY
  ld hl, #PLAYER_NORMAL_VELOCITY
  ld (#_player_velocity), hl
  ; player_status = PLAYER_IGNORED
  ld a, #PLAYER_IGNORED
  ld (#_player_status), a
  ; player_furyTime = 0, preventing fury reset
  xor a
  ld (#player_furyTime), a
  ret


_player_applyEnemyCollision::
  push ix
  ld ix, (#_player_entity)

  ; player_mojo == 0?
  ld a, (#_player_mojo)
  or a
  jr nz, hit_with_mojo

  ; player_status & PLAYER_RIDING?
  ld hl, #_player_status
  bit PLAYER_RIDING_BIT, (hl)
  jr z, die_running

  ; reset riding
  call _player_resetRiding
  ; and set falling from ufo behavior
  ld de, #_rideEnd_behavior
  jr set_behavior_and_return

die_running:
  ; player_entity->body.flags = BODY_HAS_GRAVITY | BODY_HAS_FRICTION | BODY_HAS_BOUNCE
  ld ENTITY_BODY_FLAGS (ix), #BODY_HAS_GRAVITY | BODY_HAS_FRICTION | BODY_HAS_BOUNCE
  ; player_entity->collidable.type &= ~COLLIDABLE_PLAYER
  res COLLIDABLE_PLAYER_BIT, ENTITY_COLLIDABLE_TYPE (ix)
  ; player_velocity = 0
  ld hl, #0
  ld (#_player_velocity), hl
  ; player_status = PLAYER_IGNORED
  ld a, #PLAYER_IGNORED
  ld (#_player_status), a
  ; set dying behavior
  ld de, #_playerDie_behavior
  jr set_behavior_and_return

hit_with_mojo:
  ; player_mojo = 0
  xor a
  ld (#_player_mojo), a
  ; set inmortal
  call _player_setInmortal
  ; behavior = player_status & PLAYER_RIDING?
  ;   rideHit_behavior: playerHit_behavior
  ld de, #_playerHit_behavior
  ld hl, #_player_status
  bit PLAYER_RIDING_BIT, (hl)
  jr z, set_behavior_and_return
  ld de, #_rideHit_behavior

set_behavior_and_return:
  call _ai_setBehavior_asm

  pop ix
  ret


_player_applyWaterRepellent::
  push ix
  ld ix, (#_player_entity)
  ; behavior = player_status & PLAYER_RIDING?
  ;   rideWater_behavior: playerWater_behavior
  ld de, #_playerWater_behavior
  ld hl, #_player_status
  bit PLAYER_RIDING_BIT, (hl)
  jr z, set_behavior_and_return
  ld de, #_rideWater_behavior
  jr set_behavior_and_return


_player_setInmortal::
  ; player_status |= PLAYER_INMORTAL
  ld hl, #_player_status
  set PLAYER_INMORTAL_BIT, (hl)
  ; player_entity->drawing.flags |= DRAWING_BLINK
  ld iy, (#_player_entity)
  set DRAWING_BLINK_BIT, ENTITY_DRAWING_FLAGS (iy)
  ; player_inmortalTime = INMORTAL_TIME
  ld a, #INMORTAL_TIME
  ld (#player_inmortalTime), a
  ret


_player_resetInmortal::
  ; player_entity->drawing.flags &= ~DRAWING_BLINK
  ld iy, (#_player_entity)
  res DRAWING_BLINK_BIT, ENTITY_DRAWING_FLAGS (iy)

  ; !(player_status & PLAYER_FURY)?
  ld hl, #_player_status
  bit PLAYER_FURY_BIT, (hl)
  ret nz

  ; player_status &= ~PLAYER_INMORTAL
  res PLAYER_INMORTAL_BIT, (hl)
  ret
