/*
  The Abduction of Oscar Z
  ========================

  "Oscar loved watching night skies long after his parents slept. But tonight
  that sky was watching him..."

  * PRODUCTION & CODE / Tulo / @tulinmola
  * ART & STORY / Alberto Hache / @Alberto_H9
  * MUSIC & FX / Gryzor87 / @Gryzor87

  A 2020 Dreamin`Bits production
  https://dreaminbits.com/

  MIT License

  Copyright (c) 2020 Dreamin`Bits

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the"Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#ifndef _RENDERER_H_
#define _RENDERER_H_

#include "../data/level.h"
#include "../cpc/interrupts.h"
#include "../entities/entity.h"

#define RENDERER_VIDEO_MODE   0

#define RENDERER_HUD_ADDRESS          (u8*)0x8000
#define RENDERER_HUD_SIZE             0x1000
#define RENDERER_HUD_WIDTH_IN_CHARS   50
#define RENDERER_HUD_WIDTH_IN_BYTES   (RENDERER_HUD_WIDTH_IN_CHARS << 1)
#define RENDERER_HUD_WIDTH            (RENDERER_HUD_WIDTH_IN_BYTES << 1)
#define RENDERER_HUD_HEIGHT_IN_CHARS  20
#define RENDERER_HUD_HEIGHT           (RENDERER_HUD_HEIGHT_IN_CHARS << 1)

#define RENDERER_HUD_SCREEN_POINTER(xInBytes, y) (u8*)(RENDERER_HUD_ADDRESS + ((y & 1) != 0? 0x800: 0) + (y >> 1) * RENDERER_HUD_WIDTH_IN_BYTES + xInBytes)

#define RENDERER_GAME_PAGE            cpct_pageC0
#define RENDERER_GAME_ADDRESS         (u8*)0xc000
#define RENDERER_GAME_SIZE            0x4000
#define RENDERER_GAME_WIDTH_IN_CHARS  51
#define RENDERER_GAME_WIDTH_IN_TILES  RENDERER_GAME_WIDTH_IN_CHARS
#define RENDERER_GAME_WIDTH_IN_BYTES  (RENDERER_GAME_WIDTH_IN_CHARS << 1)
#define RENDERER_GAME_HEIGHT_IN_CHARS 20
#define RENDERER_GAME_HEIGHT_IN_TILES RENDERER_GAME_HEIGHT_IN_CHARS
#define RENDERER_GAME_HEIGHT          (RENDERER_GAME_HEIGHT_IN_CHARS << 3)

#define RENDERER_GAME_SCREEN_POINTER(xInBytes, y) (u8*)((RENDERER_GAME_ADDRESS) + RENDERER_GAME_WIDTH_IN_BYTES * ((u8)((y) >> 3)) + 2048 * ((y) & 7) + (xInBytes))

extern u16 renderer_left;
extern u16 renderer_right;

void renderer_init();
void renderer_clear(const Level* level) __z88dk_fastcall;
void renderer_clearWithPosition(const Level* level, u16 position) __z88dk_callee;
#define renderer_fadeIn interrupts_startFadeIn
#define renderer_fadeOut interrupts_startFadeOut
void renderer_fadeOutWaiting();
void renderer_ensureFirstSync();

// void renderer_addEntity(Entity* entity) __z88dk_fastcall;
void renderer_removeEntity(Entity* entity) __z88dk_fastcall;
void renderer_cleanAll();

void renderer_step();
void renderer_move(u8 displacement) __z88dk_fastcall;
void renderer_draw();

extern void renderer_clearHud(u8 pattern) __z88dk_fastcall;
extern void renderer_drawHudSprite(const u8* sprite, u8* screen) __z88dk_callee;
extern void renderer_drawHudBox(u8* screen) __z88dk_fastcall;
// extern void renderer_drawHudString(const u8* string, u8* screen) __z88dk_callee;
extern void renderer_drawString(const char* string, u8* screen) __z88dk_callee;
extern u8* renderer_getScreenPointer(u16 xInBytes, u8 y) __z88dk_callee;

#endif // _RENDERER_H_
