;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.equ RENDERER_PAGE_C0, #0x30
.equ RENDERER_PAGE_80, #0x20
.equ RENDERER_PAGE_40, #0x10

.equ RENDERER_VIDEO_MODE, 0

.equ RENDERER_HUD_ADDRESS, #0x8000
.equ RENDERER_HUD_SIZE, #0x1000
.equ RENDERER_HUD_PAGE, RENDERER_PAGE_80
.equ RENDERER_HUD_WIDTH_IN_CHARS, 50
.equ RENDERER_HUD_WIDTH_IN_BYTES, RENDERER_HUD_WIDTH_IN_CHARS << 1
.equ RENDERER_HUD_HEIGHT_IN_CHARS, 20

.macro RENDERER_HUD_SCREEN_POINTER reg16, xInBytes, y
  .if y & 1
  ld reg16, #(RENDERER_HUD_ADDRESS + #0x800 + (y >> 1) * RENDERER_HUD_WIDTH_IN_BYTES + xInBytes)
  .else
  ld reg16, #(RENDERER_HUD_ADDRESS + (y >> 1) * RENDERER_HUD_WIDTH_IN_BYTES + xInBytes)
  .endif
.endm

.equ RENDERER_GAME_PAGE, RENDERER_PAGE_C0
.equ RENDERER_GAME_ADDRESS, 0xc000
.equ RENDERER_GAME_SIZE, 0x4000
.equ RENDERER_GAME_WIDTH_IN_CHARS, 51
.equ RENDERER_GAME_WIDTH_IN_TILES, RENDERER_GAME_WIDTH_IN_CHARS
.equ RENDERER_GAME_WIDTH_IN_BYTES, RENDERER_GAME_WIDTH_IN_CHARS << 1
.equ RENDERER_GAME_HEIGHT_IN_CHARS, 20
.equ RENDERER_GAME_HEIGHT_IN_TILES, RENDERER_GAME_HEIGHT_IN_CHARS
.equ RENDERER_GAME_HEIGHT, RENDERER_GAME_HEIGHT_IN_CHARS << 3

.macro RENDERER_GAME_SCREEN_POINTER xInBytes, y
  .dw #RENDERER_GAME_ADDRESS + #RENDERER_GAME_WIDTH_IN_BYTES * ((y) >> 3) + 2048 * ((y) & 7) + (xInBytes)
.endm

; typedef struct {
;   Bounds bounds;  // Char aligned bounds
;   u8* screen;     // Screen pointer
;   u8* tile;       // Backbuffer tile pointer
;   u8 unalignedBottom; // TODO??
; } Stain;

.equ STAIN_BOUNDS, 0
.equ STAIN_BOUNDS_LEFT, 0
.equ STAIN_BOUNDS_LEFT_LO, 0
.equ STAIN_BOUNDS_LEFT_HI, 1
.equ STAIN_BOUNDS_RIGHT, 2
.equ STAIN_BOUNDS_RIGHT_LO, 2
.equ STAIN_BOUNDS_RIGHT_HI, 3
.equ STAIN_BOUNDS_TOP, 4
.equ STAIN_BOUNDS_BOTTOM, 5
.equ STAIN_SCREEN, 6
.equ STAIN_SCREEN_LO, 6
.equ STAIN_SCREEN_HI, 7
.equ STAIN_TILE, 8
.equ STAIN_TILE_LO, 8
.equ STAIN_TILE_HI, 9
.equ STAIN_UNALIGNED_BOTTOM, 10
.equ STAIN_SIZE, 11

.macro STAIN left, right, top, bottom
  .dw #left, #right
  .db #top, #bottom
  .ds 4           ; screen, tile
  .db #bottom     ; unaligned bottom
.endm
; Note: if size changes, the code of renderer_addStain must change too!

.globl _renderer_left
.globl _renderer_right
.globl _renderer_semaphore

.globl _renderer_topBorderColor
.globl _renderer_bottomBorderColor

.globl _renderer_topBand
.globl _renderer_bottomBand

.globl _renderer_cleanAll
.globl _renderer_addEntity_asm
.globl _renderer_removeEntity
.globl _renderer_addStain
.globl _renderer_getScreenPointer_asm
.globl _renderer_clearHud_asm
.globl _renderer_drawString_asm
.globl _renderer_drawHudString_asm
.globl _renderer_drawHudSprite_asm
.globl _renderer_drawHudBox

.globl _interrupts_startFadeIn
.globl _interrupts_startFadeOut
; .equ _renderer_fadeIn, _interrupts_startFadeIn
; .equ _renderer_fadeOut, _interrupts_startFadeOut
.globl _renderer_fadeOutWaiting

.macro renderer_next_line_hl ?continue
  ld a, h
  add a, #0x08
  ld h, a
  and #0x38
  jr nz, continue

  ld a, h
  sub #0x40
  ld h, a
  ld a, l
  add a, #RENDERER_GAME_WIDTH_IN_BYTES
  ld l, a
  jr nc, continue

  inc h
  ld a, h
  and #0x07
  jr nz, continue

  ld a, h
  sub #0x08
  ld h, a
continue:
.endm

.macro renderer_next_line_de ?continue
  ld a, d
  add a, #0x08
  ld d, a
  and #0x38
  jr nz, continue

  ld a, d
  sub #0x40
  ld d, a
  ld a, e
  add a, #RENDERER_GAME_WIDTH_IN_BYTES
  ld e, a
  jr nc, continue

  inc d
  ld a, d
  and #0x07
  jr nz, continue

  ld a, d
  sub #0x08
  ld d, a
continue:
.endm

.macro renderer_next_byte_hl ?continue
  inc l
  jr nz, continue
  inc h
  ld a, h
  and #0x07
  jr nz, continue
  ld a, h
  sub #0x08
  ld h, a
continue:
.endm

.macro renderer_next_byte_de ?continue
  inc e
  jr nz, continue
  inc d
  ld a, d
  and #0x07
  jr nz, continue
  ld a, d
  sub #0x08
  ld d, a
continue:
.endm
