;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "renderer.h.s"
.include "entities.h.s"
.include "background.h.s"
.include "entities/entity.h.s"
.include "lang/macros.h.s"
.include "lang/debug.h.s"
.include "lang/pvector.h.s"
.include "data/level.h.s"
.include "data/game_assets.h.s"

.module renderer

.equ SHOW_COLLIDABLES, 0

.globl cpct_setVideoMode_asm
.globl cpct_memset_asm

; private renderer globals
.globl _renderer_setBackgroundInks
.globl _renderer_setBlackPalette
.globl _renderer_drawStain_asm
.globl _renderer_drawRlz3_asm
.globl _renderer_drawString_asm

.equ MAX_STAINS, ENTITY_COUNT + 2
.equ MAX_DRAWINGS, ENTITY_COUNT

_renderer_left::
  .dw #0
_renderer_right::
  .dw #0

_renderer_topBorderColor::
  .db #0
_renderer_bottomBorderColor::
  .db #0

_renderer_topBand::
  .db #0
_renderer_bottomBand::
  .db #0

_renderer_semaphore:
  .db #0

_renderer_stainCount::
  .db #0
_renderer_stains::
  .ds STAIN_SIZE * MAX_STAINS

_renderer_newSpansStain::
  STAIN 0, 0, 0, #RENDERER_GAME_HEIGHT

_renderer_ySortedStains::
  PVECTOR MAX_STAINS

_renderer_ySortedDrawings::
  PVECTOR MAX_DRAWINGS


_renderer_init::
  ld c, #RENDERER_VIDEO_MODE
  ; jp cpct_setVideoMode_asm
  ; TODO set black palette
  call cpct_setVideoMode_asm
  jp _renderer_setBlackPalette


; IN:
;   HL <- level address
_renderer_clear::
  push ix
  ex de, hl
  ld__ixl_e
  ld__ixh_d
  ld hl, #0
  call _renderer_clear_asm
  pop ix
  ret

_renderer_clearWithPosition::
  pop  af       ;; AF <- return address
  pop  de       ;; DE <- level address
  pop  hl       ;; HL <- position
  push af       ;; Put returning address in the stack again

  push ix
  ld__ixl_e
  ld__ixh_d
  call _renderer_clear_asm
  pop ix
  ret

; IN:
;   IX <- level address
;   HL <- initial position
_renderer_clear_asm::
  ld (#_renderer_left), hl

  ld bc, #RENDERER_GAME_WIDTH_IN_BYTES
  add hl, bc
  ld (#_renderer_right), hl

  ld a, LEVEL_BACKGROUND_COLOR (ix)
  ld (#_renderer_topBorderColor), a
  ld a, LEVEL_BOTTOM_COLOR (ix)
  ld (#_renderer_bottomBorderColor), a

  ld l, LEVEL_BACKGROUND_INKS_LO (ix)
  ld h, LEVEL_BACKGROUND_INKS_HI (ix)
  call _renderer_setBackgroundInks

  xor a
  ld (#_renderer_stainCount), a

  ; xor a
  call _renderer_clearHud_asm

  xor a
  ld de, #RENDERER_GAME_ADDRESS
  ld bc, #RENDERER_GAME_SIZE
  call cpct_memset_asm

  ; ensure terrain is init as non visible
  ld a, #RENDERER_GAME_HEIGHT
  ld de, #TERRAIN_HEIGHTS_ADDRESS
  ld bc, #TERRAIN_HEIGHTS_SIZE
  call cpct_memset_asm

  ld hl, #_renderer_ySortedStains
  call _pvector_init

  ld hl, #_renderer_ySortedDrawings
  call _pvector_init

  jp _renderer_cleanAll


; IN:
;   L <- pattern
_renderer_clearHud::
  ld a, l
; IN:
;   A <- pattern
; DESTROYS:
;   AF, BC, DE, HL
_renderer_clearHud_asm::
  ld de, #RENDERER_HUD_ADDRESS
  ld bc, #RENDERER_HUD_SIZE
  jp cpct_memset_asm


_renderer_fadeOutWaiting::
  call _interrupts_startFadeOut
  ld b, #20
wait_loop:
  call _renderer_waitForSemaphore
  djnz wait_loop
  ret


; ; IN:
; ;   HL <- entity address
; _renderer_addEntity::
;   ; TODO this will be removed when unused
;   push ix
;   push iy
;
;   ex de, hl
;   ld__ixl_e
;   ld__ixh_d     ; IX <- entity address
;
;   call _renderer_addEntity_asm
;
;   pop iy
;   pop ix
;   ret

; IN:
;   IX <- entity address
; DESTROYS:
;   AF, BC, DE, HL, IY
_renderer_addEntity_asm::
  ld a, ENTITY_DRAWING_FLAGS (ix)
                ; save drawing flags
  push af
    set #DRAWING_OPAQUE_BIT, ENTITY_DRAWING_FLAGS (ix)
                ; set opaque temporarily to avoid drawing stain at first drawn
    call _renderer_checkDrawing
  pop af
  or #DRAWING_REDRAW
                ; ensure redraw flag is set
  ld ENTITY_DRAWING_FLAGS (ix), a
                ; restore all other drawing flags

                ; IX <- entity address
  ld de, #_renderer_ySortedDrawings
  ld hl, #_renderer_drawingTopSorter
  jp _pvector_addSorted_ir_asm


; IN:
;   HL <- entity address
_renderer_removeEntity::
  ld c, l
  ld b, h
                ; BC <- entity address
  ld hl, #_renderer_ySortedDrawings
  jp _pvector_remove_asm


_renderer_cleanAll::
  ld de, (#_renderer_right)
                ; DE <- rendering_right
  ld hl, (#_renderer_left)
                ; HL <- rendering_left
  jp set_new_spans

; screenshot_counter:
; .db #70

_renderer_step::
  push ix
  push iy

  ; uncoment these lines to break each complete screen, useful
  ; to take large level screenshots
;   ld hl, #screenshot_counter
;   dec (hl)
;   jr nz, no_screenshot

;   BREAKPOINT
;   ld (hl), #RENDERER_GAME_WIDTH_IN_BYTES - 6
; no_screenshot:

  ; Ensure drawings are sorted from top to bottom to be drawn in order
  ld de, #_renderer_ySortedDrawings
  ld hl, #_renderer_drawingTopSorter
  call _pvector_sort_ir_asm

  ; Check if drawings changed, updating their properties and adding
  ; stains if needed.
  ld de, #_renderer_ySortedDrawings
  ld hl, #_renderer_checkDrawing
  call _pvector_apply_ix_asm

  ; Check collisions between stains and drawings to force redraws
  call _renderer_checkCollisions

  ld hl, #renderer_frame
  inc (hl)

  pop iy
  pop ix
  ret


.macro NEXT_STAIN_OR_RETURN
  ld c, (hl)
  inc hl
  ld b, (hl)

  ld a, b
  or c
  ret z

  inc hl          ; HL <- next stain pointer address
  ld__ixl_c
  ld__ixh_b       ; IX <- current stain
.endm

.macro NEXT_DRAWING_OR_RETURN
  ld a, (de)
  ld c, a
  inc de
  ld a, (de)
  ld b, a

  or c
  ret z

  inc de          ; DE <- next drawing pointer address
  ld__iyl_c
  ld__iyh_b       ; IY <- current drawing
.endm

.macro NEXT_DRAWING_OR_JUMP address
  ld a, (de)
  ld c, a
  inc de
  ld a, (de)
  ld b, a

  or c
  jr z, address

  inc de          ; DE <- next drawing pointer address
  ld__iyl_c
  ld__iyh_b       ; IY <- current drawing
.endm


_renderer_checkCollisions::
  ld hl, #_renderer_ySortedStains + PVECTOR_ITEMS
                  ; HL <- stain pointers address
  ld de, #_renderer_ySortedDrawings + PVECTOR_ITEMS
                  ; DE <- drawing pointers address

  NEXT_DRAWING_OR_RETURN
                  ; IY <- first drawing
collision_stain_loop:
  NEXT_STAIN_OR_RETURN

  ; find first drawing that collides vertically with
  ; stain, and return if not found
find_first_drawing:
  bit #DRAWING_REDRAW_BIT, ENTITY_DRAWING_FLAGS (iy)
  jr nz, next_first_drawing
                  ; Bypass drawing if already marked for redraw

  ld a, STAIN_BOUNDS_TOP (ix)
  cp ENTITY_DRAWING_BOUNDS_BOTTOM (iy)
  jr c, check_stain
  
next_first_drawing:
  NEXT_DRAWING_OR_RETURN
  jr find_first_drawing

check_stain:
  ; IY <- first drawing colliding vertically with stain
  push de
  push iy

    ; Check with drawings while colliding vertically
drawing_loop:
    bit #DRAWING_REDRAW_BIT, ENTITY_DRAWING_FLAGS (iy)
    jr nz, next_drawing
                  ; Bypass drawing if already marked for redraw

    ; still collides vertically?
    ld a, STAIN_BOUNDS_BOTTOM (ix)
    cp ENTITY_DRAWING_BOUNDS_TOP (iy)
    jr c, next_stain

    ; horizontal collision?
    ; stain.right > drawing.left
    ld a, ENTITY_DRAWING_BOUNDS_LEFT_LO (iy)
    sub STAIN_BOUNDS_RIGHT_LO (ix)
    ld a, ENTITY_DRAWING_BOUNDS_LEFT_HI (iy)
    sbc STAIN_BOUNDS_RIGHT_HI (ix)
    jr nc, next_drawing
    ; stain.left < drawing.right
    ld a, ENTITY_DRAWING_BOUNDS_RIGHT_LO (iy)
    sub STAIN_BOUNDS_LEFT_LO (ix)
    ld a, ENTITY_DRAWING_BOUNDS_RIGHT_HI (iy)
    sbc STAIN_BOUNDS_LEFT_HI (ix)
    jr c, next_drawing

    ; there is collision, so mark to redraw
    set #DRAWING_REDRAW_BIT, ENTITY_DRAWING_FLAGS (iy)

next_drawing:
    NEXT_DRAWING_OR_JUMP next_stain
    jr drawing_loop

next_stain:
  pop iy
  pop de
  jr collision_stain_loop


; IX <- first entity address
; IY <- second entity address
_renderer_drawingTopSorter::
  ld a, ENTITY_DRAWING_BOUNDS_TOP (ix)
  cp ENTITY_DRAWING_BOUNDS_TOP (iy)
  ret


; IX <- first stain address
; IY <- second stain address
_renderer_stainTopSorter::
  ld a, STAIN_BOUNDS_TOP (ix)
  cp STAIN_BOUNDS_TOP (iy)
  ret


; IX <- entity address
_renderer_checkDrawing::
  ld e, ENTITY_ANIMATION_STEP_LO (ix)
  ld d, ENTITY_ANIMATION_STEP_HI (ix)
  ld__iyl_e
  ld__iyh_d       ; IY <- step address

  ld l, ENTITY_POSITION_X_LO (ix)
  ld h, ENTITY_POSITION_X_HI (ix)
  ld d, #0
  ld e, STEP_CX (iy)

  or a            ; reset Carry
  sbc hl, de      ; HL <- left = entity.position.x - step.cx

  ld e, STEP_SPRITE_LO (iy)
  ld d, STEP_SPRITE_HI (iy)
                  ; DE <- sprite = step.sprite

  inc de
  ld a, (de)
  ld c, a         ; C <- sprite height
  dec de

  ld a, ENTITY_POSITION_Y (ix)
  sub STEP_CY (iy)
                  ; A <- top
  ; top < 0? prevent drawing when going out the top
  jr nc, ensure_bottom_in_screen
  xor a           ; A <- top = 0
ensure_bottom_in_screen:
  ld b, a         ; B <- top = MATH_MAX(entity.position.x - step.cy, 0)

  ; bottom > screen height? prevent drawing when going out the bottom
  add c           ; A <- bottom
  sub #RENDERER_GAME_HEIGHT + 1
  jr c, continue_checking_left
  inc a
  neg
  add b
  ld b, a         ; B <- MATH_MIN(top, SCREEN_HEIGHT - sprite height)
continue_checking_left:

  ; check left
  ld a, ENTITY_DRAWING_LEFT_LO (ix)
  cp l
  jr nz, changed
  ld a, ENTITY_DRAWING_LEFT_HI (ix)
  cp h
  jr nz, changed

  ; check top
  ld a, ENTITY_DRAWING_TOP (ix)
  cp b
  jr nz, changed

  ; check sprite
  ld a, ENTITY_DRAWING_SPRITE_LO (ix)
  cp e
  jr nz, changed
  ld a, ENTITY_DRAWING_SPRITE_HI (ix)
  cp d
  ret z           ; don't add stain nor draw anything if there are no changes

changed:
  ; add stain only if !opaque
  bit #DRAWING_OPAQUE_BIT, ENTITY_DRAWING_FLAGS (ix)
  jr nz, update_drawing

  exx
  push ix
    call _renderer_addStain
  pop ix
  exx

update_drawing:
                  ; HL <- left
                  ; B <- top
                  ; DE <- sprite
  ld ENTITY_DRAWING_SPRITE_LO (ix), e
  ld ENTITY_DRAWING_SPRITE_HI (ix), d
 
  ld ENTITY_DRAWING_LEFT_LO (ix), l
  ld ENTITY_DRAWING_LEFT_HI (ix), h

  ld ENTITY_DRAWING_TOP (ix), b

  push hl
    ; calculate char aligned bounds
    ld a, #0xfe
    and l
    ld ENTITY_DRAWING_BOUNDS_LEFT_LO (ix), a
    ld ENTITY_DRAWING_BOUNDS_LEFT_HI (ix), h
                  ; entity.drawing.bounds.left = left & 0xfffe

    ld a, (de)    ; A <- sprite width
    add_hl_a
    inc hl
    ld a, #0xfe
    and l
    ld ENTITY_DRAWING_BOUNDS_RIGHT_LO (ix), a
    ld ENTITY_DRAWING_BOUNDS_RIGHT_HI (ix), h
                  ; entity.drawing.bounds.right = (left + width + 1) & 0xfffe

    ld a, #0xf8
    and b
    ld ENTITY_DRAWING_BOUNDS_TOP (ix), a
                  ; entity.drawing.bounds.top = top & 0xf8

    inc de
    ld a, (de)    ; A <- height
    add b
    add #7
    and #0xf8
    ld ENTITY_DRAWING_BOUNDS_BOTTOM (ix), a
                  ; entity.drawing.bounds.bottom = (top + height + 7) & 0xf8

  pop de          ; DE <- left, B <- top
  call _renderer_getScreenPointer_asm
                  ; HL <- screen pointer

  ld ENTITY_DRAWING_SCREEN_LO (ix), l
  ld ENTITY_DRAWING_SCREEN_HI (ix), h

  set #DRAWING_REDRAW_BIT, ENTITY_DRAWING_FLAGS (ix)
                  ; force this entity to be redrawn

  set #COLLIDABLE_DIRTY_BIT, ENTITY_COLLIDABLE_STATUS (ix)
                  ; mark entity as dirty for collision system

  ret


; IX <- entity address
; DESTROYS IX!
_renderer_addStain::
  ld hl, #_renderer_stainCount
  ld a, (hl)      ; A <- _renderer_stainCount
  inc (hl)        ; _renderer_stainCount++

  ld l, a
  ld h, #0        ; HL <- stain count
  ld c, l
  ld b, h         ; BC <- stain count

  add hl, hl      ; HL <- HL * 2
  add hl, hl      ;       HL * 4
  add hl, hl      ;       HL * 8
  add hl, bc      ;       HL * 9
  add hl, bc      ;       HL * 10
  add hl, bc      ;       HL * 11
  ex de, hl       ; DE <- _renderer_stainCount * STAIN_SIZE

  ld iy, #_renderer_stains
  add iy, de      ; IY <- stain address
  push iy
    ld e, ENTITY_DRAWING_BOUNDS_LEFT_LO (ix)
    ld d, ENTITY_DRAWING_BOUNDS_LEFT_HI (ix)
    ld b, ENTITY_DRAWING_BOUNDS_TOP (ix)
                  ; DE <- xInBytes
                  ; B <- y

    ld STAIN_BOUNDS_LEFT_LO (iy), e
    ld STAIN_BOUNDS_LEFT_HI (iy), d

    ld STAIN_BOUNDS_TOP (iy), b

    ld a, ENTITY_DRAWING_BOUNDS_RIGHT_LO (ix)
    ld STAIN_BOUNDS_RIGHT_LO (iy), a
    ld a, ENTITY_DRAWING_BOUNDS_RIGHT_HI (ix)
    ld STAIN_BOUNDS_RIGHT_HI (iy), a

    ld a, ENTITY_DRAWING_BOUNDS_BOTTOM (ix)
    ld STAIN_BOUNDS_BOTTOM (iy), a

    push bc
    push de
      call _background_getTilePointer_asm
                  ; HL <- tile buffer address
      ld STAIN_TILE_LO (iy), l
      ld STAIN_TILE_HI (iy), h
    pop de
    pop bc

    call _renderer_getScreenPointer_asm
                  ; HL <- screen address
    ld STAIN_SCREEN_LO (iy), l
    ld STAIN_SCREEN_HI (iy), h

  pop ix          ; IX <- stain address
  ld de, #_renderer_ySortedStains
  ld hl, #_renderer_stainTopSorter
  jp _pvector_addSorted_ir_asm


; IN:
;   L <- displacement
_renderer_move::
  ld b, l         ; B <- displacement

  ld hl, (#_renderer_right)
  ld a, l
  and #0xfe
  ld l, a         ; HL <- renderer_right & 0xfffe
  push hl

  ; renderer_left += displacement;
  ld a, b
  ld hl, (#_renderer_left)
  add_hl_a
  ld (#_renderer_left), hl

  ; renderer_right += displacement;
  ld a, b
  ld hl, (#_renderer_right)
  add_hl_a
  ld (#_renderer_right), hl

  ld a, l
  and #0xfe
  ld e, a
  ld d, h         ; DE <- x1 = renderer_right & 0xfffe
  pop hl          ; HL <- x0

  ; ensuring x0 != x1
  ld a, e
  cp l
  jp nz, set_new_spans
  ld a, d
  cp h
  ret z
  jp set_new_spans


_renderer_draw::
  push ix
  push iy

  ; Get first stain
  ld hl, #_renderer_ySortedStains + PVECTOR_ITEMS
  ld a, (hl)
  inc hl
  ld b, (hl)
  inc hl

  ld__ixl_a
  ld__ixh_b       ; IX <- stain address
  ex de, hl       ; DE <- next stain pointer address

  ; checking if it is newSpansStain
  cp #<_renderer_newSpansStain
  jr nz, wait_for_semaphore
  ld a, b
  cp #>_renderer_newSpansStain
  jr nz, wait_for_semaphore

  ; then draw it before waiting for rendering semaphore (because
  ; it will lie in invisible screen zone)
  push de
    call _renderer_drawStain_asm
  pop de

  ; going to next stain
  ld a, (de)
  ld b, a
  inc de
  ld a, (de)

  ld__ixl_b
  ld__ixh_a       ; IX <- stain address
  inc de          ; DE <- next stain pointer address

wait_for_semaphore:
  call _renderer_waitForSemaphore

  ; Iterate over all drawings
  ld hl, #_renderer_ySortedDrawings + PVECTOR_ITEMS
outer_loop:
  ld c, (hl)
  inc hl
  ld b, (hl)

  ld a, b
  or c
  jr z, draw_remaining_stains

  ld__iyl_c
  ld__iyh_b       ; IY <- drawing address
  inc hl          ; HL <- next drawing pointer address

  bit #DRAWING_REDRAW_BIT, ENTITY_DRAWING_FLAGS (iy)
  jr z, outer_loop
                  ; ignore even testing drawing if it is not marked to draw

inner_loop:
  ; stain != NULL?
  ld__a_ixl
  or__ixh
  jr z, draw_drawing

  ; checking if it collides *vertically* with stain
  ; stain.bottom > drawing.top
  ld a, ENTITY_DRAWING_BOUNDS_TOP (iy)
  cp STAIN_BOUNDS_BOTTOM (ix)
  jr nc, draw_drawing
  ; stain.top < drawing.bottom
  ld a, ENTITY_DRAWING_BOUNDS_BOTTOM (iy)
  cp STAIN_BOUNDS_TOP (ix)
  jr c, draw_drawing
  ; drawing stain before drawing

  ld b, STAIN_BOUNDS_BOTTOM (ix)
  call _renderer_waitForScanline

  push de
  push hl
  push iy
    call _renderer_drawStain_asm
  pop iy
  pop hl
  pop de

  ; get next stain
  ld a, (de)
  ld b, a
  inc de
  ld a, (de)

  ld__ixl_b
  ld__ixh_a       ; IX <- stain address
  inc de          ; DE <- next stain pointer address

  jr inner_loop

draw_drawing:
  ; TODO stop if ray reached bounds? Or at least show some
  ; alert to redesign level to avoid too much drawing at
  ; the same time.

  ld b, ENTITY_DRAWING_BOUNDS_BOTTOM (iy)
  call _renderer_waitForScanline

  push de
  push hl
    call draw_entity
  pop hl
  pop de

  jr outer_loop
  
draw_remaining_stains:
  ; stain != NULL?
  ld__a_ixl
  or__ixh
  jr z, reset_stains

  ex de, hl       ; HL <- stain pointer address
stain_loop:
  push hl
    call _renderer_drawStain_asm
  pop hl

  ld c, (hl)
  inc hl
  ld b, (hl)

  ld a, b
  or c
  jr z, reset_stains

  ld__ixl_c
  ld__ixh_b
  inc hl
  jr stain_loop

  ; Reset stain list
reset_stains:
  xor a
  ld (#_renderer_stainCount), a
  ld hl, #_renderer_ySortedStains
  call _pvector_clear

.if SHOW_COLLIDABLES
  ld de, #_renderer_ySortedDrawings
  ld hl, #_renderer_drawCollidable
  call _pvector_apply_ix_asm
.endif

  pop iy
  pop ix
  ret


; IN:
;   HL <- left in bytes
;   DE <- right in bytes
set_new_spans:
  push ix

  ld ix, #_renderer_newSpansStain

  ld STAIN_BOUNDS_LEFT_LO (ix), l
  ld STAIN_BOUNDS_LEFT_HI (ix), h

  ld STAIN_BOUNDS_RIGHT_LO (ix), e
  ld STAIN_BOUNDS_RIGHT_HI (ix), d

  ex de, hl         ; DE <- x in bytes
  ld b, STAIN_BOUNDS_TOP (ix)
                    ; B <- y
  
  push de
    call _renderer_getScreenPointer_asm
                    ; HL <- screen address
    ld STAIN_SCREEN_LO (ix), l
    ld STAIN_SCREEN_HI (ix), h
  pop de
                    ; DE <- x in bytes
  ld b, STAIN_BOUNDS_TOP (ix)
                    ; B <- y
  call _background_getTilePointer_asm
                    ; HL <- tile buffer address
  ld STAIN_TILE_LO (ix), l
  ld STAIN_TILE_HI (ix), h

  ; Add as a stain
  ld de, #_renderer_ySortedStains
  ld hl, #_renderer_stainTopSorter
  call _pvector_addSorted_ir_asm

  pop ix
  ret

; IY <- entity address
draw_entity:
  res #DRAWING_REDRAW_BIT, ENTITY_DRAWING_FLAGS (iy)
                    ; reset redraw flag

  ; is it blinking?
  bit #DRAWING_BLINK_BIT, ENTITY_DRAWING_FLAGS (iy)
  jr z, continue_drawing
.equ renderer_frame, . + 1
  ld a, #0
  bit 2, a
  ret z

continue_drawing:
  ld l, ENTITY_DRAWING_SPRITE_LO (iy)
  ld h, ENTITY_DRAWING_SPRITE_HI (iy)
  inc hl
  inc hl            ; HL <- sprite address

  ld e, ENTITY_DRAWING_SCREEN_LO (iy)
  ld d, ENTITY_DRAWING_SCREEN_HI (iy)
                    ; DE <- screen address

  bit #DRAWING_TEXT_BIT, ENTITY_DRAWING_FLAGS (iy)
  jp z, _renderer_drawRlz3_asm

  ld c, l
  ld b, h           ; BC <- string address
  jp _renderer_drawString_asm


.if SHOW_COLLIDABLES
; IX <- entity
_renderer_drawCollidable::
  ld e, ENTITY_COLLIDABLE_BOUNDS_LEFT_LO (ix)
  ld d, ENTITY_COLLIDABLE_BOUNDS_LEFT_HI (ix)
                    ; DE <- xInBytes
  ld b, ENTITY_COLLIDABLE_BOUNDS_TOP (ix)
                    ; B <- y
  call _renderer_getScreenPointer_asm
                    ; HL <- left screen address
  push hl
    ld e, ENTITY_COLLIDABLE_BOUNDS_RIGHT_LO (ix)
    ld d, ENTITY_COLLIDABLE_BOUNDS_RIGHT_HI (ix)
    dec de          ; DE <- xInBytes - 1
    ld b, ENTITY_COLLIDABLE_BOUNDS_TOP (ix)
                    ; B <- y
    call _renderer_getScreenPointer_asm
    ex de, hl
                    ; DE <- right screen address
  pop hl            ; HL <- left screen address

  ld a, ENTITY_COLLIDABLE_BOUNDS_BOTTOM (ix)
  sub ENTITY_COLLIDABLE_BOUNDS_TOP (ix)
  ld b, a           ; B <- bounds height

draw_collidable_bounds:
  ld a, #0xff
  ld (hl), a
  ld (de), a
  dec b
  ret z

  renderer_next_line_hl
  renderer_next_line_de
  jr draw_collidable_bounds
.endif


_renderer_ensureFirstSync::
  xor a
  ld (#_renderer_semaphore), a
  ; jr _renderer_waitForSemaphore

; DESTROYS:
;   AF, HL
_renderer_waitForSemaphore::
  ld hl, #_renderer_semaphore
  xor a
  ; while !semaphore
check_semaphore:
  cp (hl)
  jr z, check_semaphore

  ld (hl), a      ; reset semaphore
  ret


; IN:
;   B <- y
; DESTROYS:
;   AF
_renderer_waitForScanline::
  ; TODO maybe wait for exact scanline? This will give some
  ; time, but not too much. Also it is is relatively complicated
  ; and be must be very well thought out...
  ld a, (#_renderer_bottomBand)
  cp b
  ret nc
  jr _renderer_waitForScanline
