;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "lang/macros.h.s"
.include "data/game_assets.h.s"
.include "renderer.h.s"

.module renderer

.equ MASK_1, 0xaa
.equ MASK_2, 0x55

.equ GAME_DISPLACEMENT, #0x03
.equ HUD_DISPLACEMENT, #0x21

.equ RIGHT_GRAY, 0x01
.equ LEFT_GRAY, 0x02
.equ BOTH_GRAY, 0x03

.equ RIGHT_WHITE, 0x55
.equ LEFT_WHITE, 0xaa
.equ BOTH_WHITE, 0xff

.equ LEFT_GRAY_RIGHT_WHITE, (LEFT_GRAY | RIGHT_WHITE)
.equ LEFT_WHITE_RIGHT_GRAY, (LEFT_WHITE | RIGHT_GRAY)

.macro back_to_loop
  jp (ix)
  ; 8c, 2 bytes
.endm

.macro draw_masked mask, value
  ld a, (hl)    ; 7c
  and #mask     ; 7c
  or #value     ; 7c
  ld (hl), a    ; 7c
  ; 24c
  back_to_loop  ; 8c
  ; 26c, 8 bytes
.endm

.macro draw_3 value
  ld (hl), #value ; 10c
  back_to_loop    ; 8c
  ; 18c, 4 bytes
.endm

_renderer_drawChar3::
  ; Note: this is not callable. It's defined only for stats purposes, so
  ; I can now exactly how much space these functions take.

draw_tt:
  back_to_loop

draw_tg:
  draw_masked MASK_1, RIGHT_GRAY

draw_tw:
  draw_masked MASK_1, RIGHT_WHITE

draw_gt:
  draw_masked MASK_2, LEFT_GRAY

draw_gg:
  draw_3 BOTH_GRAY

draw_gw:
  draw_3 LEFT_GRAY_RIGHT_WHITE

draw_wt:
  draw_masked MASK_2, LEFT_WHITE

draw_wg:
  draw_3 LEFT_WHITE_RIGHT_GRAY

draw_ww:
  draw_3 BOTH_WHITE

do_draw_char:
  push ix
  ld b, #FONT_HEIGHT
                ; B <- height counter

loop:
  ld ix, #next_byte
execute_command:
  ld a, (de)
  inc de
  ld (#char_command_address), a
.equ char_command_address, . + 1
  jr__0

next_byte:
  inc l
  ld ix, #next_line
  jr execute_command

next_line:
  dec l
.equ continue_displacement, . + 1
  djnz hud_continue

  pop ix
  ret

continue:
  renderer_next_line_hl
  jr loop

hud_continue:
  bit 3, h
  jr z, hud_next_line_odd 

  ; next line (add -0x800 + 0x64)
  res 3, h
  ld a, #RENDERER_HUD_WIDTH_IN_BYTES
  add l
  ld l, a
  ld a, h
  adc #0
  ld h, a
  jr loop

hud_next_line_odd:
  ;; next line (add 0x800)
  set 3, h
  jr loop

  ;; DE <- sprite address
  ;; HL <- screen address
_renderer_drawChar3_asm::
  ld a, #GAME_DISPLACEMENT
  ld (#continue_displacement), a
  jr do_draw_char

  ;; DE <- sprite address
  ;; HL <- screen address
_renderer_drawHudChar3_asm::
  ld a, #HUD_DISPLACEMENT
  ld (#continue_displacement), a
  jr do_draw_char
