;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "data/game_assets.h.s"
.include "renderer.h.s"
.include "lang/macros.h.s"
.include "lang/debug.h.s"

.module renderer

.globl _renderer_drawTileSpan_asm
.globl _renderer_drawTerrainSpan_asm

.equ TERRAIN_HEIGHTS_HI_ADDRESS, >TERRAIN_HEIGHTS_ADDRESS
.equ RENDERER_TILE_BUFFER_HI_MASK, >TILE_BUFFER_ADDRESS | 0x03

; _renderer_drawStain::
;   push ix
;   push iy
;   ex de, hl
;   ld__ixl_e
;   ld__ixh_d
;   call _renderer_drawStain_asm
;   pop iy
;   pop ix
;   ret

_renderer_drawStain_asm::
; IN:
;   IX <- stain address
  ld l, STAIN_BOUNDS_RIGHT_LO(ix)
  ld h, STAIN_BOUNDS_RIGHT_HI(ix)

  ld e, STAIN_BOUNDS_LEFT_LO(ix)
  ld d, STAIN_BOUNDS_LEFT_HI(ix)

  or a        ; reset carry flaf
  sbc hl, de  ; HL <- width in bytes

  rr h        ; (no carry set so we don't mind about H final value)
  rr l        ; L <- width in chars
  ld b, l     ; B <- width in chars

  ld e, STAIN_SCREEN_LO(ix)
  ld d, STAIN_SCREEN_HI(ix)

  ld l, STAIN_TILE_LO(ix)
  ld h, STAIN_TILE_HI(ix)

  ld a, STAIN_BOUNDS_LEFT_LO(ix)
  or a        ; clear carry flag
  rla         ; A <- left in pixels = terrain index

  exx
    ld h, #TERRAIN_HEIGHTS_HI_ADDRESS
    ld l, a   ; HL <- terrain heights address
  exx

loop:
  push bc
  push de
  push hl
    call draw_stain_span
  pop hl
  pop de
  pop bc

  dec b
  ret z

  ;; next screen char column
  inc e
  renderer_next_byte_de

  ;; next tile column
  ld a, #RENDERER_GAME_HEIGHT_IN_CHARS
  add l
  ld l, a
  ld a, h
  adc #0
  and #RENDERER_TILE_BUFFER_HI_MASK
  ld h, a

  jr loop

draw_stain_span:
; IN:
;   HL <- tile address
;   DE <- screen address
  exx
    ld b, (hl)  ; B <- first height value
    ld c, b     ; C <- first height value
    ld d, l     ; D <- store index for later

    inc l
    ld a, (hl)  ; A <- second pixel height
    cp b        ; test second pixel with max
    jr c, continue_finding_range1
    ld b, a
continue_finding_range1:
    cp c        ; test second pixel with min
    jr nc, continue_finding_range2
    ld c, a
continue_finding_range2:
    inc l
    ld a, (hl)  ; A <- third pixel height
    cp b        ; test third pixel with max
    jr c, continue_finding_range3
    ld b, a
continue_finding_range3:
    cp c        ; test third pixel with min
    jr nc, continue_finding_range4
    ld c, a
continue_finding_range4:
    inc l
    ld a, (hl)  ; A <- fourth pixel height
    cp b        ; test fourth pixel with max
    jr c, continue_finding_range5
    ld b, a
continue_finding_range5:
    cp c        ; test fourth pixel with min
    jr nc, continue_finding_range6
    ld c, a
continue_finding_range6:
    inc l       ; HL <- next position heights address
    ; B/C <- max/min terrain height
    ld a, c     ; A <- min terrain height
  exx

                ; A <- min terrain height
  cp STAIN_BOUNDS_BOTTOM(ix)
  jr c, draw_tiles_and_terrain

  ; draw complete tile span without terrain
  ld a, STAIN_BOUNDS_BOTTOM(ix)
  sub STAIN_BOUNDS_TOP(ix)
  jp _renderer_drawTileSpan_asm

draw_tiles_and_terrain:
  exx
    ld a, b     ; A <- max terrain height
  exx
  cp STAIN_BOUNDS_BOTTOM(ix)
  jr c, continue
  ld a, STAIN_BOUNDS_BOTTOM(ix)
                ; A <- min(max terrain height, stain bounds bottom)

continue:
  add #7
  and #0xf8   ; A <- tiles bottom
  sub STAIN_BOUNDS_TOP(ix)

  jr c, draw_terrain

  push de
  push hl
    call _renderer_drawTileSpan_asm
  pop hl
  pop de

draw_terrain:
  exx
    ld a, STAIN_BOUNDS_BOTTOM(ix)
    ld__iyl_a ; IYL <- bottom height
    ld__iyh_b ; IYH <- max terrain height

    push hl
      ld l, d
      push ix   ; TODO It'd be nice to not use this in call
      call _renderer_drawTerrainSpan_asm
      pop ix
    pop hl
  exx
  ret
