;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "data/game_assets.h.s"
.include "renderer.h.s"
.include "lang/macros.h.s"
.include "lang/debug.h.s"

.module renderer

.equ TERRAIN_HEIGHTS_HI_ADDRESS, >TERRAIN_HEIGHTS_ADDRESS
.equ TERRAIN_SCREENS_HI_ADDRESS, >TERRAIN_SCREENS_ADDRESS
.equ TERRAIN_TEXTURE_HI_ADDRESS, >TERRAIN_TEXTURE_ADDRESS

.equ LEFT_MASK, #0x55
.equ RIGHT_MASK, #0xaa

.equ DEC_L_OPCODE, #0x2d
.equ NOP_OPCODE, #0x00

_renderer_drawTerrainSpan_asm::
; IN:
;   L <- x in pixels
;   IYL <- max height value to draw
;   IYH <- max terrain height
  ld a, l       ; L/A <- x in pixels

  exx
    ld h, #TERRAIN_TEXTURE_HI_ADDRESS
    ; ld l, a

    ld b, a     ; B' <- first byte, left pixel index
    inc a
    ld c, a     ; C' <- first byte, right pixel index
    inc a
    ld d, a     ; D' <- second byte, left pixel index
    inc a
    ld e, a     ; E' <- second byte, right pixel index
  exx

  ld h, #TERRAIN_HEIGHTS_HI_ADDRESS
                ; HL <- heights address

  ld b, (hl)    ; B <- first byte, left pixel height
  inc l
  push hl
    ld c, (hl)    ; C <- first byte, right pixel height
    dec l

    call draw_pixel_column
  pop hl
  inc l
  ld b, (hl)    ; B <- second byte, left pixel height
  inc l
  ld c, (hl)    ; C <- second byte, right pixel height
  dec l

  exx
    push bc
    ld b, d
    ld c, e
  exx
  call draw_pixel_column
  exx
    ld d, b
    ld e, c
    pop bc
  exx

  ld__a_iyl
  sub__iyh
  ret c
  ret z

  ld b, a
char_loop:
  exx
    ld l, e
    inc e
    ld a, (hl)
    rrca        ; A <- right texture pixel

    ld l, d
    inc d
    or (hl)     ; A <- left | right texture pixel
  exx
  ld (de), a
  dec e
  exx
    ld l, c
    inc c
    ld a, (hl)
    rrca        ; A <- right texture pixel

    ld l, b
    inc b
    or (hl)     ; A <- left | right texture pixel
  exx
  ld (de), a
  dec b
  ret z

  inc e
  renderer_next_line_de

  jr char_loop

draw_pixel_column:
; IN:
;   B <- left pixel height
;   C <- right pixel height
;   B' <- left pixel index
;   C' <- right pixel index
;   L <- heights/screens/texture index
; OUT:
;   DE <- next screen pointer
  ld h, #TERRAIN_SCREENS_HI_ADDRESS
                ; HL <- screen address address
  ld e, (hl)
  inc l
  ld d, (hl)    ; DE <- screen address
  dec l

  ld a, b
  sub c         ; A <- first - second
  jr nz, continue_pixel_column

  ; B and C are equal
  ld__a_iyh     ; A <- max terrain height value
  sub b         ; A <- byte column counter
  or a
  ret z

  ld b, a       ; B <- byte column counter
  jr byte_loop

continue_pixel_column:
  ; set texture address
  ld h, #TERRAIN_TEXTURE_HI_ADDRESS
                ; Hx <- texture address
  jr c, draw_left_pixel

; draw_right_pixel:
  ld c, a       ; C <- temporal counter
  ; update byte column counter
  ld__a_iyh
  sub b
  ld__ixh_a     ; IXH <- byte column counter
  inc l         ; HL <- right pixel texture address
  ld b, c       ; B <- counter
  ; update exx texture indices
  ld a, b
  exx
    add c
    ld c, a
  exx
right_pixel_loop:
  ld c, (hl)
  inc l
  srl c         ; C <- right texture pixel
  ld a, (de)
  and #RIGHT_MASK
                ; A <- screen byte masked
  or c          ; A <- right texture pixel | screen byte masked
  ld (de), a
  renderer_next_line_de
  djnz right_pixel_loop

  dec l         ; restore index
  jr draw_bytes

draw_left_pixel:
  neg           ; A <- -A
  ld b, a       ; B <- counter
  ; update byte column counter
  ld__a_iyh
  sub c
  ld__ixh_a     ; IXH <- byte column counter
  ld c, #LEFT_MASK
                ; C <- preload mask
  ; update exx texture indices
  ld a, b
  exx
    add b
    ld b, a
  exx
left_pixel_loop:
  ld a, (de)
  and c
  or (hl)
  inc l
  ld (de), a
  renderer_next_line_de
  djnz left_pixel_loop

draw_bytes:
  ld__a_ixh
  or a
  ret z

  ld b, a
byte_loop:
  exx
    ld l, c
    inc c
    ld a, (hl)
    rrca        ; A <- right texture pixel

    ld l, b
    inc b
    or (hl)     ; A <- left | right texture pixel
  exx
  ld (de), a
  renderer_next_line_de
  djnz byte_loop

  ret
