;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "data/game_assets.h.s"
.include "renderer.h.s"
.include "lang/macros.h.s"

.module renderer

.equ RENDERER_TILE_BUFFER_HI_MASK, >TILE_BUFFER_ADDRESS | 0x03

_renderer_drawTileSpan_asm::
; IN:
;   HL <- tile address
;   DE <- screen address
;   A <- height in pixels
; DESTROYS:
;   AF, BC, DE, HL
  rra
  rra
  rra
  and #0x1f   ; A <- height in tiles

  ret z       ; Early escape for 0 height

  ld b, a     ; B <- height in tiles

loop:
  ld a, (hl)  ; A <- tile color or ((tile index << 1) & 1)
  bit 0, a
  jr nz, draw_tile

  ; draw color pattern
  ld (de), a
  inc e
  ld (de), a
  set 5, d
  ld (de), a
  dec e
  ld (de), a
  set 3, d
  ld (de), a
  inc e
  ld (de), a
  res 5, d
  ld (de), a
  dec e
  ld (de), a
  set 4, d
  ld (de), a
  inc e
  ld (de), a
  set 5, d
  ld (de), a
  dec e
  ld (de), a
  res 3,d
  ld (de), a
  inc e
  ld (de), a
  res 5, d
  ld (de), a
  dec e
  ld (de), a
  djnz next_line
  ret

draw_tile:
  push hl
  push bc
  ld hl, #_game_tilesetIndices
  res 0, a
  add l
  ld l, a
  ld a, h
  adc #0
  ld h, a   ; HL <- tile index

  ld a, (hl)
  inc hl
  ld h, (hl)
  ld l, a   ; HL <- tile address

  ldi
  ld a, (hl)
  ld (de), a
  inc l
  set 5, d
  ld a, (hl)
  ld (de), a
  inc l
  dec e
  ld a, (hl)
  ld (de), a
  inc l
  set 3, d
  ldi
  ld a, (hl)
  ld (de), a
  inc l
  res 5, d
  ld a, (hl)
  ld (de), a
  inc l
  dec e
  ld a, (hl)
  ld (de), a
  inc l
  set 4, d
  ldi
  ld a, (hl)
  ld (de), a
  inc l
  set 5, d
  ld a, (hl)
  ld (de), a
  inc l
  dec e
  ld a, (hl)
  ld (de), a
  inc l
  res 3, d
  ldi
  ld a, (hl)
  ld (de), a
  inc l
  res 5, d
  ld a, (hl)
  ld (de), a
  inc l
  dec e
  ld a, (hl)
  ld (de), a
  pop bc
  pop hl

tile_drawn:
  djnz next_line
  ret

next_line:
  ; next tile
  inc hl
  ld a, h
  and #RENDERER_TILE_BUFFER_HI_MASK
  ld h, a

  ; next screen line
  res 4, d
  ld a, #RENDERER_GAME_WIDTH_IN_BYTES
  add e
  ld e, a
  jp nc, loop
  inc d
  res 3, d
  jp loop
