;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "data/game_assets.h.s"
.include "renderer.h.s"

.module renderer

.equ TABLE_HI_ADDRESS, >SCREEN_POINTER_TABLE_ADDRESS

_renderer_getScreenPointer::
  pop hl      ;; HL <- return address
  pop de      ;; DE <- xInBytes
  dec sp
  pop bc      ;; B = y
  push hl     ;; Put returning address in the stack again

; In:
;   DE <- xInBytes
;   B <- y
; Out:
;   HL <- screen address
; Destroys:
;   AF, BC, HL (keeps DE untouched)
_renderer_getScreenPointer_asm::
; http://www.cpcwiki.eu/forum/programming/hardware-scrolling-problem-with-spritestiles/msg910/#msg910
; u16 bits_0_10 = (drawer_screenPointerTable[y >> 3] + xInBytes) & 0x07ff;
  ld a, b
  rra
  rra
  and #0x3e   ; A <- (y >> 3) * 2

  ld l, a
  ld h, #TABLE_HI_ADDRESS
              ; HL <- screen pointer table address

  ld c, (hl)
  inc hl
  ld h, (hl)
  ld l, c     ; HL <- drawer_screenPointerTable[y >> 3]

  add hl, de  ; HL <- drawer_screenPointerTable[y >> 3] + xInBytes

  ld a, h
  and #0x07
  ld h, a     ; HL <- HL & 0x07ff

; u16 bits_11_13 = (y % 8) << 11;
  ld a, b
  and #0x07
  rlca
  rlca
  rlca        ; A* <- (y % 8) << 11

; return (u8*)(bits_0_10 + bits_11_13 + RENDERER_GAME_ADDRESS);
  ld bc, #RENDERER_GAME_ADDRESS
  add hl, bc  ; HL <- bits_0_10 + RENDERER_GAME_ADDRESS
  add h
  ld h, a     ; HL <- bits_0_10 + bits_11_13 + RENDERER_GAME_ADDRESS

  ret
