;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "renderer.h.s"
.include "data/level.h.s"
.include "data/game_assets.h.s"
.include "lang/macros.h.s"
.include "lang/math.h.s"
.include "lang/debug.h.s"

.module terrain

.globl __muluschar
.globl __mulint
.globl __divuint
; TODO these functions abobe are wrappers that deal with
; stack to be callable from C code. But as we call them
; from ASM, we could call their actual implementations
; directly using registers as parameters.
.globl ___sdcc_call_iy

.globl cpct_getRandom_mxor_u8_asm   ; TODO this will be extracted
                                    ; to a random module?

.equ LOOK_AHEAD, 128 - RENDERER_GAME_WIDTH_IN_BYTES
.equ RANDOM_FIRST_HEIGHT, 100
.equ MAX_HEIGHT, 160

.equ RANDOM_MIN_WIDTH, 8
.equ RANDOM_MAX_WIDTH, 40
.equ RANDOM_MIN_HEIGHT, 64
.equ RANDOM_MAX_HEIGHT, 144

.equ SCREENS_HI_ADDRESS, >TERRAIN_SCREENS_ADDRESS
.equ HEIGHTS_HI_ADDRESS, >TERRAIN_HEIGHTS_ADDRESS

.globl _terrain_initTexture

_terrain_level::
  .dw #0
_terrain_index::
  .db #0
_terrain_right::
  .dw #0

_terrain_width::
  .db #0

; IN
;   HL <- level address
_terrain_clear::
  ex de, hl       ; DE <- level address

  ; terrain_nextSection = terrain_nextLevelSection
  ld hl, #_terrain_nextLevelSection
  call _terrain_clear_common
                  ; IY <- level address

  exx
  ; firstHeight = level->terrainStart
    ld l, LEVEL_TERRAIN_START (iy)
  exx

  ; find level pointer through renderer left position
  ld hl, (#_renderer_left)
  ld de, (#_terrain_level)
  ld b, #0
find_start_loop:
  ld a, l
  or h
  jr z, level_start_found

  ld a, (de)      ; A <- section width
  or a
  rra
  ld c, a
  sbc hl, bc
  inc de
  ld a, (de)      ; A <- section height diff
  exx
    bit 7, a
    jr nz, update_start_height
    dec a         ; take care of non-negative values
update_start_height:
    add l         ; L <- first terrain height
    ld l, a
  exx
  inc de          ; DE <- next level section address
  jr find_start_loop

level_start_found:
  ld (#_terrain_level), de
                  ; Update terrain level address variable
  exx
    ld a, l
  exx
  ; set first level section configuration
  jp terrain_nextLevelSection_asm


; IN:
;   HL <- level address
_terrainRandom_clear::
  ex de, hl       ; DE <- level address

  ; terrain_nextSection = terrain_nextLevelSection
  ld hl, #_terrain_nextRandomSection
  call _terrain_clear_common

  ; firstHeight = RANDOM_FIRST_HEIGHT
  ld l, #RANDOM_FIRST_HEIGHT
  jp _terrain_nextRandomSection


  ; DE <- level address
  ; HL <- nextSection function address
_terrain_clear_common::
  ld (#terrain_next_section), hl
  
  ld__iyl_e
  ld__iyh_d       ; IY <- level address

  ; terrain_level = level->terrain
  ld l, LEVEL_TERRAIN_LO (iy)
  ld h, LEVEL_TERRAIN_HI (iy)
  ld (#_terrain_level), hl

  ; terrain_right = renderer_left
  ld hl, (#_renderer_left)
  ld (#_terrain_right), hl
  ; terrain_index = (u8)(terrain_right << 1)
  sla l
  ld a, l
  ld (#_terrain_index), a

  ; init texture
  ld l, LEVEL_TEXTURE_LO (iy)
  ld h, LEVEL_TEXTURE_HI (iy)
  jp _terrain_initTexture


_terrain_step::
  ; index = renderer_right + LOOK_AHEAD
  ld hl, (#_renderer_right)
  ld bc, #LOOK_AHEAD
  add hl, bc
  ld c, l
  ld b, h         ; BC <- renderer_right + LOOK_AHEAD

  ; while (terrain_right < index)
  ld hl, (#_terrain_right)
                  ; HL <- terrain_right
step_loop:
  ld a, l
  sub c
  ld a, h
  sbc b
  ret nc
  
  ; a = terrain_generate()
  push bc
  push hl
    push hl
      call _terrain_generate
      push hl
        call _terrain_generate
        ld a, l
      pop hl      ; L <- first pixel terrain height
      ld h, a     ; H <- second pixel terrain height
    pop de        ; DE <- xInBytes

    ld a, h
    cp l
    jr c, get_byte_screen_pointer
    ld a, l       ; A <- min(first and second pixel terrain height)

get_byte_screen_pointer:
                  ; DE <- xInBytes
    ld b, a       ; B <- y
    push hl
      call _renderer_getScreenPointer_asm
      ex de, hl   ; DE <- screen address

      ; screens[terrain_index >> 1] = screen
      ld a, (#_terrain_index)
      ld b, a     ; store terrain_index for later
      and #0xfe
      ld l, a
      ld h, #SCREENS_HI_ADDRESS
      ld (hl), e
      inc hl
      ld (hl), d
    pop de        ; D <- first pixel terrain height
                  ; E <- second pixel terrain height

    ; heights[terrain_index++] = MATH_MIN(a, MAX_HEIGHT)
    ld l, b       ; L <- terrain index
    ld h, #HEIGHTS_HI_ADDRESS
    ld a, e
    cp #MAX_HEIGHT
    jr c, store_first_height
    ld a, #MAX_HEIGHT
store_first_height:
    ld (hl), a

    ; heights[terrain_index++] = MATH_MIN(b, MAX_HEIGHT)
    inc l
    ld a, d
    cp #MAX_HEIGHT
    jr c, store_second_height
    ld a, #MAX_HEIGHT
store_second_height:
    ld (hl), a
    inc l
    ld a, l
    ld (#_terrain_index), a
  pop hl
  pop bc

  ; _terrain_right++
  inc hl
  ld (#_terrain_right), hl
  jr step_loop


_terrain_generate::
  ; fixed fy = (i8)terrain_height * math_sin(terrain_angle >> 8)
  ld a, (#terrain_angle + 1)
  ld l, a
  call _math_sin  ; L <- math_sin(terrain_angle >> 8)

  ld h, l         ; H <- unsigned char
.equ terrain_height, . + 1
  ld l, #0        ; L <- signed char
  push hl
    call __muluschar
                  ; HL <- fy
  pop af

  ; height = terrain_y0 + (fy >> 8)
.equ terrain_y0, . + 1
  ld a, #0
  add h
  ld d, a         ; D <- height (result to return)

  ; terrain_angle += terrain_delta
.equ terrain_angle, . + 1
  ld hl, #0
.equ terrain_delta, . + 1
  ld bc, #0
  add hl, bc
  ld (#terrain_angle), hl

  ; --terrain_width == 0?
  ld hl, #_terrain_width
  dec (hl)
  ld l, d         ; L <- height (result)
  ret nz

  ; generate next section
  push hl
    ; call next section function
.equ terrain_next_section, . + 2
    ld iy, #0
    call ___sdcc_call_iy
  pop hl
  ret


; IN:
;   L <- height
_terrain_nextLevelSection::
  ; terrain_y0 = height
  ld a, l
; IN:
;   A <- height
terrain_nextLevelSection_asm:
  ld (#terrain_y0), a

  ; terrain_width = *(terrain_level++)
  ld hl, (#_terrain_level)
  ld a, (hl)
  ld c, a         ; C <- height for later
  ld (#_terrain_width), a
  inc hl

  ; terrain_height = *(terrain_level++)
  ld a, (hl)
  ld (#terrain_height), a
  inc hl
  ld (#_terrain_level), hl

  ; C <- terrain_width
update_delta_and_angle:
  ; terrain_delta = 0xff00 / terrain_width
  ld b, #0        ; BC <- (u16)terrain_width
  push bc
  ld hl, #0xff00
  push hl
  call __divuint
  pop af
  pop af
  ; TODO this div has some considerations:
  ; - numerator is actually a byte (not a word)
  ; - denominator is the constant 0xff00
  ; - there is no chance of division by zero case (terrain_width > 1)
  ; As this is the only division in the project, it'd be nice to
  ; take SDCC's `divu16` code and addapt to this special case.
  ; Code at tools/sdcc-3.6.8-r9946/src/device/lib/gbz80/div.s:359
  ld (#terrain_delta), hl

  ; terrain_angle = 0
  ld hl, #0
  ld (#terrain_angle), hl
  ret


; IN
;   L <- height
_terrain_nextRandomSection::
  ; terrain_y0 = height
  ld a, l
  ld (#terrain_y0), a

  push hl
    ; terrain_width = random_getBetween(RANDOM_MIN_WIDTH, RANDOM_MAX_WIDTH)
    ld bc, #(RANDOM_MIN_WIDTH << 8) + #RANDOM_MAX_WIDTH
    call _random_getBetween_asm
    ld a, l
    ld (#_terrain_width), a
  pop hl

  ; y0 = MATH_MAX(RANDOM_MIN_HEIGHT, height - terrain_width)
  ld h, a         ; H <- save terrain_width for later
  neg             ; A <- -terrain_width
  add l           ; A <- height - terrain_width
  ld b, a         ; B <- height - terrain_width
  cp #RANDOM_MIN_HEIGHT
  jr nc, get_y1
  ld b, #RANDOM_MIN_HEIGHT
                  ; B <- MATH_MAX(RANDOM_MIN_HEIGHT, height - terrain_width)

get_y1:
  ; y1 = MATH_MIN(RANDOM_MAX_HEIGHT, height + terrain_width * 2)
  sla h
  ld a, h
  add l
  ld c, a         ; C <- height + terrain_width * 2
  cp #RANDOM_MAX_HEIGHT
  jr c, get_ny
  ld c, #RANDOM_MAX_HEIGHT
                  ; C <- MATH_MIN(RANDOM_MAX_HEIGHT, height + terrain_width * 2)

get_ny:
                  ; B <- y0
                  ; C <- y1
  push hl
    call _random_getBetween_asm
                  ; L <- ny
  
  ; terrain_height = ny - height
  pop de          ; E <- height
  ld a, l

  sub e
  ld (#terrain_height), a

  ld a, (#_terrain_width)
  ld c, a         ; TODO could be saved from before
  jr update_delta_and_angle


; IN:
;   HL <- xInBytes
; OUT:
;   L <- platform height
; DESTROYS:
;   none
_terrain_getHeight::
  sla l
  ld h, #HEIGHTS_HI_ADDRESS
  ld l, (hl)
  ret


; IN:
;   B <- min
;   C <- max
; OUT:
;   L <- random number between a and b
; TODO move this to random module
_random_getBetween_asm::
  ; min + ((random * (max - min)) >> 8)
  ld a, c
  sub b
  ld c, a         ; C <- (max - min)
  push bc
    call cpct_getRandom_mxor_u8_asm
                  ; L <- random
  pop bc

  push bc
    ld h, #0      ; HL <- (u16)random
    push hl
    ld l, c       ; HL <- (u16)(max - min)
    push hl
    call __mulint ; HL <- (random * (max - min))
; TODO look for muluchar or similar for this
    pop af
    pop af
  pop bc

  ld a, h         ; A <- (random * (max - min)) >> 8
  add b           ; A <- min + ((random * (max - min)) >> 8)
  ld l, a
  ret


; IN:
;   HL <- texture data address
_terrain_initTexture::
  ld de, #TERRAIN_TEXTURE_ADDRESS

texture_loop:
  ld a, (hl)  ; A <- counter
  or a
  ret z

  ld b, a     ; B <- counter
  inc hl
  ld a, (hl)  ; A <- color index
texture_inner_loop:
  ld (de), a
  inc e
  djnz texture_inner_loop

  inc hl
  jr texture_loop
