;; The Abduction of Oscar Z
;; ========================
;;
;; "Oscar loved watching night skies long after his parents slept. But tonight
;; that sky was watching him..."
;;
;; * PRODUCTION & CODE / Tulo / @tulinmola
;; * ART & STORY / Alberto Hache / @Alberto_H9
;; * MUSIC & FX / Gryzor87 / @Gryzor87
;;
;; A 2020 Dreamin`Bits production
;; https://dreaminbits.com/
;;
;; MIT License
;;
;; Copyright (c) 2020 Dreamin`Bits
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in all
;; copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
;; SOFTWARE.

.include "renderer.h.s"
.include "background.h.s"
.include "lang/macros.h.s"
.include "data/level.h.s"
.include "data/game_assets.h.s"

.module water

.equ TILE_BUFFER_HI_ADDRESS, >TILE_BUFFER_ADDRESS

_water_level::
  .dw 0
_water_counter::
  .db 0
_water_height::
  .db 0
_water_index::
  .dw 0
_water_background::
  .dw 0

; IN:
;   HL <- level address
; DESTROYS:
;   AF, BC, DE, HL
_water_clear::
  ex de, hl       ; DE <- level address
  ld__iyl_e
  ld__iyh_d       ; IY <- level address

  ; water_level = level->water;
  ld e, LEVEL_WATER_LO (iy)
  ld d, LEVEL_WATER_HI (iy)
                  ; DE <- level platform data

  ; water_tile = level->waterColor;
  ld a, LEVEL_WATER_COLOR (iy)
  ld (#_water_tile), a

  ; water_index = renderer_left >> 1;
  ld hl, (#_renderer_left)
  srl h
  rr l
  ld (#_water_index), hl

  ; find level start
  ld b, #0
find_start_loop:
  ld a, (de)    ; A <- *water_level
  ld c, a       ; BC <- counter
  or a
  sbc hl, bc    ; HL -= counter
  jr c, found_start
  inc de        ; skip counter
  inc de        ; skip height
  jr find_start_loop

found_start:
  add hl, bc
  sub l

  ; water_counter = current counter - diff
  ld (#_water_counter), a
  inc de
  ; update water_height
  ld a, (de)
  ld (#_water_height), a
  inc de
  ; save water level
  ld (#_water_level), de

  ; water_background = background_getTilePointer(renderer_left, 0)
  ld de, (#_renderer_left)
                ; DE <- xInBytes
                ; B <- y = 0 from previous ld b, #0
  call _background_getTilePointer_asm
  ld (#_water_background), hl

  ret


; void water_step_C() {
;   u16 index = renderer_right >> 1;
;   while (water_index < index) {
;     water_generate();
;     water_drawColumn();
;
;     water_index++;
;   }
; }
_water_step::
  ; u16 index = renderer_right >> 1;
  ld bc, (_renderer_right)
  srl b
  rr c          ; BC <- renderer_right >> 1

  ld hl, (#_water_index)
                ; HL <- water index

  ; while (water_index < index) {
step_loop:
  ld a, l
  sub c
  ld a, h
  sbc b
  ret nc

  push bc
  push hl
  call _water_generate
  call _water_drawColumn
  pop hl
  pop bc

  ; water_index++;
  inc hl
  ld (#_water_index), hl
  jr step_loop


_water_drawColumn::
  ld hl, (#_water_background)
                ; HL <- tile buffer pointer
  ld de, #0x0300 + TILE_BUFFER_HI_ADDRESS
                ; DE <- constants for optimizing bitwise operations

  ; u8* water = (u8*)WATER_HEIGHTS_ADDRESS;
  ; u8 waterHeight = water[water_index & 0x3f] >> 3;
  ld a, (#_water_index)
  and #0x3f
  ld bc, #WATER_HEIGHTS_ADDRESS
  add c
  ld c, a       ; HL <- &water[water_index % 64]
  ld a, (bc)

  or a
  jr nz, draw_with_water

  ld bc, #RENDERER_GAME_HEIGHT_IN_TILES << 8
                ; B <- RENDERER_GAME_HEIGHT_IN_TILES
                ; C <- 0

; B <- tile counter
; C <- tile to draw
draw_tiles:
  ld (hl), c
  inc hl

  ld a, h
  and d         ; and #0x03
  or e          ; or #TILE_BUFFER_HI_ADDRESS
  ld h, a       ; HL <- next tile buffer address
                ; taking care of tile buffer 1K overflow

  djnz draw_tiles

  ld (#_water_background), hl
  ret

draw_with_water:
  ; A <- height
  rra           ; Note height 3 LSBs are always 0 and !C (because of previous 'or a')
  rra
  rra           ; A <- water[water_index & 0x3f] >> 3

  push af
  ld b, a
  ld c, #0      ; draw background tiles (0)
  call draw_tiles
  pop af

  neg
  add #RENDERER_GAME_HEIGHT_IN_TILES
                ; A <- RENDERER_GAME_HEIGHT_IN_TILES - current height
  ld b, a
.equ _water_tile, . + 1
  ld a, #0
  ld c, a       ; draw water tiles
  jp draw_tiles


; u8 water_getHeight_C(u16 xInBytes) {
;   u8* heights = (u8*)WATER_HEIGHTS_ADDRESS;
;   return heights[(xInBytes >> 1) & 0x3f];
; }
; IN:
;   HL <- xInBytes
; OUT:
;   L <- height value
; DESTROYS:
;   AF, HL
_water_getHeight::
  ld a, l
  rra
  and #0x3f
  ld hl, #WATER_HEIGHTS_ADDRESS
  add l
  ld l, a       ; HL <- &water[(xInBytes >> 1) % 64]
  ld l, (hl)
  ret

; void water_generate_C() {
;   u8* heights = (u8*)WATER_HEIGHTS_ADDRESS;
;
;   if (--water_counter == 0) {
;     water_counter = *water_level++;
;     water_height = *water_level++;
;   }
;
;   heights[water_index & 0x3f] = water_height;
; }
_water_generate::
  ; if (--water_counter == 0) {
  ld hl, #_water_counter
  dec (hl)
  jr nz, update_height

  ; water_counter = *water_level++
  ld hl, (#_water_level)
  ld a, (hl)
  ld (#_water_counter), a
  inc hl
  ; water_height = *water_level++
  ld a, (hl)
  ld (#_water_height), a
  inc hl
  ld (#_water_level), hl

update_height:
  ; heights[water_index & 0x3f] = water_height;
  ld a, (#_water_index)
  and #0x3f
  ld hl, #WATER_HEIGHTS_ADDRESS
  add l
  ld l, a       ; HL <- &water[water_index % 64]

  ld a, (#_water_height)
  ld (hl), a    ; heights[water_index & 0x3f] = water_height;

  ret
