/*
  The Abduction of Oscar Z
  ========================

  "Oscar loved watching night skies long after his parents slept. But tonight
  that sky was watching him..."

  * PRODUCTION & CODE / Tulo / @tulinmola
  * ART & STORY / Alberto Hache / @Alberto_H9
  * MUSIC & FX / Gryzor87 / @Gryzor87

  A 2020 Dreamin`Bits production
  https://dreaminbits.com/

  MIT License

  Copyright (c) 2020 Dreamin`Bits

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the"Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#include <cpctelera.h>
#include "renderer/renderer.h"
#include "system/firmware.h"
#include "lang/bool.h"
#include "data/screens_assets.h"

#define LOGO_WAIT       175
#define EXPERIENCE_WAIT 250
#define BLACK_WAIT      50
#define SCREEN_WAIT     150

#define SPRITE(name, index) (void*)(name ## _ ## index ## _ADDRESS)
#define SCREEN(xInBytes, y) cpctm_screenPtr(CPCT_VMEM_START, xInBytes, y)
#define DRAW_INDEXED_SPRITE(name, index, xInBytes, y) cpct_drawSprite(SPRITE(name, index), SCREEN(xInBytes, y), name ## _WIDTH_IN_BYTES, name ## _HEIGHT)
#define DRAW_SPRITE(name, xInBytes, y) DRAW_INDEXED_SPRITE(name, 0, xInBytes, y)

bool wait(u16 frames);
void showAndWait(u16 frames);

// mix code.mmap ../screens/obj/screens.map -i _screens_assets
// mix assets.create ../art/screens.json -o ../screens/src/data -c
// cloc --exclude-dir=data src/
void main(void) {
  firmware_disable();
  renderer_init();
  screens_assets_init();

  renderer_clear();
  renderer_fadeIn();

  DRAW_SPRITE(LOGO, 31, 73);
  showAndWait(LOGO_WAIT);

  DRAW_SPRITE(EXPERIENCE, 24, 92);
  DRAW_SPRITE(HEADPHONES, 20, 152);
  showAndWait(EXPERIENCE_WAIT);

  DRAW_INDEXED_SPRITE(LOAD_SCREEN, 0, 0, 0);
  DRAW_INDEXED_SPRITE(LOAD_SCREEN, 1, LOAD_SCREEN_WIDTH_IN_BYTES, 0);
  renderer_fadeIn();

  wait(SCREEN_WAIT);

  firmware_reenable();
}

bool wait(u16 frames) {
  while (frames--) {
    cpct_scanKeyboard();
    if (cpct_isAnyKeyPressed()) {
      return false;
    }

    __asm__("halt");
    __asm__("halt");
    cpct_waitVSYNC();
  }

  return true;
}

void showAndWait(u16 frames) {
  u8 doWait;

  renderer_fadeIn();
  doWait = wait(frames);
  renderer_fadeOut();

  renderer_clear();
  if (doWait) {
    wait(BLACK_WAIT);
  }
}
